@@ -49,6 +49,11 @@ static int checkSpec(rpmts ts, Header h)
     return rc;
 }
 
+/*
+ * Kurwa, durni ameryka?ce sobe zawsze my?l?, ?e ca?y ?wiat mwi po
+ * angielsku...
+ */
+/* XXX this is still a dumb test but at least it's i18n aware */
 /**
  */
 static int isSpecFile(const char * specfile)
@@ -61,7 +66,7 @@ static int isSpecFile(const char * specfile)
     int count;
     int checking;
 
-    fd = Fopen(specfile, "r.ufdio");
+    fd = Fopen(specfile, "r");
     if (fd == NULL || Ferror(fd)) {
 	rpmError(RPMERR_OPEN, _("Unable to open spec file %s: %s\n"),
 		specfile, Fstrerror(fd));
@@ -82,13 +87,8 @@ static int isSpecFile(const char * specfile)
 	    /*@switchbreak@*/ break;
 /*@-boundsread@*/
 	default:
-#if 0
 	    if (checking && !(isprint(*s) || isspace(*s))) return 0;
 	    /*@switchbreak@*/ break;
-#else
-	    if (checking && !(isprint(*s) || isspace(*s)) && *(unsigned char *)s < 32) return 0;
-	    /*@switchbreak@*/ break;
-#endif
 /*@=boundsread@*/
 	}
     }
@@ -105,23 +105,19 @@ static int buildForTarget(rpmts ts, const char * arg, BTA_t ba)
     const char * passPhrase = ba->passPhrase;
     const char * cookie = ba->cookie;
     int buildAmount = ba->buildAmount;
-    const char * buildRootURL = NULL;
     const char * specFile;
     const char * specURL;
     int specut;
     char buf[BUFSIZ];
     Spec spec = NULL;
+    int verify = 1;
+    int xx;
     int rc;
 
 #ifndef	DYING
     rpmSetTables(RPM_MACHTABLE_BUILDARCH, RPM_MACHTABLE_BUILDOS);
 #endif
 
-    /*@-branchstate@*/
-    if (ba->buildRootOverride)
-	buildRootURL = rpmGenPath(NULL, ba->buildRootOverride, NULL);
-    /*@=branchstate@*/
-
     /*@-compmempass@*/ /* FIX: static zcmds heartburn */
     if (ba->buildMode == 't') {
 	FILE *fp;
@@ -134,7 +130,7 @@ static int buildForTarget(rpmts ts, const char * arg, BTA_t ba)
 
 	specDir = rpmGetPath("%{_specdir}", NULL);
 
-	tmpSpecFile = rpmGetPath("%{_specdir}/", "rpm-spec.XXXXXX", NULL);
+	tmpSpecFile = (char *) rpmGetPath("%{_specdir}/", "rpm-spec.XXXXXX", NULL);
 #if defined(HAVE_MKSTEMP)
 	(void) close(mkstemp(tmpSpecFile));
 #else
@@ -144,7 +140,7 @@ static int buildForTarget(rpmts ts, const char * arg, BTA_t ba)
 	(void) isCompressed(arg, &res);
 
 	cmd = alloca(strlen(arg) + 50 + strlen(tmpSpecFile));
-	sprintf(cmd, "%s < '%s' | tar xOvf - Specfile 2>&1 > '%s'",
+	sprintf(cmd, "%s < %s | tar xOvf - Specfile 2>&1 > %s",
 			zcmds[res & 0x3], arg, tmpSpecFile);
 	if (!(fp = popen(cmd, "r"))) {
 	    rpmError(RPMERR_POPEN, _("Failed to open tar pipe: %m\n"));
@@ -156,7 +152,7 @@ static int buildForTarget(rpmts ts, const char * arg, BTA_t ba)
 	    /* Try again */
 	    (void) pclose(fp);
 
-	    sprintf(cmd, "%s < '%s' | tar xOvf - --wildcards \\*.spec 2>&1 > '%s'",
+	    sprintf(cmd, "%s < %s | tar xOvf - \\*.spec 2>&1 > %s",
 		    zcmds[res & 0x3], arg, tmpSpecFile);
 	    if (!(fp = popen(cmd, "r"))) {
 		rpmError(RPMERR_POPEN, _("Failed to open tar pipe: %m\n"));
@@ -206,10 +202,7 @@ static int buildForTarget(rpmts ts, const char * arg, BTA_t ba)
 	   directory for this run */
 
 	if (*arg != '/') {
-	    if (!getcwd(buf, BUFSIZ)) {
-		rpmError(RPMERR_STAT, "getcwd failed: %m\n");
-		return 1;
-	    }
+	    if (getcwd(buf, BUFSIZ) == NULL) strcpy(buf, ".");
 	    strcat(buf, "/");
 	    strcat(buf, arg);
 	} else 
@@ -228,11 +221,7 @@ static int buildForTarget(rpmts ts, const char * arg, BTA_t ba)
     specut = urlPath(specURL, &specFile);
     if (*specFile != '/') {
 	char *s = alloca(BUFSIZ);
-	if (!getcwd(s, BUFSIZ)) {
-	    rpmError(RPMERR_STAT, "getcwd failed: %m\n");
-	    rc = 1;
-	    goto exit;
-	}
+	if (getcwd(s, BUFSIZ) == NULL) strcpy(s, ".");
 	strcat(s, "/");
 	strcat(s, arg);
 	specURL = s;
@@ -264,8 +253,8 @@ static int buildForTarget(rpmts ts, const char * arg, BTA_t ba)
     /* Parse the spec file */
 #define	_anyarch(_f)	\
 (((_f)&(RPMBUILD_PREP|RPMBUILD_BUILD|RPMBUILD_INSTALL|RPMBUILD_PACKAGEBINARY)) == 0)
-    if (parseSpec(ts, specURL, ba->rootdir, buildRootURL, 0, passPhrase,
-		cookie, _anyarch(buildAmount), ba->force))
+    if (parseSpec(ts, specURL, ba->rootdir, 0, passPhrase,
+		cookie, _anyarch(buildAmount), ba->force, verify))
     {
 	rc = 1;
 	goto exit;
@@ -277,7 +266,7 @@ static int buildForTarget(rpmts ts, const char * arg, BTA_t ba)
     }
 
     /* Assemble source header from parsed components */
-    initSourceHeader(spec);
+    xx = initSourceHeader(spec, NULL);
 
     /* Check build prerequisites */
     if (!ba->noDeps && checkSpec(ts, spec->sourceHeader)) {
@@ -296,19 +285,20 @@ static int buildForTarget(rpmts ts, const char * arg, BTA_t ba)
 
 exit:
     spec = freeSpec(spec);
-    buildRootURL = _free(buildRootURL);
     return rc;
 }
 /*@=boundswrite@*/
 
 int build(rpmts ts, const char * arg, BTA_t ba, const char * rcfile)
 {
-    char *t, *te;
+    const char *t, *te;
     int rc = 0;
-    char * targets = ba->targets;
+    const char * targets = rpmcliTargets;
+    char *target;
 #define	buildCleanMask	(RPMBUILD_RMSOURCE|RPMBUILD_RMSPEC)
     int cleanFlags = ba->buildAmount & buildCleanMask;
     rpmVSFlags vsflags, ovsflags;
+    int nbuilds = 0;
 
     vsflags = rpmExpandNumeric("%{_vsflags_build}");
     if (ba->qva_flags & VERIFY_DIGEST)
@@ -321,6 +311,7 @@ int build(rpmts ts, const char * arg, BTA_t ba, const char * rcfile)
 
     if (targets == NULL) {
 	rc =  buildForTarget(ts, arg, ba);
+	nbuilds++;
 	goto exit;
     }
 
@@ -330,7 +321,7 @@ int build(rpmts ts, const char * arg, BTA_t ba, const char * rcfile)
 
     ba->buildAmount &= ~buildCleanMask;
     for (t = targets; *t != '\0'; t = te) {
-	char *target;
+	/* Parse out next target platform. */ 
 	if ((te = strchr(t, ',')) == NULL)
 	    te = t + strlen(t);
 	target = alloca(te-t+1);
@@ -341,23 +332,36 @@ int build(rpmts ts, const char * arg, BTA_t ba, const char * rcfile)
 	else	/* XXX Perform clean-up after last target build. */
 	    ba->buildAmount |= cleanFlags;
 
-	printf(_("Building for target %s\n"), target);
+	rpmMessage(RPMMESS_DEBUG, _("    target platform: %s\n"), target);
 
 	/* Read in configuration for target. */
-	rpmFreeMacros(NULL);
-	rpmFreeRpmrc();
-	(void) rpmReadConfigFiles(rcfile, target);
+	if (t != targets) {
+	    rpmFreeMacros(NULL);
+	    rpmFreeRpmrc();
+	    (void) rpmReadConfigFiles(rcfile, target);
+	}
 	rc = buildForTarget(ts, arg, ba);
+	nbuilds++;
 	if (rc)
 	    break;
     }
 
 exit:
-    vsflags = rpmtsSetVSFlags(ts, ovsflags);
     /* Restore original configuration. */
-    rpmFreeMacros(NULL);
-    rpmFreeRpmrc();
-    (void) rpmReadConfigFiles(rcfile, NULL);
+    if (nbuilds > 1) {
+	t = targets;
+	if ((te = strchr(t, ',')) == NULL)
+	    te = t + strlen(t);
+	target = alloca(te-t+1);
+	strncpy(target, t, (te-t));
+	target[te-t] = '\0';
+	if (*te != '\0')
+	    te++;
+	rpmFreeMacros(NULL);
+	rpmFreeRpmrc();
+	(void) rpmReadConfigFiles(rcfile, target);
+    }
+    vsflags = rpmtsSetVSFlags(ts, ovsflags);
 
     return rc;
 }
