@@ -42,25 +42,29 @@
 #include <string.h>
 #include <errno.h>
 #include <sys/types.h>
+#include <sys/ioctl.h>
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
 #endif
+#if defined(HAVE_SYS_TIME_H)
+#include <sys/time.h>
+#endif
 #ifdef HAVE_LIBZ
 #include <zlib.h>
 #endif
 
-#ifndef lint
-FILE_RCSID("@(#)$Id: compress.c,v 1.42 2005/03/06 05:58:22 christos Exp $")
-#endif
 
+#ifndef lint
+FILE_RCSID("@(#)$File: compress.c,v 1.50 2007/03/01 22:14:54 christos Exp $")
+#endif
 
 /*@-nullassign@*/
 /*@unchecked@*/ /*@observer@*/
 private struct {
-/*@observer@*/
+/*@observer@*/ /*@null@*/
 	const char *magic;
 	size_t maglen;
-/*@observer@*/
+/*@observer@*/ /*@null@*/
 	const char *const argv[3];
 	int silent;
 } compr[] = {
@@ -80,26 +84,27 @@ private struct {
 /*@=nullassign@*/
 
 /*@unchecked@*/
-private int ncompr = sizeof(compr) / sizeof(compr[0]);
+private size_t ncompr = sizeof(compr) / sizeof(compr[0]);
+
+#define NODATA ((size_t)~0)
 
 
 private ssize_t swrite(int fd, const void *buf, size_t n)
 	/*@*/;
-private ssize_t sread(int fd, /*@out@*/ void *buf, size_t n)
-	/*@modifies buf @*/;
 private size_t uncompressbuf(struct magic_set *ms, int fd, size_t method,
     const unsigned char *old, unsigned char **newch, size_t n)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies ms, *newch, fileSystem, internalState @*/;
 #ifdef HAVE_LIBZ
-private size_t uncompressgzipped(struct magic_set *ms, const unsigned char *old,
+private size_t
+uncompressgzipped(struct magic_set *ms, const unsigned char *old,
     unsigned char **newch, size_t n)
 	/*@modifies ms, *newch @*/;
 #endif
 
 protected int
-file_zmagic(struct magic_set *ms, int fd, const unsigned char *buf,
-    size_t nbytes)
+file_zmagic(struct magic_set *ms, int fd, const char *name,
+    const unsigned char *buf, size_t nbytes)
 {
 	unsigned char *newbuf = NULL;
 	size_t i, nsz;
@@ -113,14 +118,14 @@ file_zmagic(struct magic_set *ms, int fd, const unsigned char *buf,
 			continue;
 		if (memcmp(buf, compr[i].magic, compr[i].maglen) == 0 &&
 		    (nsz = uncompressbuf(ms, fd, i, buf, &newbuf,
-		    nbytes)) != 0) {
+		    nbytes)) != NODATA) {
 			ms->flags &= ~MAGIC_COMPRESS;
 			rv = -1;
-			if (file_buffer(ms, -1, newbuf, nsz) == -1)
+			if (file_buffer(ms, -1, name, newbuf, nsz) == -1)
 				goto error;
 			if (file_printf(ms, " (") == -1)
 				goto error;
-			if (file_buffer(ms, -1, buf, nbytes) == -1)
+			if (file_buffer(ms, -1, NULL, buf, nbytes) == -1)
 				goto error;
 			if (file_printf(ms, ")") == -1)
 				goto error;
@@ -163,14 +168,55 @@ swrite(int fd, const void *buf, size_t n)
 /*
  * `safe' read for sockets and pipes.
  */
-private ssize_t
-sread(int fd, void *buf, size_t n)
+protected ssize_t
+sread(int fd, void *buf, size_t n, int canbepipe)
 {
-	int rv;
+	int rv, cnt;
+#ifdef FIONREAD
+	int t = 0;
+#endif
 	size_t rn = n;
 
+	if (fd == STDIN_FILENO)
+		goto nocheck;
+
+#ifdef FIONREAD
+	if (canbepipe && (ioctl(fd, FIONREAD, &t) == -1) || (t == 0)) {
+#ifdef FD_ZERO
+		for (cnt = 0;; cnt++) {
+			fd_set check;
+			struct timeval tout = {0, 100 * 1000};
+			int rv;
+
+			FD_ZERO(&check);
+			FD_SET(fd, &check);
+
+			/*
+			 * Avoid soft deadlock: do not read if there
+			 * is nothing to read from sockets and pipes.
+			 */
+			rv = select(fd + 1, &check, NULL, NULL, &tout);
+			if (rv == -1) {
+				if (errno == EINTR || errno == EAGAIN)
+					continue;
+			} else if (rv == 0 && cnt >= 5) {
+				return 0;
+			} else
+				break;
+		}
+#endif
+		(void)ioctl(fd, FIONREAD, &t);
+	}
+
+	if (t > 0 && (size_t)t < n) {
+		n = t;
+		rn = n;
+	}
+#endif
+
+nocheck:
 	do
-		switch (rv = read(fd, buf, n)) {
+		switch ((rv = read(fd, buf, n))) {
 		case -1:
 			if (errno == EINTR)
 				continue;
@@ -217,7 +263,7 @@ file_pipe2file(struct magic_set *ms, int fd, const void *startbuf,
 	if (swrite(tfd, startbuf, nbytes) != (ssize_t)nbytes)
 		r = 1;
 	else {
-		while ((r = sread(fd, buf, sizeof(buf))) > 0)
+		while ((r = sread(fd, buf, sizeof(buf), 1)) > 0)
 			if (swrite(tfd, buf, (size_t)r) != r)
 				break;
 	}
@@ -313,15 +359,16 @@ uncompressgzipped(struct magic_set *ms, const unsigned char *old,
 	}
 
 	n = (size_t)z.total_out;
-	inflateEnd(&z);
+	(void)inflateEnd(&z);
 	
 	/* let's keep the nul-terminate tradition */
-	(*newch)[n++] = '\0';
+	(*newch)[n] = '\0';
 
 	return n;
 }
 #endif
 
+/*@mayexit@*/
 private size_t
 uncompressbuf(struct magic_set *ms, int fd, size_t method,
     const unsigned char *old, unsigned char **newch, size_t n)
@@ -338,7 +385,7 @@ uncompressbuf(struct magic_set *ms, int fd, size_t method,
 
 	if ((fd != -1 && pipe(fdin) == -1) || pipe(fdout) == -1) {
 		file_error(ms, errno, "cannot create pipe");	
-		return 0;
+		return NODATA;
 	}
 	switch (fork()) {
 	case 0:	/* child */
@@ -368,10 +415,11 @@ uncompressbuf(struct magic_set *ms, int fd, size_t method,
 		    compr[method].argv[0], strerror(errno));
 #endif
 		exit(EXIT_FAILURE);
-		/*@notreached@*/ break;
+		/*@notreached@*/
+		break;
 	case -1:
 		file_error(ms, errno, "could not fork");
-		return 0;
+		return NODATA;
 
 	default: /* parent */
 		(void) close(fdout[1]);
@@ -384,17 +432,17 @@ uncompressbuf(struct magic_set *ms, int fd, size_t method,
 			switch (fork()) {
 			case 0: /* child */
 				(void)close(fdout[0]);
-				if (swrite(fdin[1], old, n) != n) {
+				if (swrite(fdin[1], old, n) != (ssize_t)n) {
 #ifdef DEBUG
 					(void)fprintf(stderr,
 					    "Write failed (%s)\n",
 					    strerror(errno));
 #endif
 					exit(EXIT_FAILURE);
-					/*@notreached@*/ /*@innerbreak@*/ break;
 				}
 				exit(EXIT_SUCCESS);
-				/*@notreached@*/ /*@innerbreak@*/ break;
+				/*@notreached@**/
+				/*@innerbreak@*/ break;
 
 			case -1:
 #ifdef DEBUG
@@ -402,7 +450,8 @@ uncompressbuf(struct magic_set *ms, int fd, size_t method,
 				    strerror(errno));
 #endif
 				exit(EXIT_FAILURE);
-				/*@notreached@*/ /*@innerbreak@*/ break;
+				/*@notreached@**/
+				/*@innerbreak@*/ break;
 
 			default:  /* parent */
 				/*@innerbreak@*/ break;
@@ -419,22 +468,20 @@ uncompressbuf(struct magic_set *ms, int fd, size_t method,
 			n = 0;
 			goto err;
 		}
-		if ((r = sread(fdout[0], *newch, HOWMANY)) <= 0) {
+		if ((r = sread(fdout[0], *newch, HOWMANY, 0)) <= 0) {
 #ifdef DEBUG
 			(void)fprintf(stderr, "Read failed (%s)\n",
 			    strerror(errno));
 #endif
 			free(*newch);
 			n = 0;
-/*@-type@*/
 			newch[0] = '\0';
-/*@=type@*/
 			goto err;
 		} else {
 			n = r;
 		}
  		/* NUL terminate, as every buffer is handled here. */
- 		(*newch)[n++] = '\0';
+ 		(*newch)[n] = '\0';
 err:
 		if (fdin[1] != -1)
 			(void) close(fdin[1]);
