@@ -4,12 +4,17 @@
 
 #include "system.h"
 
+#include "rpmio_internal.h"	/* XXX PGPHASHALGO_MD5 */
 #include <rpmcli.h>		/* XXX rpmcliPackagesTotal */
 
 #include <rpmmacro.h>		/* XXX rpmExpand("%{_dependency_whiteout}" */
+#include <envvar.h>
+#include <ugid.h>		/* XXX user()/group() probes */
 
-#include "rpmdb.h"		/* XXX response cache needs dbiOpen et al. */
+#define	_RPMDB_INTERNAL		/* XXX response cache needs dbiOpen et al. */
+#include "rpmdb.h"
 
+#define	_RPMEVR_INTERNAL
 #include "rpmds.h"
 #include "rpmfi.h"
 
@@ -24,6 +29,7 @@
 /*@access tsortInfo @*/
 /*@access rpmte @*/		/* XXX for install <-> erase associate. */
 /*@access rpmts @*/
+/*@access rpmDiskSpaceInfo @*/
 
 /*@access alKey @*/	/* XXX for reordering and RPMAL_NOMATCH assign */
 
@@ -145,6 +151,8 @@ static int rpmHeadersIdentical(Header first, Header second)
 	/*@*/
 {
     const char * one, * two;
+    rpmds A, B;
+    int rc;
 
     if (!headerGetEntry(first, RPMTAG_HDRID, NULL, (void **) &one, NULL))
 	one = NULL;
@@ -158,12 +166,18 @@ static int rpmHeadersIdentical(Header first, Header second)
     if (!one && two)
 	return 0;
     /* XXX Headers w/o digests case devolves to NEVR comparison. */
-    return ((rpmVersionCompare(first, second) == 0) ? 1 : 0);
+    A = rpmdsThis(first, RPMTAG_REQUIRENAME, RPMSENSE_EQUAL);
+    B = rpmdsThis(second, RPMTAG_REQUIRENAME, RPMSENSE_EQUAL);
+    rc = rpmdsCompare(A, B);
+    A = rpmdsFree(A);
+    B = rpmdsFree(B);
+    return rc;
 }
 
 int rpmtsAddInstallElement(rpmts ts, Header h,
 			fnpyKey key, int upgrade, rpmRelocation relocs)
 {
+    rpmdepFlags depFlags = rpmtsDFlags(ts);
     uint_32 tscolor = rpmtsColor(ts);
     uint_32 dscolor;
     uint_32 hcolor;
@@ -184,34 +198,70 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
     int rc;
     int oc;
 
-    /*
-     * Check for previously added versions with the same name and arch/os.
-     */
-    arch = NULL;
-    xx = hge(h, RPMTAG_ARCH, NULL, (void **)&arch, NULL);
-    os = NULL;
-    xx = hge(h, RPMTAG_OS, NULL, (void **)&os, NULL);
     hcolor = hGetColor(h);
     pkgKey = RPMAL_NOMATCH;
 
-    /* XXX Always add source headers. */
+    /*
+     * Always add source headers.
+     */
     isSource = (headerIsEntry(h, RPMTAG_SOURCERPM) == 0) ;
     if (isSource) {
 	oc = ts->orderCount;
 	goto addheader;
     }
 
+    /*
+     * Check platform affinity of binary packages.
+     */
+    arch = NULL;
+    xx = hge(h, RPMTAG_ARCH, NULL, (void **)&arch, NULL);
+    os = NULL;
+    xx = hge(h, RPMTAG_OS, NULL, (void **)&os, NULL);
+    if (nplatpat > 1) {
+	const char * platform = NULL;
+
+	if (hge(h, RPMTAG_PLATFORM, NULL, (void **)&platform, NULL))
+	    platform = xstrdup(platform);
+	else
+	    platform = rpmExpand(arch, "-unknown-", os, NULL);
+
+	rc = rpmPlatformScore(platform, platpat, nplatpat);
+	if (rc <= 0) {
+	    const char * pkgNEVR = hGetNEVRA(h, NULL);
+	    rpmps ps = rpmtsProblems(ts);
+	    rpmpsAppend(ps, RPMPROB_BADPLATFORM, pkgNEVR, key,
+                        platform, NULL, NULL, 0);
+	    ps = rpmpsFree(ps);
+	    pkgNEVR = _free(pkgNEVR);
+	    ec = 1;
+	}
+	platform = _free(platform);
+	if (ec)
+	    goto exit;
+    }
+
+    /*
+     * Always install compatible binary packages.
+     */
+    if (!upgrade) {
+	oc = ts->orderCount;
+	goto addheader;
+    }
+
+    /*
+     * Check that upgrade package is uniquely newer, replace older if necessary.
+     */
     oldChk = rpmdsThis(h, RPMTAG_REQUIRENAME, (RPMSENSE_LESS));
     newChk = rpmdsThis(h, RPMTAG_REQUIRENAME, (RPMSENSE_EQUAL|RPMSENSE_GREATER));
     /* XXX can't use rpmtsiNext() filter or oc will have wrong value. */
     for (pi = rpmtsiInit(ts), oc = 0; (p = rpmtsiNext(pi, 0)) != NULL; oc++) {
 	rpmds this;
 
-	/* XXX Only added packages need be checked for dupes. */
+	/* XXX Only added packages need be checked for dupes here. */
 	if (rpmteType(p) == TR_REMOVED)
 	    continue;
 
-	/* XXX Never check source headers. */
+	/* XXX Never check source header NEVRAO. */
 	if (rpmteIsSource(p))
 	    continue;
 
@@ -221,9 +271,17 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
 
 	    if (arch == NULL || (parch = rpmteA(p)) == NULL)
 		continue;
+	    /* XXX hackery for i[3456]86 alias matching. */
+	    if (arch[0] == 'i' && arch[2] == '8' && arch[3] == '6') {
+		if (arch[0] != parch[0]) continue;
+		if (arch[2] != parch[2]) continue;
+		if (arch[3] != parch[3]) continue;
+	    } else if (strcmp(arch, parch))
+		continue;
 	    if (os == NULL || (pos = rpmteO(p)) == NULL)
 		continue;
-	    if (strcmp(arch, parch) || strcmp(os, pos))
+
+	    if (strcmp(os, pos))
 		continue;
 	}
 
@@ -231,7 +289,7 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
 	if ((this = rpmteDS(p, RPMTAG_NAME)) == NULL)
 	    continue;	/* XXX can't happen */
 
-	/* If newer NEVR was previously added, then skip adding older. */
+	/* If newer NEVRAO already added, then skip adding older. */
 	rc = rpmdsCompare(newChk, this);
 	if (rc != 0) {
 	    const char * pkgNEVR = rpmdsDNEVR(this);
@@ -245,7 +303,7 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
 	    break;
 	}
 
-	/* If older NEVR was previously added, then replace old with new. */
+	/* If older NEVRAO already added, then replace old with new. */
 	rc = rpmdsCompare(oldChk, this);
 	if (rc != 0) {
 	    const char * pkgNEVR = rpmdsDNEVR(this);
@@ -264,7 +322,7 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
     oldChk = rpmdsFree(oldChk);
     newChk = rpmdsFree(newChk);
 
-    /* If newer NEVR was already added, exit now. */
+    /* If newer (or same) NEVRAO was already added, exit now. */
     if (ec)
 	goto exit;
 
@@ -334,6 +392,7 @@ assert(p != NULL);
 
     /* On upgrade, erase older packages of same color (if any). */
 
+  if (!(depFlags & RPMDEPS_FLAG_NOUPGRADE)) {
     mi = rpmtsInitIterator(ts, RPMTAG_PROVIDENAME, rpmteN(p), 0);
     while((oh = rpmdbNextIterator(mi)) != NULL) {
 	int lastx;
@@ -363,7 +422,9 @@ assert(lastx >= 0 && lastx < ts->orderCount);
 
     }
     mi = rpmdbFreeIterator(mi);
+  }
 
+  if (!(depFlags & RPMDEPS_FLAG_NOOBSOLETES)) {
     obsoletes = rpmdsLink(rpmteDS(p, RPMTAG_OBSOLETENAME), "Obsoletes");
     obsoletes = rpmdsInit(obsoletes);
     if (obsoletes != NULL)
@@ -433,6 +494,7 @@ assert(lastx >= 0 && lastx < ts->orderCount);
 	mi = rpmdbFreeIterator(mi);
     }
     obsoletes = rpmdsFree(obsoletes);
+  }
 
     ec = 0;
 
@@ -468,15 +530,19 @@ static int unsatisfiedDepend(rpmts ts, rpmds dep, int adding)
     DBT * key = alloca(sizeof(*key));
     DBT * data = alloca(sizeof(*data));
     rpmdbMatchIterator mi;
+    nsType NSType;
     const char * Name;
+    int_32 Flags;
     Header h;
     int _cacheThisRC = 1;
     int rc;
     int xx;
-    int retrying = 0;
+    int retries = 10;
 
     if ((Name = rpmdsN(dep)) == NULL)
 	return 0;	/* XXX can't happen */
+    Flags = rpmdsFlags(dep);
+    NSType = rpmdsNSType(dep);
 
     /*
      * Check if dbiOpen/dbiPut failed (e.g. permissions), we can't cache.
@@ -523,7 +589,7 @@ static int unsatisfiedDepend(rpmts ts, rpmds dep, int adding)
 
 	    if (rc >= 0) {
 		rpmdsNotify(dep, _("(cached)"), rc);
-		return rc;
+		return rpmdsNegateRC(dep, rc);
 	    }
 	}
     }
@@ -532,47 +598,259 @@ retry:
     rc = 0;	/* assume dependency is satisfied */
 
     /* Expand macro probe dependencies. */
-    if (Name[0] == '%' && Name[1] == '{' && Name[strlen(Name)-1] == '}') {
+    if (NSType == RPMNS_TYPE_FUNCTION) {
 	xx = rpmExpandNumeric(Name);
-	/* XXX how should macro values be mapped? */
 	rc = (xx ? 0 : 1);
-	if (rpmdsFlags(dep) & RPMSENSE_MISSINGOK)
+	if (Flags & RPMSENSE_MISSINGOK)
 	    goto unsatisfied;
-	rpmdsNotify(dep, _("(macro probe)"), rc);
+	rpmdsNotify(dep, _("(function probe)"), rc);
+	goto exit;
+    }
+
+    /* Evaluate user/group lookup probes. */
+    if (NSType == RPMNS_TYPE_USER) {
+	const char *s;
+	uid_t uid = 0;
+	s = Name; while (*s && xisdigit(*s)) s++;
+
+	if (*s)
+	    xx = unameToUid(Name, &uid);
+	else {
+	    uid = strtol(Name, NULL, 10);
+	    xx = (uidToUname(uid) ? 0 : -1);
+	}
+	rc = (xx >= 0 ? 0 : 1);
+	if (Flags & RPMSENSE_MISSINGOK)
+	    goto unsatisfied;
+	rpmdsNotify(dep, _("(user lookup)"), rc);
+	goto exit;
+    }
+    if (NSType == RPMNS_TYPE_GROUP) {
+	const char *s;
+	gid_t gid = 0;
+	s = Name; while (*s && xisdigit(*s)) s++;
+
+	if (*s)
+	    xx = gnameToGid(Name, &gid);
+	else {
+	    gid = strtol(Name, NULL, 10);
+	    xx = (gidToGname(gid) ? 0 : -1);
+	}
+	rc = (xx >= 0 ? 0 : 1);
+	if (Flags & RPMSENSE_MISSINGOK)
+	    goto unsatisfied;
+	rpmdsNotify(dep, _("(group lookup)"), rc);
 	goto exit;
     }
 
     /* Evaluate access(2) probe dependencies. */
-    if (strlen(Name) > 5 && Name[strlen(Name)-1] == ')'
-     && (	(  strchr("Rr_", Name[0]) != NULL
-		&& strchr("Ww_", Name[1]) != NULL
-		&& strchr("Xx_", Name[2]) != NULL
-		&& Name[3] == '(')
-	  ||	!strncmp(Name, "exists(", sizeof("exists(")-1)
-	  ||	!strncmp(Name, "executable(", sizeof("executable(")-1)
-	  ||	!strncmp(Name, "readable(", sizeof("readable(")-1)
-	  ||	!strncmp(Name, "writable(", sizeof("writable(")-1)
-	))
-    {
+    if (NSType == RPMNS_TYPE_ACCESS) {
 	rc = rpmioAccess(Name, NULL, X_OK);
-	if (rpmdsFlags(dep) & RPMSENSE_MISSINGOK)
+	if (Flags & RPMSENSE_MISSINGOK)
 	    goto unsatisfied;
 	rpmdsNotify(dep, _("(access probe)"), rc);
 	goto exit;
     }
 
+    /* Evaluate mtab lookup and diskspace probe dependencies. */
+    if (NSType == RPMNS_TYPE_MOUNTED) {
+	const char ** fs = NULL;
+	int nfs = 0;
+	int i = 0;
+
+	xx = rpmtsInitDSI(ts);
+	fs = ts->filesystems;
+	nfs = ts->filesystemCount;
+
+	if (fs != NULL)
+	for (i = 0; i < nfs; i++) {
+	    if (!strcmp(fs[i], Name))
+		break;
+	}
+	rc = (i < nfs ? 0 : 1);
+	if (Flags & RPMSENSE_MISSINGOK)
+	    goto unsatisfied;
+	rpmdsNotify(dep, _("(mtab probe)"), rc);
+	goto exit;
+    }
+
+    if (NSType == RPMNS_TYPE_DISKSPACE) {
+	size_t nb = strlen(Name);
+	rpmDiskSpaceInfo dsi = NULL;
+	const char ** fs = NULL;
+	size_t fslen = 0, longest = 0;
+	int nfs = 0;
+	int i = 0;
+
+	xx = rpmtsInitDSI(ts);
+	fs = ts->filesystems;
+	nfs = ts->filesystemCount;
+
+	if (fs != NULL)
+	for (i = 0; i < nfs; i++) {
+	    fslen = strlen(fs[i]);
+	    if (fslen > nb)
+		continue;
+	    if (strncmp(fs[i], Name, fslen))
+		continue;
+	    if (fslen > 1 && Name[fslen] != '/' && Name[fslen] != '\0')
+		continue;
+	    if (fslen < longest)
+		continue;
+	    longest = fslen;
+	    dsi = ts->dsi + i;
+	}
+	if (dsi == NULL)
+	    rc = 1;	/* no mounted paths !?! */
+	else {
+	    char * end = NULL;
+	    long long needed = strtoll(rpmdsEVR(dep), &end, 0);
+
+	    if (end && *end) {
+		if (strchr("Gg", end[0]) && strchr("Bb", end[1]) && !end[2])
+		    needed *= 1024 * 1024 * 1024;
+		if (strchr("Mm", end[0]) && strchr("Bb", end[1]) && !end[2])
+		    needed *= 1024 * 1024;
+		if (strchr("Kk", end[0]) && strchr("Bb", end[1]) && !end[2])
+		    needed *= 1024;
+	    } else
+		needed *= 1024 * 1024;	/* XXX assume Mb if no units given */
+
+	    needed = BLOCK_ROUND(needed, dsi->f_bsize);
+	    xx = (dsi->f_bavail - needed);
+	    if ((Flags & RPMSENSE_LESS) && xx < 0) rc = 0;
+	    else if ((Flags & RPMSENSE_GREATER) && xx > 0) rc = 0;
+	    else if ((Flags & RPMSENSE_EQUAL) && xx == 0) rc = 0;
+	    else rc = 1;
+	}
+	if (Flags & RPMSENSE_MISSINGOK)
+	    goto unsatisfied;
+	rpmdsNotify(dep, _("(diskspace probe)"), rc);
+	goto exit;
+    }
+
+    if (NSType == RPMNS_TYPE_DIGEST) {
+	const char * EVR = rpmdsEVR(dep);
+	FD_t fd = Fopen(Name, "r");
+
+	rc = 1;		/* XXX assume failure */
+	if (fd && !Ferror(fd)) {
+	    pgpHashAlgo digestHashAlgo = PGPHASHALGO_MD5;
+	    DIGEST_CTX ctx = rpmDigestInit(digestHashAlgo, RPMDIGEST_NONE);
+	    const char * digest = NULL;
+	    size_t digestlen = 0;
+	    int asAscii = 1;
+	    size_t nbuf = 8 * BUFSIZ;
+	    char * buf = alloca(nbuf);
+	    size_t nb;
+
+	    while ((nb = Fread(buf, sizeof(buf[0]), nbuf, fd)) > 0)
+		xx = rpmDigestUpdate(ctx, buf, nb);
+	    xx = Fclose(fd);	fd = NULL;
+	    xx = rpmDigestFinal(ctx, (void **)&digest, &digestlen, asAscii);
+
+	    xx = (EVR && *EVR && digest && *digest) ? strcmp(EVR, digest) : -1;
+	    /* XXX only equality makes sense for digest compares */
+	    if ((Flags & RPMSENSE_EQUAL) && xx == 0) rc = 0;
+	}
+	if (Flags & RPMSENSE_MISSINGOK)
+	    goto unsatisfied;
+	rpmdsNotify(dep, _("(digest probe)"), rc);
+	goto exit;
+    }
+
+    if (NSType == RPMNS_TYPE_GNUPG) {
+	static const char gnupg_pre[] = "%(%{__gpg} -qv ";
+	static const char gnupg_post[] = " 2>/dev/null; echo $?)";
+	const char * t = rpmExpand(gnupg_pre, Name, gnupg_post, NULL);
+
+	rc = (t && t[0] == '0') ? 0 : 1;
+	t = _free(t);
+	if (Flags & RPMSENSE_MISSINGOK)
+	    goto unsatisfied;
+	rpmdsNotify(dep, _("(gnupg probe)"), rc);
+	goto exit;
+    }
+
+    if (NSType == RPMNS_TYPE_MACRO) {
+	static const char macro_pre[] = "%{?";
+	static const char macro_post[] = ":0}";
+	const char * a = rpmExpand(macro_pre, Name, macro_post, NULL);
+
+	rc = (a && a[0] == '0') ? 0 : 1;
+	a = _free(a);
+	if (Flags & RPMSENSE_MISSINGOK)
+	    goto unsatisfied;
+	rpmdsNotify(dep, _("(macro probe)"), rc);
+	goto exit;
+    }
+
+    if (NSType == RPMNS_TYPE_ENVVAR) {
+	const char * a = envGet(Name);
+	const char * b = rpmdsEVR(dep);
+
+	/* Existence test if EVR is missing/empty. */
+	if (!(b && *b))
+	    rc = (!(a && *a));
+	else {
+	    int sense = (a && *a) ? strcmp(a, b) : -1;
+
+	    if ((Flags & RPMSENSE_SENSEMASK) == RPMSENSE_NOTEQUAL)
+		rc = (sense == 0);
+	    else if (sense < 0 && (Flags & RPMSENSE_LESS))
+		rc = 0;
+	    else if (sense > 0 && (Flags & RPMSENSE_GREATER))
+		rc = 0;
+	    else if (sense == 0 && (Flags & RPMSENSE_EQUAL))
+		rc = 0;
+	    else
+		rc = (sense != 0);
+	}
+
+	if (Flags & RPMSENSE_MISSINGOK)
+	    goto unsatisfied;
+	rpmdsNotify(dep, _("(envvar probe)"), rc);
+	goto exit;
+    }
+
+    if (NSType == RPMNS_TYPE_RUNNING) {
+	char *t = NULL;
+	pid_t pid = strtol(Name, &t, 10);
+
+	if (t == NULL || *t != '\0') {
+	    const char * fn = rpmGetPath("%{_varrun}/", Name, ".pid", NULL);
+	    FD_t fd = NULL;
+
+	    if (fn && *fn != '%' && (fd = Fopen(fn, "r")) && !Ferror(fd)) {
+		char buf[32];
+		size_t nb = Fread(buf, sizeof(buf[0]), sizeof(buf), fd);
+
+		if (nb > 0)
+		    pid = strtol(buf, &t, 10);
+	    } else
+		pid = 0;
+	    if (fd != NULL)
+		(void) Fclose(fd);
+	    fn = _free(fn);
+	}
+	rc = (pid > 0 ? (kill(pid, 0) < 0 && errno == ESRCH) : 1);
+	if (Flags & RPMSENSE_MISSINGOK)
+	    goto unsatisfied;
+	rpmdsNotify(dep, _("(running probe)"), rc);
+	goto exit;
+    }
+
     /* Search system configured provides. */
     if (!rpmioAccess("/etc/rpm/sysinfo", NULL, R_OK)) {
-	static rpmds sysinfoP = NULL;
-	static int oneshot = -1;
-
-	if (oneshot)
-	    oneshot = rpmdsSysinfo(&sysinfoP, NULL);
-	if (sysinfoP != NULL) {
-	    if (rpmdsSearch(sysinfoP, dep) >= 0) {
-		rpmdsNotify(dep, _("(sysinfo provides)"), rc);
-		goto exit;
-	    }
+#ifdef	NOTYET	/* XXX just sysinfo Provides: for now. */
+	rpmTag tagN = (Name[0] == '/' ? RPMTAG_DIRNAMES : RPMTAG_PROVIDENAME);
+#else
+	rpmTag tagN = RPMTAG_PROVIDENAME;
+#endif
+	rpmds P = rpmdsFromPRCO(ts->PRCO, tagN);
+	if (rpmdsSearch(P, dep) >= 0) {
+	    rpmdsNotify(dep, _("(sysinfo provides)"), rc);
+	    goto exit;
 	}
     }
 
@@ -581,7 +859,7 @@ retry:
      * on rpmlib provides. The dependencies look like "rpmlib(YaddaYadda)".
      * Check those dependencies now.
      */
-    if (!strncmp(Name, "rpmlib(", sizeof("rpmlib(")-1)) {
+    if (NSType == RPMNS_TYPE_RPMLIB) {
 	static rpmds rpmlibP = NULL;
 	static int oneshot = -1;
 
@@ -597,7 +875,7 @@ retry:
 	goto unsatisfied;
     }
 
-    if (!strncmp(Name, "cpuinfo(", sizeof("cpuinfo(")-1)) {
+    if (NSType == RPMNS_TYPE_CPUINFO) {
 	static rpmds cpuinfoP = NULL;
 	static int oneshot = -1;
 
@@ -613,7 +891,7 @@ retry:
 	goto unsatisfied;
     }
 
-    if (!strncmp(Name, "getconf(", sizeof("getconf(")-1)) {
+    if (NSType == RPMNS_TYPE_GETCONF) {
 	static rpmds getconfP = NULL;
 	static int oneshot = -1;
 
@@ -629,7 +907,7 @@ retry:
 	goto unsatisfied;
     }
 
-    if (!strncmp(Name, "uname(", sizeof("uname(")-1)) {
+    if (NSType == RPMNS_TYPE_UNAME) {
 	static rpmds unameP = NULL;
 	static int oneshot = -1;
 
@@ -645,32 +923,29 @@ retry:
 	goto unsatisfied;
     }
 
-    if (!strncmp(Name, "soname(", sizeof("soname(")-1)) {
+    if (NSType == RPMNS_TYPE_SONAME) {
 	rpmds sonameP = NULL;
-	rpmPRCO PRCO = memset(alloca(sizeof(*PRCO)), 0, sizeof(*PRCO));
+	rpmPRCO PRCO = rpmdsNewPRCO(NULL);
 	char * fn = strcpy(alloca(strlen(Name)+1), Name);
 	int flags = 0;	/* XXX RPMELF_FLAG_SKIPREQUIRES? */
 	rpmds ds;
 
-	/* Strip the soname(/path/to/dso) wrapper. */
-	fn += sizeof("soname(")-1;
-
 	/* XXX Only absolute paths for now. */
 	if (*fn != '/')
 	    goto unsatisfied;
 	fn[strlen(fn)-1] = '\0';
 
 	/* Extract ELF Provides: from /path/to/DSO. */
-	PRCO->Pdsp = &sonameP;
 	xx = rpmdsELF(fn, flags, rpmdsMergePRCO, PRCO);
+	sonameP = rpmdsFromPRCO(PRCO, RPMTAG_PROVIDENAME);
 	if (!(xx == 0 && sonameP != NULL))
 	    goto unsatisfied;
 
 	/* Search using the original {EVR,"",Flags} from the dep set. */
-	ds = rpmdsSingle(rpmdsTagN(dep), rpmdsEVR(dep), "", rpmdsFlags(dep));
+	ds = rpmdsSingle(rpmdsTagN(dep), rpmdsEVR(dep), "", Flags);
 	xx = rpmdsSearch(sonameP, ds);
 	ds = rpmdsFree(ds);
-	sonameP = rpmdsFree(sonameP);
+	PRCO = rpmdsFreePRCO(PRCO);
 
 	/* Was the dependency satisfied? */
 	if (xx >= 0) {
@@ -698,10 +973,8 @@ retry:
 	    /* depFlags better be 0! */
 
 	    mi = rpmtsInitIterator(ts, RPMTAG_BASENAMES, Name, 0);
-
 	    (void) rpmdbPruneIterator(mi,
 			ts->removedPackages, ts->numRemovedPackages, 1);
-
 	    while ((h = rpmdbNextIterator(mi)) != NULL) {
 		rpmdsNotify(dep, _("(db files)"), rc);
 		mi = rpmdbFreeIterator(mi);
@@ -729,13 +1002,13 @@ retry:
      * Search for an unsatisfied dependency.
      */
 /*@-boundsread@*/
-    if (adding && !retrying && !(rpmtsFlags(ts) & RPMTRANS_FLAG_NOSUGGEST)) {
+    if (adding == 1 && retries > 0 && !(rpmtsDFlags(ts) & RPMDEPS_FLAG_NOSUGGEST)) {
 	if (ts->solve != NULL) {
 	    xx = (*ts->solve) (ts, dep, ts->solveData);
 	    if (xx == 0)
 		goto exit;
 	    if (xx == -1) {
-		retrying = 1;
+		retries--;
 		rpmalMakeIndex(ts->addedPackages);
 		goto retry;
 	    }
@@ -744,7 +1017,7 @@ retry:
 /*@=boundsread@*/
 
 unsatisfied:
-    if (rpmdsFlags(dep) & RPMSENSE_MISSINGOK) {
+    if (Flags & RPMSENSE_MISSINGOK) {
 	rc = 0;	/* dependency is unsatisfied, but just a hint. */
 	rpmdsNotify(dep, _("(hint skipped)"), rc);
     } else {
@@ -788,7 +1061,8 @@ exit:
 		_cacheDependsRC = 0;
 	}
     }
-    return rc;
+
+    return rpmdsNegateRC(dep, rc);
 }
 
 /**
@@ -798,6 +1072,7 @@ exit:
  * @param requires	Requires: dependencies (or NULL)
  * @param conflicts	Conflicts: dependencies (or NULL)
  * @param dirnames	Dirnames: dependencies (or NULL)
+ * @param linktos	Filelinktos: dependencies (or NULL)
  * @param depName	dependency name to filter (or NULL)
  * @param tscolor	color bits for transaction set (0 disables)
  * @param adding	dependency is from added package set?
@@ -807,12 +1082,14 @@ static int checkPackageDeps(rpmts ts, const char * pkgNEVRA,
 		/*@null@*/ rpmds requires,
 		/*@null@*/ rpmds conflicts,
 		/*@null@*/ rpmds dirnames,
+		/*@null@*/ rpmds linktos,
 		/*@null@*/ const char * depName, uint_32 tscolor, int adding)
 	/*@globals rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState @*/
-	/*@modifies ts, requires, conflicts, rpmGlobalMacroContext,
-		fileSystem, internalState */
+	/*@modifies ts, requires, conflicts, dirnames, linktos,
+		rpmGlobalMacroContext, fileSystem, internalState */
 {
+    rpmps ps = rpmtsProblems(ts);
     uint_32 dscolor;
     const char * Name;
     int rc;
@@ -849,7 +1126,7 @@ static int checkPackageDeps(rpmts ts, const char * pkgNEVRA,
 	    }
 	    /*@=branchstate@*/
 
-	    rpmdsProblem(ts->probs, pkgNEVRA, requires, suggestedKeys, adding);
+	    rpmdsProblem(ps, pkgNEVRA, requires, suggestedKeys, adding);
 
 	}
 	    /*@switchbreak@*/ break;
@@ -881,7 +1158,7 @@ static int checkPackageDeps(rpmts ts, const char * pkgNEVRA,
 	/* 1 == unsatisfied, 0 == satsisfied */
 	switch (rc) {
 	case 0:		/* conflicts exist. */
-	    rpmdsProblem(ts->probs, pkgNEVRA, conflicts, NULL, adding);
+	    rpmdsProblem(ps, pkgNEVRA, conflicts, NULL, adding);
 	    /*@switchbreak@*/ break;
 	case 1:		/* conflicts don't exist. */
 	    /*@switchbreak@*/ break;
@@ -923,7 +1200,7 @@ static int checkPackageDeps(rpmts ts, const char * pkgNEVRA,
 	    }
 	    /*@=branchstate@*/
 
-	    rpmdsProblem(ts->probs, pkgNEVRA, dirnames, suggestedKeys, adding);
+	    rpmdsProblem(ps, pkgNEVRA, dirnames, suggestedKeys, adding);
 
 	}
 	    /*@switchbreak@*/ break;
@@ -934,6 +1211,51 @@ static int checkPackageDeps(rpmts ts, const char * pkgNEVRA,
 	}
     }
 
+    linktos = rpmdsInit(linktos);
+    if (linktos != NULL)
+    while (!ourrc && rpmdsNext(linktos) >= 0) {
+
+	if ((Name = rpmdsN(linktos)) == NULL)
+	    continue;	/* XXX can't happen */
+	if (*Name == '\0')	/* XXX most linktos are empty */
+		continue;
+
+	/* Filter out linktos that came along for the ride. */
+	if (depName != NULL && strcmp(depName, Name))
+	    continue;
+
+	/* Ignore colored linktos not in our rainbow. */
+	dscolor = rpmdsColor(linktos);
+	if (tscolor && dscolor && !(tscolor & dscolor))
+	    continue;
+
+	rc = unsatisfiedDepend(ts, linktos, adding);
+
+	switch (rc) {
+	case 0:		/* requirements are satisfied. */
+	    /*@switchbreak@*/ break;
+	case 1:		/* requirements are not satisfied. */
+	{   fnpyKey * suggestedKeys = NULL;
+
+	    /*@-branchstate@*/
+	    if (ts->availablePackages != NULL) {
+		suggestedKeys = rpmalAllSatisfiesDepend(ts->availablePackages,
+				linktos, NULL);
+	    }
+	    /*@=branchstate@*/
+
+	    rpmdsProblem(ps, pkgNEVRA, linktos, suggestedKeys, adding);
+
+	}
+	    /*@switchbreak@*/ break;
+	case 2:		/* something went wrong! */
+	default:
+	    ourrc = 1;
+	    /*@switchbreak@*/ break;
+	}
+    }
+
+    ps = rpmpsFree(ps);
     return ourrc;
 }
 
@@ -952,7 +1274,9 @@ static int checkPackageSet(rpmts ts, const char * dep,
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, mi, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
-    int scareMem = 1;
+    rpmdepFlags depFlags = rpmtsDFlags(ts);
+    uint_32 tscolor = rpmtsColor(ts);
+    int scareMem = 0;
     Header h;
     int ec = 0;
 
@@ -960,18 +1284,32 @@ static int checkPackageSet(rpmts ts, const char * dep,
 		ts->removedPackages, ts->numRemovedPackages, 1);
     while ((h = rpmdbNextIterator(mi)) != NULL) {
 	const char * pkgNEVRA;
-	rpmds requires, conflicts, dirnames;
+	rpmds requires = NULL;
+	rpmds conflicts = NULL;
+	rpmds dirnames = NULL;
+	rpmds linktos = NULL;
 	int rc;
 
 	pkgNEVRA = hGetNEVRA(h, NULL);
-	requires = rpmdsNew(h, RPMTAG_REQUIRENAME, scareMem);
+	if (!(depFlags & RPMDEPS_FLAG_NOREQUIRES))
+	    requires = rpmdsNew(h, RPMTAG_REQUIRENAME, scareMem);
+	if (!(depFlags & RPMDEPS_FLAG_NOCONFLICTS))
+	    conflicts = rpmdsNew(h, RPMTAG_CONFLICTNAME, scareMem);
+	if (!(depFlags & RPMDEPS_FLAG_NOPARENTDIRS))
+	    dirnames = rpmdsNew(h, RPMTAG_DIRNAMES, scareMem);
+	if (!(depFlags & RPMDEPS_FLAG_NOLINKTOS))
+	    linktos = rpmdsNew(h, RPMTAG_FILELINKTOS, scareMem);
+
 	(void) rpmdsSetNoPromote(requires, _rpmds_nopromote);
-	conflicts = rpmdsNew(h, RPMTAG_CONFLICTNAME, scareMem);
 	(void) rpmdsSetNoPromote(conflicts, _rpmds_nopromote);
-	dirnames = rpmdsNew(h, RPMTAG_DIRNAMES, scareMem);
 	(void) rpmdsSetNoPromote(dirnames, _rpmds_nopromote);
-	rc = checkPackageDeps(ts, pkgNEVRA, requires, conflicts, dirnames,
-		dep, 0, adding);
+	(void) rpmdsSetNoPromote(linktos, _rpmds_nopromote);
+
+	rc = checkPackageDeps(ts, pkgNEVRA,
+		requires, conflicts, dirnames, linktos,
+		dep, tscolor, adding);
+
+	linktos = rpmdsFree(linktos);
 	dirnames = rpmdsFree(dirnames);
 	conflicts = rpmdsFree(conflicts);
 	requires = rpmdsFree(requires);
@@ -1111,8 +1449,8 @@ static int ignoreDep(const rpmts ts, const rpmte p, const rpmte q)
     if (!badDepsInitialized) {
 	char * s = rpmExpand("%{?_dependency_whiteout}", NULL);
 	const char ** av = NULL;
-	int anaconda = rpmtsFlags(ts) & RPMTRANS_FLAG_ANACONDA;
-	int msglvl = (anaconda || (rpmtsFlags(ts) & RPMTRANS_FLAG_DEPLOOPS))
+	int anaconda = rpmtsDFlags(ts) & RPMDEPS_FLAG_ANACONDA;
+	int msglvl = (anaconda || (rpmtsDFlags(ts) & RPMDEPS_FLAG_DEPLOOPS))
 			? RPMMESS_WARNING : RPMMESS_DEBUG;
 	int ac = 0;
 	int i;
@@ -1183,6 +1521,11 @@ static void markLoop(/*@special@*/ tsortInfo tsi, rpmte q)
     /*@=branchstate@*/
 }
 
+/*
+ * Return display string a dependency, adding contextual flags marker.
+ * @param f		dependency flags
+ * @return		display string
+ */
 static inline /*@observer@*/ const char * identifyDepend(int_32 f)
 	/*@*/
 {
@@ -1221,7 +1564,8 @@ static inline /*@observer@*/ const char * identifyDepend(int_32 f)
 static /*@owned@*/ /*@null@*/ const char *
 zapRelation(rpmte q, rpmte p,
 		int zap, /*@in@*/ /*@out@*/ int * nzaps, int msglvl)
-	/*@modifies q, p, requires, *nzaps @*/
+	/*@globals rpmGlobalMacroContext, h_errno @*/
+	/*@modifies q, p, *nzaps, rpmGlobalMacroContext @*/
 {
     rpmds requires;
     tsortInfo tsi_prev;
@@ -1387,14 +1731,16 @@ static int orderListIndexCmp(const void * one, const void * two)	/*@*/
 /**
  * Add element to list sorting by tsi_qcnt.
  * @param p		new element
- * @retval qp		address of first element
- * @retval rp		address of last element
+ * @retval *qp		first element
+ * @retval *rp		last element
+ * @param prefcolor	preferred color
  */
 /*@-boundswrite@*/
 /*@-mustmod@*/
 static void addQ(/*@dependent@*/ rpmte p,
 		/*@in@*/ /*@out@*/ rpmte * qp,
-		/*@in@*/ /*@out@*/ rpmte * rp)
+		/*@in@*/ /*@out@*/ rpmte * rp,
+		uint_32 prefcolor)
 	/*@modifies p, *qp, *rp @*/
 {
     rpmte q, qprev;
@@ -1414,6 +1760,10 @@ static void addQ(/*@dependent@*/ rpmte p,
 	 q != NULL;
 	 qprev = q, q = rpmteTSI(q)->tsi_suc)
     {
+	/* XXX Insure preferred color first. */
+	if (rpmteColor(p) != prefcolor && rpmteColor(p) != rpmteColor(q))
+	    continue;
+
 	/* XXX Insure removed after added. */
 	if (rpmteType(p) == TR_REMOVED && rpmteType(p) != rpmteType(q))
 	    continue;
@@ -1439,12 +1789,22 @@ static void addQ(/*@dependent@*/ rpmte p,
 /*@=mustmod@*/
 /*@=boundswrite@*/
 
+/*@unchecked@*/
+#ifdef	NOTYET
+static uint32_t _autobits = _notpre(_ALL_REQUIRES_MASK);
+#define isAuto(_x)	((_x) & _autobits)
+#else
+static uint32_t _autobits = 0xffffffff;
+#define	isAuto(_x)	(1)
+#endif
+
 /*@-bounds@*/
 int rpmtsOrder(rpmts ts)
 {
     rpmds requires;
     int_32 Flags;
-    int anaconda = rpmtsFlags(ts) & RPMTRANS_FLAG_ANACONDA;
+    int anaconda = rpmtsDFlags(ts) & RPMDEPS_FLAG_ANACONDA;
+    uint_32 prefcolor = rpmtsPrefColor(ts);
     rpmtsi pi; rpmte p;
     rpmtsi qi; rpmte q;
     rpmtsi ri; rpmte r;
@@ -1480,7 +1840,7 @@ int rpmtsOrder(rpmts ts)
     while ((p = rpmtsiNext(pi, TR_REMOVED)) != NULL) {
 	alKey pkgKey;
 	fnpyKey key;
-	int tscolor = 0x3;
+	uint_32 tscolor = rpmtsColor(ts);
 	pkgKey = RPMAL_NOMATCH;
 /*@-abstract@*/
 	key = (fnpyKey) p;
@@ -1490,7 +1850,7 @@ int rpmtsOrder(rpmts ts)
 			rpmteFI(p, RPMTAG_BASENAMES), tscolor);
 	/* XXX pretend erasedPackages are just appended to addedPackages. */
 	pkgKey = (alKey)(((long)pkgKey) + ts->numAddedPackages);
-	rpmteSetAddedKey(p, pkgKey);
+	(void) rpmteSetAddedKey(p, pkgKey);
     }
     pi = rpmtsiFree(pi);
     rpmalMakeIndex(ts->erasedPackages);
@@ -1521,10 +1881,10 @@ int rpmtsOrder(rpmts ts)
     pi = rpmtsiInit(ts);
     while ((p = rpmtsiNext(pi, oType)) != NULL) {
 
-      if ((requires = rpmteDS(p, RPMTAG_REQUIRENAME)) != NULL) {
-
 	memset(selected, 0, sizeof(*selected) * ts->orderCount);
 
+      if ((requires = rpmteDS(p, RPMTAG_REQUIRENAME)) != NULL) {
+
 	/* Avoid narcisstic relations. */
 	selected[rpmtsiOc(pi)] = 1;
 
@@ -1536,6 +1896,8 @@ int rpmtsOrder(rpmts ts)
 	while (rpmdsNext(requires) >= 0) {
 
 	    Flags = rpmdsFlags(requires);
+	    if (!isAuto(Flags))
+		/*@innercontinue@*/ continue;
 
 	    switch (rpmteType(p)) {
 	    case TR_REMOVED:
@@ -1561,6 +1923,8 @@ int rpmtsOrder(rpmts ts)
 	while (rpmdsNext(requires) >= 0) {
 
 	    Flags = rpmdsFlags(requires);
+	    if (!isAuto(Flags))
+		/*@innercontinue@*/ continue;
 
 	    switch (rpmteType(p)) {
 	    case TR_REMOVED:
@@ -1581,27 +1945,29 @@ int rpmtsOrder(rpmts ts)
 	}
       }
 
-#ifdef	NOTYET
-      {	const char * Name = rpmteN(p);
-	if (Name == NULL || !strcmp(Name, "setup") || !strcmp(Name, "filesystem"))
-	    continue;
+      if (_autobits != 0xffffffff)
+      {
 
-	/* Finally synthesize a requirement on parent directories. */
-	if ((requires = rpmteDS(p, RPMTAG_DIRNAMES)) == NULL)
-	    continue;
-	requires = rpmdsInit(requires);
+	/* Order by requiring parent directories pre-requsites. */
+	requires = rpmdsInit(rpmteDS(p, RPMTAG_DIRNAMES));
 	if (requires != NULL)
 	while (rpmdsNext(requires) >= 0) {
 
 	    /* T3. Record next "q <- p" relation (i.e. "p" requires "q"). */
 	    (void) addRelation(ts, p, selected, requires);
 
-	    /* XXX only add the 1st dirname to avoid loops. */
-	    break;
+	}
+
+	/* Order by requiring no dangling symlinks. */
+	requires = rpmdsInit(rpmteDS(p, RPMTAG_FILELINKTOS));
+	if (requires != NULL)
+	while (rpmdsNext(requires) >= 0) {
+
+	    /* T3. Record next "q <- p" relation (i.e. "p" requires "q"). */
+	    (void) addRelation(ts, p, selected, requires);
 
 	}
       }
-#endif
 
     }
     pi = rpmtsiFree(pi);
@@ -1648,7 +2014,7 @@ rescan:
 	if (rpmteTSI(p)->tsi_count != 0)
 	    continue;
 	rpmteTSI(p)->tsi_suc = NULL;
-	addQ(p, &q, &r);
+	addQ(p, &q, &r, prefcolor);
 	qlen++;
     }
     pi = rpmtsiFree(pi);
@@ -1713,7 +2079,7 @@ rescan:
 		/* XXX TODO: add control bit. */
 		rpmteTSI(p)->tsi_suc = NULL;
 /*@-nullstate@*/	/* XXX FIX: rpmteTSI(q)->tsi_suc can be NULL. */
-		addQ(p, &rpmteTSI(q)->tsi_suc, &r);
+		addQ(p, &rpmteTSI(q)->tsi_suc, &r, prefcolor);
 /*@=nullstate@*/
 		qlen++;
 	    }
@@ -1788,7 +2154,7 @@ rescan:
 	    while ((p = q) != NULL && (q = rpmteTSI(p)->tsi_chain) != NULL) {
 		const char * dp;
 		char buf[4096];
-		int msglvl = (anaconda || (rpmtsFlags(ts) & RPMTRANS_FLAG_DEPLOOPS))
+		int msglvl = (anaconda || (rpmtsDFlags(ts) & RPMDEPS_FLAG_DEPLOOPS))
 			? RPMMESS_WARNING : RPMMESS_DEBUG;
 ;
 
@@ -1835,8 +2201,10 @@ rescan:
 	rpmMessage(RPMMESS_ERROR, _("rpmtsOrder failed, %d elements remain\n"),
 			loopcheck);
 
+#ifdef	NOTYET
 	/* Do autorollback goal since we could not sort this transaction properly. */
-	(void) rpmtsDoARBGoal(ts, NULL, RPMPROB_FILTER_NONE);
+	(void) rpmtsRollback(ts, RPMPROB_FILTER_NONE, 0, NULL);
+#endif
 
 	return loopcheck;
     }
@@ -1909,6 +2277,7 @@ assert(newOrderCount == ts->orderCount);
 
 int rpmtsCheck(rpmts ts)
 {
+    rpmdepFlags depFlags = rpmtsDFlags(ts);
     uint_32 tscolor = rpmtsColor(ts);
     rpmdbMatchIterator mi = NULL;
     rpmtsi pi = NULL; rpmte p;
@@ -1936,18 +2305,24 @@ int rpmtsCheck(rpmts ts)
      */
     pi = rpmtsiInit(ts);
     while ((p = rpmtsiNext(pi, TR_ADDED)) != NULL) {
-	rpmds provides;
+	rpmds provides, requires, conflicts, dirnames, linktos;
 
 /*@-nullpass@*/	/* FIX: rpmts{A,O} can return null. */
 	rpmMessage(RPMMESS_DEBUG, "========== +++ %s %s/%s 0x%x\n",
 		rpmteNEVR(p), rpmteA(p), rpmteO(p), rpmteColor(p));
 /*@=nullpass@*/
+	requires = (!(depFlags & RPMDEPS_FLAG_NOREQUIRES)
+	    ? rpmteDS(p, RPMTAG_REQUIRENAME) : NULL);
+	conflicts = (!(depFlags & RPMDEPS_FLAG_NOCONFLICTS)
+	    ? rpmteDS(p, RPMTAG_CONFLICTNAME) : NULL);
+	dirnames = (!(depFlags & RPMDEPS_FLAG_NOPARENTDIRS)
+	    ? rpmteDS(p, RPMTAG_DIRNAMES) : NULL);
+	linktos = (!(depFlags & RPMDEPS_FLAG_NOLINKTOS)
+	    ? rpmteDS(p, RPMTAG_FILELINKTOS) : NULL);
+
 	rc = checkPackageDeps(ts, rpmteNEVRA(p),
-			rpmteDS(p, RPMTAG_REQUIRENAME),
-			rpmteDS(p, RPMTAG_CONFLICTNAME),
-			rpmteDS(p, RPMTAG_DIRNAMES),
-			NULL,
-			tscolor, 1);
+			requires, conflicts, dirnames, linktos,
+			NULL, tscolor, 1);
 	if (rc)
 	    goto exit;
 
@@ -1961,6 +2336,18 @@ int rpmtsCheck(rpmts ts)
 	    if ((Name = rpmdsN(provides)) == NULL)
 		/*@innercontinue@*/ continue;	/* XXX can't happen */
 
+#ifdef	NOTYET
+	    if (rpmdsNSType(provides) == RPMNS_TYPE_ENVVAR) {
+		const char * EVR = rpmdsEVR(provides);
+		if (rpmdsNegateRC(provides, 0))
+		    EVR = NULL;
+		rc = envPut(Name, EVR);
+		if (!rc)
+		    /*@innercontinue@*/ continue;
+		/*@innerbreak@*/ break;
+	    }
+#endif
+
 	    /* Adding: check provides key against conflicts matches. */
 	    if (!checkDependentConflicts(ts, Name))
 		/*@innercontinue@*/ continue;
@@ -1985,14 +2372,6 @@ int rpmtsCheck(rpmts ts)
 		rpmteNEVR(p), rpmteA(p), rpmteO(p), rpmteColor(p));
 /*@=nullpass@*/
 
-#if defined(DYING) || defined(__LCLINT__)
-	/* XXX all packages now have Provides: name = version-release */
-	/* Erasing: check name against requiredby matches. */
-	rc = checkDependentPackages(ts, rpmteN(p));
-	if (rc)
-		goto exit;
-#endif
-
 	rc = 0;
 	provides = rpmteDS(p, RPMTAG_PROVIDENAME);
 	provides = rpmdsInit(provides);
@@ -2029,6 +2408,22 @@ int rpmtsCheck(rpmts ts)
     }
     pi = rpmtsiFree(pi);
 
+    /*
+     * Make sure transaction dependencies are satisfied.
+     */
+    {	const char * tsNEVRA = "transaction dependencies";
+	rpmds R = rpmdsFromPRCO(ts->PRCO, RPMTAG_REQUIRENAME);
+	rpmds C = rpmdsFromPRCO(ts->PRCO, RPMTAG_CONFLICTNAME);
+	rpmds D = NULL;
+	rpmds L = NULL;
+	const char * dep = NULL;
+	int adding = 2;
+	tscolor = 0;	/* XXX no coloring for transaction dependencies. */
+	rc = checkPackageDeps(ts, tsNEVRA, R, C, D, L, dep, tscolor, adding);
+	if (rc)
+	    goto exit;
+    }
+
     rc = 0;
 
 exit:
@@ -2044,9 +2439,14 @@ exit:
 	xx = rpmdbCloseDBI(rpmtsGetRdb(ts), RPMDBI_DEPENDS);
     /*@=branchstate@*/
 
-     /* If we have a failure we need to do the autorollback goal if specified.  */
-    if (rc || rpmpsNumProblems(rpmtsProblems(ts)) > 0)
-	(void) rpmtsDoARBGoal(ts, NULL, RPMPROB_FILTER_NONE);
+#ifdef	NOTYET
+     /* On failed dependencies, perform the autorollback goal (if any). */
+    {	rpmps ps = rpmtsProblems(ts);
+	if (rc || rpmpsNumProblems(ps) > 0)
+	    (void) rpmtsRollback(ts, RPMPROB_FILTER_NONE, 0, NULL);
+	ps = rpmpsFree(ps);
+    }
+#endif
 
     return rc;
 }
