@@ -8,6 +8,7 @@
 #include <rpmlib.h>
 #include <rpmmacro.h>	/* XXX for %_i18ndomains */
 
+#define	_RPMEVR_INTERNAL
 #include <rpmds.h>
 #include <rpmfi.h>
 
@@ -230,13 +231,18 @@ static /*@only@*/ char * base64Format(int_32 type, const void * data,
 	}
 	/*@=globs@*/
 
-	val = t = xmalloc(nt + padding + 1);
-
+	val = t = xcalloc(1, nt + padding + 1);
 	*t = '\0';
-	if ((enc = b64encode(data, ns)) != NULL) {
+
+    /* XXX b64encode accesses uninitialized memory. */
+    { 	unsigned char * _data = xcalloc(1, ns+1);
+	memcpy(_data, data, ns);
+	if ((enc = b64encode(_data, ns)) != NULL) {
 	    t = stpcpy(t, enc);
 	    enc = _free(enc);
 	}
+	_data = _free(_data);
+    }
 /*@=boundswrite@*/
     }
 
@@ -769,32 +775,38 @@ static /*@only@*/ char * depflagsFormat(int_32 type, const void * data,
 	/*@requires maxRead(data) >= 0 @*/
 {
     char * val;
-    char buf[10];
-    int anint;
 
     if (type != RPM_INT32_TYPE) {
 	val = xstrdup(_("(invalid type)"));
     } else {
-	anint = *((int_32 *) data);
-	buf[0] = '\0';
+	int anint = *((int_32 *) data);
+	char *t, *buf;
+
+	t = buf = alloca(32);
+	*t = '\0';
 
 /*@-boundswrite@*/
 #ifdef	NOTYET	/* XXX appending markers breaks :depflags format. */
 	if (anint & RPMSENSE_SCRIPT_PRE)
-	    strcat(buf,"(pre)");
+	    t = stpcpy(t, "(pre)");
 	if (anint & RPMSENSE_SCRIPT_POST)
-	    strcat(buf,"(post)");
+	    t = stpcpy(t, "(post)");
 	if (anint & RPMSENSE_SCRIPT_PREUN)
-	    strcat(buf,"(preun)");
+	    t = stpcpy(t, "(preun)");
 	if (anint & RPMSENSE_SCRIPT_POSTUN)
-	    strcat(buf,"(postun)");
+	    t = stpcpy(t, "(postun)");
 #endif
+	if (anint & RPMSENSE_SENSEMASK)
+	    *t++ = ' ';
 	if (anint & RPMSENSE_LESS)
-	    strcat(buf, "<");
+	    *t++ = '<';
 	if (anint & RPMSENSE_GREATER)
-	    strcat(buf, ">");
+	    *t++ = '>';
 	if (anint & RPMSENSE_EQUAL)
-	    strcat(buf, "=");
+	    *t++ = '=';
+	if (anint & RPMSENSE_SENSEMASK)
+	    *t++ = ' ';
+	*t = '\0';
 /*@=boundswrite@*/
 
 	val = xmalloc(5 + padding);
@@ -1356,11 +1368,14 @@ static int i18nTag(Header h, int_32 tag, /*@out@*/ rpmTagType * type,
 	const char * msgid;
 
 	{   const char * tn = tagName(tag);
-	    const char * n;
+	    const char * n = NULL;
 	    char * mk;
-	    (void) headerNVR(h, &n, NULL, NULL);
-	    mk = alloca(strlen(n) + strlen(tn) + sizeof("()"));
-	    sprintf(mk, "%s(%s)", n, tn);
+	    size_t nb = sizeof("()");
+	    int xx = headerNVR(h, &n, NULL, NULL);
+	    if (tn)	nb += strlen(tn);
+	    if (n)	nb += strlen(n);
+	    mk = alloca(nb);
+	    sprintf(mk, "%s(%s)", (n?n:""), (tn?tn:""));
 	    msgkey = mk;
 	}
 
