@@ -42,10 +42,6 @@ rpmRC rpmMkdirPath (const char * dpath, const char * dname)
 	    return RPMRC_FAIL;
 	}
     }
-    if ((rc = Access(dpath, W_OK))) {
-	rpmError(RPMERR_CREATE, _("cannot write to %%%s %s\n"), dname, dpath);
-	return RPMRC_FAIL;
-    }
     return RPMRC_OK;
 }
 
@@ -176,7 +172,7 @@ int makeTempFile(const char * prefix, const char ** fnptr, FD_t * fdptr)
 	    /*@switchbreak@*/ break;
 	}
 
-	fd = Fopen(tempfn, "w+x.ufdio");
+	fd = Fopen(tempfn, "w+x");
 	/* XXX FIXME: errno may not be correct for ufdio */
     } while ((fd == NULL || Ferror(fd)) && errno == EEXIST);
 
@@ -220,6 +216,8 @@ int makeTempFile(const char * prefix, const char ** fnptr, FD_t * fdptr)
 
 errxit:
     tempfn = _free(tempfn);
+    if (fnptr)
+	*fnptr = NULL;
     /*@-usereleased@*/
     if (fd != NULL) (void) Fclose(fd);
     /*@=usereleased@*/
