@@ -12,7 +12,6 @@
 #include "rpmts.h"
 
 #include "misc.h"	/* XXX stripTrailingChar() */
-#include "legacy.h"	/* XXX legacyRetrofit() */
 #include "rpmlead.h"
 
 #include "header_internal.h"	/* XXX headerCheck */
@@ -542,12 +541,11 @@ verifyinfo_exit:
 	/* Parse the parameters from the OpenPGP packets that will be needed. */
 	xx = pgpPrtPkts(sig, info->count, dig, (_print_pkts & rpmIsDebug()));
 	if (dig->signature.version != 3 && dig->signature.version != 4) {
-	    rpmMessage(RPMMESS_ERROR,
-		_("skipping header with unverifiable V%u signature\n"),
+	    rpmMessage(RPMMESS_WARNING,
+		_("only V3 and V4 signatures can be verified, skipping V%u signature\n"),
 		dig->signature.version);
 	    rpmtsCleanDig(ts);
-	    rc = RPMRC_FAIL;
-	    goto exit;
+	    goto verifyinfo_exit;
 	}
 
 	ildl[0] = htonl(ril);
@@ -583,12 +581,11 @@ verifyinfo_exit:
 	/* Parse the parameters from the OpenPGP packets that will be needed. */
 	xx = pgpPrtPkts(sig, info->count, dig, (_print_pkts & rpmIsDebug()));
 	if (dig->signature.version != 3 && dig->signature.version != 4) {
-	    rpmMessage(RPMMESS_ERROR,
-		_("skipping header with unverifiable V%u signature\n"),
+	    rpmMessage(RPMMESS_WARNING,
+		_("only V3 and V4 signatures can be verified, skipping V%u signature\n"),
 		dig->signature.version);
 	    rpmtsCleanDig(ts);
-	    rc = RPMRC_FAIL;
-	    goto exit;
+	    goto verifyinfo_exit;
 	}
 	/*@fallthrough@*/
     case RPMTAG_SHA1HEADER:
@@ -657,6 +654,7 @@ rpmRC rpmReadHeader(rpmts ts, FD_t fd, Header *hdrp, const char ** msg)
     size_t uc;
     int_32 nb;
     Header h = NULL;
+    const char * origin = NULL;
     rpmRC rc = RPMRC_FAIL;		/* assume failure */
     int xx;
 
@@ -725,6 +723,11 @@ rpmRC rpmReadHeader(rpmts ts, FD_t fd, Header *hdrp, const char ** msg)
     }
     h->flags |= HEADERFLAG_ALLOCATED;
     ei = NULL;	/* XXX will be freed with header */
+
+    /* Save the opened path as the header origin. */
+    origin = fdGetOPath(fd);
+    if (origin != NULL)
+	(void) headerSetOrigin(h, origin);
     
 exit:
 /*@-boundswrite@*/
@@ -762,6 +765,7 @@ rpmRC rpmReadPackageFile(rpmts ts, FD_t fd, const char * fn, Header * hdrp)
     const char * msg;
     rpmVSFlags vsflags;
     rpmRC rc = RPMRC_FAIL;	/* assume failure */
+    rpmop opsave = memset(alloca(sizeof(*opsave)), 0, sizeof(*opsave));
     int xx;
     int i;
 
@@ -781,6 +785,9 @@ rpmRC rpmReadPackageFile(rpmts ts, FD_t fd, const char * fn, Header * hdrp)
     }
 #endif
 
+    /* Snapshot current I/O counters (cached persistent I/O reuses counters) */
+    (void) rpmswAdd(opsave, fdstat_op(fd, FDSTAT_READ));
+
     memset(l, 0, sizeof(*l));
     rc = readLead(fd, l);
     if (rc != RPMRC_OK)
@@ -918,10 +925,10 @@ rpmRC rpmReadPackageFile(rpmts ts, FD_t fd, const char * fn, Header * hdrp)
 	/* Parse the parameters from the OpenPGP packets that will be needed. */
 	xx = pgpPrtPkts(sig, siglen, dig, (_print_pkts & rpmIsDebug()));
 	if (dig->signature.version != 3 && dig->signature.version != 4) {
-	    rpmMessage(RPMMESS_ERROR,
-		_("skipping package %s with unverifiable V%u signature\n"),
-		fn, dig->signature.version);
-	    rc = RPMRC_FAIL;
+	    rpmMessage(RPMMESS_WARNING,
+		_("only V3 and V4 signatures can be verified, skipping V%u signature\n"),
+		dig->signature.version);
+	    rc = RPMRC_OK;	/* XXX return header w/o verify */
 	    goto exit;
 	}
     {	void * uh = NULL;
@@ -944,10 +951,10 @@ rpmRC rpmReadPackageFile(rpmts ts, FD_t fd, const char * fn, Header * hdrp)
 	/* Parse the parameters from the OpenPGP packets that will be needed. */
 	xx = pgpPrtPkts(sig, siglen, dig, (_print_pkts & rpmIsDebug()));
 	if (dig->signature.version != 3 && dig->signature.version != 4) {
-	    rpmMessage(RPMMESS_ERROR,
-		_("skipping package %s with unverifiable V%u signature\n"), 
-		fn, dig->signature.version);
-	    rc = RPMRC_FAIL;
+	    rpmMessage(RPMMESS_WARNING,
+		_("only V3 and V4 signatures can be verified, skipping V%u signature\n"),
+		dig->signature.version);
+	    rc = RPMRC_OK;	/* XXX return header w/o verify */
 	    goto exit;
 	}
 	/*@fallthrough@*/
@@ -976,10 +983,10 @@ rpmRC rpmReadPackageFile(rpmts ts, FD_t fd, const char * fn, Header * hdrp)
 	xx = pgpPrtPkts(sig, siglen, dig, (_print_pkts & rpmIsDebug()));
 
 	if (dig->signature.version != 3 && dig->signature.version != 4) {
-	    rpmMessage(RPMMESS_ERROR,
-		_("skipping package %s with unverifiable V%u signature\n"),
-		fn, dig->signature.version);
-	    rc = RPMRC_FAIL;
+	    rpmMessage(RPMMESS_WARNING,
+		_("only V3 and V4 signatures can be verified, skipping V%u signature\n"),
+		dig->signature.version);
+	    rc = RPMRC_OK;	/* XXX return header w/o verify */
 	    goto exit;
 	}
 	/*@fallthrough@*/
@@ -1008,6 +1015,7 @@ rpmRC rpmReadPackageFile(rpmts ts, FD_t fd, const char * fn, Header * hdrp)
 		fddig->hashctx = NULL;
 		/*@switchbreak@*/ break;
 	    case PGPHASHALGO_SHA1:
+	    case PGPHASHALGO_RIPEMD160:
 #if HAVE_BEECRYPT_API_H
 	    case PGPHASHALGO_SHA256:
 	    case PGPHASHALGO_SHA384:
@@ -1050,9 +1058,7 @@ rpmRC rpmReadPackageFile(rpmts ts, FD_t fd, const char * fn, Header * hdrp)
 
 exit:
     if (rc != RPMRC_FAIL && h != NULL && hdrp != NULL) {
-	/* Convert legacy headers on the fly ... */
-	legacyRetrofit(h, l);
-	
+
 	/* Append (and remap) signature tags to the metadata. */
 	headerMergeLegacySigs(h, sigh);
 
@@ -1062,47 +1068,15 @@ exit:
 /*@=boundswrite@*/
     }
     h = headerFree(h);
+
+    /* Accumulate time reading package header. */
+    (void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_READHDR),
+		fdstat_op(fd, FDSTAT_READ));
+    (void) rpmswSub(rpmtsOp(ts, RPMTS_OP_READHDR),
+		opsave);
+
     rpmtsCleanDig(ts);
     sigh = rpmFreeSignature(sigh);
     return rc;
 }
-
-/**
- * Check for supported payload format in header.
- * @param h		header to check
- * @return		RPMRC_OK if supported, RPMRC_FAIL otherwise
- */
-rpmRC headerCheckPayloadFormat(Header h) {
-    rpmRC rc = RPMRC_FAIL;
-    int xx;
-    const char *payloadfmt = NULL;
-
-    xx = headerGetEntry(h, RPMTAG_PAYLOADFORMAT, NULL, 
-			(void **)&payloadfmt, NULL);
-    /* 
-     * XXX Ugh, rpm 3.x packages don't have payload format tag. Instead
-     * of blinly allowing, should check somehow (HDRID existence or... ?)
-     */
-    if (!payloadfmt)
-	return RPMRC_OK;
-
-    if (payloadfmt && strncmp(payloadfmt, "cpio", strlen("cpio")) == 0) {
-	rc = RPMRC_OK;
-    } else {
-        const char *nevra = hGetNEVRA(h, NULL);
-        if (payloadfmt && strncmp(payloadfmt, "drpm", strlen("drpm")) == 0) {
-            rpmMessage(RPMMESS_ERROR,
-                     _("%s is a Delta RPM and cannot be directly installed\n"),
-                     nevra);
-        } else {
-            rpmMessage(RPMMESS_ERROR, 
-                     _("Unsupported payload (%s) in package %s\n"),
-                     payloadfmt ? payloadfmt : "none", nevra);
-        } 
-        nevra = _free(nevra);
-    }
-    return rc;
-}
-
-
 /*@=bounds@*/
