@@ -13,32 +13,19 @@
 extern time_t get_date(const char * p, void * now);	/* XXX expedient lies */
 /*@=redecl@*/
 
+/*@-fullinitblock@*/
 /*@unchecked@*/
-struct rpmInstallArguments_s rpmIArgs = {
-    0,			/* transFlags */
-			/* probFilter */
-    (RPMPROB_FILTER_REPLACEOLDFILES | RPMPROB_FILTER_REPLACENEWFILES),
-    0,			/* installInterfaceFlags */
-    0,			/* eraseInterfaceFlags */
-    0,			/* qva_flags */
-    0,			/* arbtid */
-    0,			/* rbtid */
-    NULL,		/* rbtidExcludes */
-    0,			/* numrbtidExcludes */
-    0,			/* numRelocations */
-    0,			/* noDeps */
-    0,			/* incldocs */
-    NULL,		/* relocations */
-    NULL,		/* prefix */
-    NULL		/* rootdir */
+struct rpmQVKArguments_s rpmIArgs = {
+    .probFilter = (RPMPROB_FILTER_REPLACEOLDFILES | RPMPROB_FILTER_REPLACENEWFILES),
 };
+/*@=fullinitblock@*/
 
 #define	POPT_RELOCATE		-1021
 #define	POPT_EXCLUDEPATH	-1022
 #define	POPT_ROLLBACK		-1023
 #define	POPT_ROLLBACK_EXCLUDE	-1024
-/* -1025 thrugh -1032 are common in rpmcli.h. */
-#define	POPT_AUTOROLLBACK_GOAL	-1033
+/* -1025 thrugh -1033 are common in rpmcli.h. */
+#define	POPT_AUTOROLLBACK_GOAL	-1036
 
 #define	alloca_strdup(_s)	strcpy(alloca(strlen(_s)+1), (_s))
 
@@ -68,7 +55,7 @@ static void installArgCallback( /*@unused@*/ poptContext con,
 	/*@globals rpmIArgs, stderr, fileSystem @*/
 	/*@modifies rpmIArgs, stderr, fileSystem @*/
 {
-    struct rpmInstallArguments_s * ia = &rpmIArgs;
+    QVA_t ia = &rpmIArgs;
 
     /* XXX avoid accidental collisions with POPT_BIT_SET for flags */
     /*@-branchstate@*/
@@ -229,22 +216,47 @@ struct poptOption rpmInstallPoptTable[] = {
 	installArgCallback, 0, NULL, NULL },
 /*@=type@*/
 
- { "aid", '\0', POPT_BIT_SET, &rpmIArgs.transFlags, RPMTRANS_FLAG_ADDINDEPS,
+#if 0
+ { "aid", '\0', POPT_BIT_SET, &rpmIArgs.depFlags, RPMDEPS_FLAG_ADDINDEPS,
 	N_("add suggested packages to transaction"), NULL },
+ { "anaconda", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+ 	&rpmIArgs.depFlags, RPMDEPS_FLAG_ANACONDA|RPMDEPS_FLAG_DEPLOOPS,
+	N_("use anaconda \"presentation order\""), NULL},
+ { "deploops", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+ 	&rpmIArgs.depFlags, RPMDEPS_FLAG_DEPLOOPS,
+	N_("print dependency loops as warning"), NULL},
+ { "nosuggest", '\0', POPT_BIT_SET,
+	&rpmIArgs.depFlags, RPMDEPS_FLAG_NOSUGGEST,
+	N_("do not suggest missing dependency resolution(s)"), NULL},
+ { "noconflicts", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmIArgs.depFlags, RPMDEPS_FLAG_NOCONFLICTS,
+	N_("do not check added package conflicts"), NULL},
+ { "nolinktos", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmIArgs.depFlags, RPMDEPS_FLAG_NOLINKTOS,
+	N_("ignore added package requires on symlink targets"), NULL},
+ { "noobsoletes", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmIArgs.depFlags, RPMDEPS_FLAG_NOOBSOLETES,
+	N_("ignore added package obsoletes"), NULL},
+ { "noparentdirs", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmIArgs.depFlags, RPMDEPS_FLAG_NOPARENTDIRS,
+	N_("ignore added package requires on file parent directory"), NULL},
+ { "norequires", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmIArgs.depFlags, RPMDEPS_FLAG_NOREQUIRES,
+	N_("do not check added package requires"), NULL},
+ { "noupgrade", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmIArgs.depFlags, RPMDEPS_FLAG_NOUPGRADE,
+	N_("ignore added package upgrades"), NULL},
+#endif
 
  { "allfiles", '\0', POPT_BIT_SET,
 	&rpmIArgs.transFlags, RPMTRANS_FLAG_ALLFILES,
   N_("install all files, even configurations which might otherwise be skipped"),
 	NULL},
  { "allmatches", '\0', POPT_BIT_SET,
-	&rpmIArgs.eraseInterfaceFlags, UNINSTALL_ALLMATCHES,
+	&rpmIArgs.installInterfaceFlags, INSTALL_ALLMATCHES,
 	N_("remove all packages which match <package> (normally an error is generated if <package> specified multiple packages)"),
 	NULL},
 
- { "anaconda", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
- 	&rpmIArgs.transFlags, RPMTRANS_FLAG_ANACONDA|RPMTRANS_FLAG_DEPLOOPS,
-	N_("use anaconda \"presentation order\""), NULL},
-
  { "apply", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN, &rpmIArgs.transFlags,
 	(_noTransScripts|_noTransTriggers|
 		RPMTRANS_FLAG_APPLYONLY|RPMTRANS_FLAG_PKGCOMMIT),
@@ -254,10 +266,6 @@ struct poptOption rpmInstallPoptTable[] = {
 	&rpmIArgs.probFilter, RPMPROB_FILTER_FORCERELOCATE,
 	N_("relocate files in non-relocatable package"), NULL},
 
- { "deploops", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
- 	&rpmIArgs.transFlags, RPMTRANS_FLAG_DEPLOOPS,
-	N_("print dependency loops as warning"), NULL},
-
  { "dirstash", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
 	&rpmIArgs.transFlags, RPMTRANS_FLAG_DIRSTASH,
 	N_("save erased package files by renaming into sub-directory"), NULL},
@@ -286,12 +294,14 @@ struct poptOption rpmInstallPoptTable[] = {
 	N_("<packagefile>+") },
  { "hash", 'h', POPT_BIT_SET, &rpmIArgs.installInterfaceFlags, INSTALL_HASH,
 	N_("print hash marks as package installs (good with -v)"), NULL},
+#ifndef	DIEDIEDIE
  { "ignorearch", '\0', POPT_BIT_SET,
 	&rpmIArgs.probFilter, RPMPROB_FILTER_IGNOREARCH,
 	N_("don't verify package architecture"), NULL},
  { "ignoreos", '\0', POPT_BIT_SET,
 	&rpmIArgs.probFilter, RPMPROB_FILTER_IGNOREOS,
 	N_("don't verify package operating system"), NULL},
+#endif
  { "ignoresize", '\0', POPT_BIT_SET, &rpmIArgs.probFilter,
 	(RPMPROB_FILTER_DISKSPACE|RPMPROB_FILTER_DISKNODES),
 	N_("don't check disk space before installing"), NULL},
@@ -325,10 +335,6 @@ struct poptOption rpmInstallPoptTable[] = {
 	N_("do not reorder package installation to satisfy dependencies"),
 	NULL},
 
- { "nosuggest", '\0', POPT_BIT_SET, &rpmIArgs.transFlags,
-	RPMTRANS_FLAG_NOSUGGEST,
-	N_("do not suggest missing dependency resolution(s)"), NULL},
-
  { "noscripts", '\0', 0, NULL, RPMCLI_POPT_NOSCRIPTS,
 	N_("do not execute package scriptlet(s)"), NULL },
 
@@ -374,7 +380,7 @@ struct poptOption rpmInstallPoptTable[] = {
  { "percent", '\0', POPT_BIT_SET,
 	&rpmIArgs.installInterfaceFlags, INSTALL_PERCENT,
 	N_("print percentages as package installs"), NULL},
- { "prefix", '\0', POPT_ARG_STRING, &rpmIArgs.prefix, 0,
+ { "prefix", '\0', POPT_ARG_STRING, &rpmIArgs.qva_prefix, 0,
 	N_("relocate the package to <dir>, if relocatable"),
 	N_("<dir>") },
  { "relocate", '\0', POPT_ARG_STRING, 0, POPT_RELOCATE,
