@@ -31,10 +31,10 @@ int specedit = 0;
 #define POPT_HDLIST		-1011
 #define POPT_FTSWALK		-1012
 
-/* -1025 thrugh -1032 are common in rpmcli.h. */
-#define	POPT_TARGETPLATFORM	-1036
+/* -1025 thrugh -1033 are common in rpmcli.h. */
 #define	POPT_TRUST		-1037
 #define	POPT_WHATNEEDS		-1038
+#define	POPT_SPECSRPM		-1039
 
 /* ========== Query/Verify/Signature source args */
 static void rpmQVSourceArgCallback( /*@unused@*/ poptContext con,
@@ -90,6 +90,11 @@ static void rpmQVSourceArgCallback( /*@unused@*/ poptContext con,
 	qva->qva_source |= RPMQV_SPECFILE;
 	qva->qva_sourceCount++;
 	break;
+/* XXX SPECSRPM is not verify sources */
+    case POPT_SPECSRPM:
+	qva->qva_source |= RPMQV_SPECSRPM;
+	qva->qva_sourceCount++;
+	break;
     case POPT_QUERYBYNUMBER:
 	qva->qva_source |= RPMQV_DBOFFSET; 
 	qva->qva_sourceCount++;
@@ -135,6 +140,8 @@ struct poptOption rpmQVSourcePoptTable[] = {
 	N_("query/verify a header instance"), "HDRNUM" },
  { "specfile", '\0', 0, 0, POPT_SPECFILE,
 	N_("query a spec file"), N_("<spec>") },
+ { "specsrpm", '\0', POPT_ARGFLAG_DOC_HIDDEN, 0, POPT_SPECSRPM,
+	N_("query source metadata from spec file parse"), N_("<spec>") },
  { "tid", '\0', POPT_ARGFLAG_DOC_HIDDEN, 0, POPT_QUERYBYTID,
 	N_("query/verify package(s) from install transaction"), "TID" },
  { "triggeredby", '\0', 0, 0, POPT_TRIGGEREDBY, 
@@ -251,18 +258,6 @@ static void queryArgCallback(poptContext con,
 	qva->qva_flags |= VERIFY_SCRIPT;
 	break;
 
-    case POPT_TARGETPLATFORM:
-	if (qva->targets) {
-	    int len = strlen(qva->targets) + 1 + strlen(arg) + 1;
-	    qva->targets = xrealloc(qva->targets, len);
-	    strcat(qva->targets, ",");
-	} else {
-	    qva->targets = xmalloc(strlen(arg) + 1);
-	    qva->targets[0] = '\0';
-	}
-	strcat(qva->targets, arg);
-	break;
-
     /* XXX perhaps POPT_ARG_INT instead of callback. */
     case POPT_TRUST:
     {	char * end = NULL;
@@ -296,6 +291,10 @@ struct poptOption rpmQueryPoptTable[] = {
  { "list", 'l', 0, 0, 'l',
 	N_("list files in package"), NULL },
 
+ { "aid", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmQVKArgs.depFlags, RPMDEPS_FLAG_ADDINDEPS,
+	N_("add suggested packages to transaction"), NULL },
+
  /* Duplicate file attr flags from packages into command line options. */
  { "noconfig", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
 	&rpmQVKArgs.qva_fflags, RPMFILE_CONFIG,
@@ -318,13 +317,13 @@ struct poptOption rpmQueryPoptTable[] = {
  { "qf", '\0', POPT_ARG_STRING | POPT_ARGFLAG_DOC_HIDDEN, 0, 
 	POPT_QUERYFORMAT, NULL, NULL },
  { "queryformat", '\0', POPT_ARG_STRING, 0, POPT_QUERYFORMAT,
-	N_("use the following query format"), "QUERYFORMAT" },
+	N_("use the following query format"), N_("QUERYFORMAT") },
  { "specedit", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &specedit, -1,
 	N_("substitute i18n sections into spec file"), NULL },
  { "state", 's', 0, 0, 's',
 	N_("display the states of the listed files"), NULL },
- { "target", '\0', POPT_ARG_STRING|POPT_ARGFLAG_DOC_HIDDEN, 0,  POPT_TARGETPLATFORM,
-        N_("specify target platform"), "CPU-VENDOR-OS" },
+ { "target", '\0', POPT_ARG_STRING|POPT_ARGFLAG_DOC_HIDDEN, 0,  RPMCLI_POPT_TARGETPLATFORM,
+        N_("specify target platform"), N_("CPU-VENDOR-OS") },
    POPT_TABLEEND
 };
 
@@ -339,6 +338,10 @@ struct poptOption rpmVerifyPoptTable[] = {
  { NULL, '\0', POPT_ARG_INCLUDE_TABLE, rpmQVSourcePoptTable, 0,
 	NULL, NULL },
 
+ { "aid", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmQVKArgs.depFlags, RPMDEPS_FLAG_ADDINDEPS,
+	N_("add suggested packages to transaction"), NULL },
+
  /* Duplicate file attr flags from packages into command line options. */
  { "noconfig", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
 	&rpmQVKArgs.qva_fflags, RPMFILE_CONFIG,
