@@ -15,6 +15,7 @@
 #include "fsm.h"		/* XXX CPIO_FOO/FSM_FOO constants */
 #include "psm.h"
 
+#define	_RPMEVR_INTERNAL
 #include "rpmds.h"
 
 #define _RPMFI_INTERNAL
@@ -52,25 +53,20 @@ int rpmVersionCompare(Header first, Header second)
 {
     const char * one, * two;
     int_32 * epochOne, * epochTwo;
+    static int_32 zero = 0;
     int rc;
 
     if (!headerGetEntry(first, RPMTAG_EPOCH, NULL, (void **) &epochOne, NULL))
-	epochOne = NULL;
+	epochOne = &zero;
     if (!headerGetEntry(second, RPMTAG_EPOCH, NULL, (void **) &epochTwo, NULL))
-	epochTwo = NULL;
+	epochTwo = &zero;
 
-    if (epochOne != NULL && epochTwo == NULL)
-	return 1;
-    else if (epochOne == NULL && epochTwo != NULL)
-	return -1;
-    else if (epochOne != NULL && epochTwo != NULL) {
 /*@-boundsread@*/
-	if (*epochOne < *epochTwo)
-	    return -1;
-	else if (*epochOne > *epochTwo)
-	    return 1;
+    if (*epochOne < *epochTwo)
+	return -1;
+    else if (*epochOne > *epochTwo)
+	return 1;
 /*@=boundsread@*/
-    }
 
     rc = headerGetEntry(first, RPMTAG_VERSION, NULL, (void **) &one, NULL);
     rc = headerGetEntry(second, RPMTAG_VERSION, NULL, (void **) &two, NULL);
@@ -85,70 +81,6 @@ int rpmVersionCompare(Header first, Header second)
     return rpmvercmp(one, two);
 }
 
-/**
- * Macros to be defined from per-header tag values.
- * @todo Should other macros be added from header when installing a package?
- */
-/*@observer@*/ /*@unchecked@*/
-static struct tagMacro {
-/*@observer@*/ /*@null@*/ const char *	macroname; /*!< Macro name to define. */
-    rpmTag	tag;		/*!< Header tag to use for value. */
-} tagMacros[] = {
-    { "name",		RPMTAG_NAME },
-    { "version",	RPMTAG_VERSION },
-    { "release",	RPMTAG_RELEASE },
-    { "epoch",		RPMTAG_EPOCH },
-    { NULL, 0 }
-};
-
-/**
- * Define per-header macros.
- * @param fi		transaction element file info
- * @param h		header
- * @return		0 always
- */
-static int rpmInstallLoadMacros(rpmfi fi, Header h)
-	/*@globals rpmGlobalMacroContext @*/
-	/*@modifies rpmGlobalMacroContext @*/
-{
-    HGE_t hge = (HGE_t) fi->hge;
-    struct tagMacro * tagm;
-    union {
-/*@unused@*/ void * ptr;
-/*@unused@*/ const char ** argv;
-	const char * str;
-	int_32 * i32p;
-    } body;
-    char numbuf[32];
-    rpmTagType type;
-
-    for (tagm = tagMacros; tagm->macroname != NULL; tagm++) {
-	if (!hge(h, tagm->tag, &type, (void **) &body, NULL))
-	    continue;
-	switch (type) {
-	case RPM_INT32_TYPE:
-/*@-boundsread@*/
-	    sprintf(numbuf, "%d", *body.i32p);
-/*@=boundsread@*/
-	    addMacro(NULL, tagm->macroname, NULL, numbuf, -1);
-	    /*@switchbreak@*/ break;
-	case RPM_STRING_TYPE:
-	    addMacro(NULL, tagm->macroname, NULL, body.str, -1);
-	    /*@switchbreak@*/ break;
-	case RPM_NULL_TYPE:
-	case RPM_CHAR_TYPE:
-	case RPM_INT8_TYPE:
-	case RPM_INT16_TYPE:
-	case RPM_BIN_TYPE:
-	case RPM_STRING_ARRAY_TYPE:
-	case RPM_I18NSTRING_TYPE:
-	default:
-	    /*@switchbreak@*/ break;
-	}
-    }
-    return 0;
-}
-
 /**
  * Mark files in database shared with this package as "replaced".
  * @param psm		package state machine data
@@ -233,7 +165,7 @@ static rpmRC markReplacedFiles(const rpmpsm psm)
 rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
 		const char ** specFilePtr, const char ** cookie)
 {
-    int scareMem = 1;
+    int scareMem = 1;	/* XXX fi->h is needed */
     rpmfi fi = NULL;
     const char * _sourcedir = NULL;
     const char * _specdir = NULL;
@@ -291,14 +223,13 @@ rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
 	goto exit;
     }
 
-/*@-nullpass@*/		/* FIX fi->h may be null */
+assert(fi->h != NULL);
     fi->te->h = headerLink(fi->h);
-/*@=nullpass@*/
     fi->te->fd = fdLink(fd, "installSourcePackage");
     hge = fi->hge;
     hfd = fi->hfd;
 
-/*@i@*/ (void) rpmInstallLoadMacros(fi, fi->h);
+/*@i@*/ (void) headerMacrosLoad(fi->h);
 
     psm->fi = rpmfiLink(fi, NULL);
     /*@-assignexpose -usereleased @*/
@@ -348,6 +279,12 @@ rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
 	rpmrc = RPMRC_FAIL;
 	goto exit;
     }
+    if (Access(_sourcedir, W_OK)) {
+	rpmError(RPMERR_CREATE, _("cannot write to %%%s %s\n"),
+		"_sourcedir", _sourcedir);
+	rpmrc = RPMRC_FAIL;
+	goto exit;
+    }
 
     _specdir = rpmGenPath(rpmtsRootDir(ts), "%{_specdir}", "");
     rpmrc = rpmMkdirPath(_specdir, "specdir");
@@ -355,6 +292,12 @@ rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
 	rpmrc = RPMRC_FAIL;
 	goto exit;
     }
+    if (Access(_specdir, W_OK)) {
+	rpmError(RPMERR_CREATE, _("cannot write to %%%s %s\n"),
+		"_specdir", _specdir);
+	rpmrc = RPMRC_FAIL;
+	goto exit;
+    }
 
     /* Build dnl/dil with {_sourcedir, _specdir} as values. */
     if (i < fi->fc) {
@@ -521,11 +464,11 @@ static pid_t psmWait(rpmpsm psm)
 static rpmRC runLuaScript(rpmpsm psm, Header h, const char *sln,
 		   int progArgc, const char **progArgv,
 		   const char *script, int arg1, int arg2)
-	/*@globals fileSystem, internalState @*/
+	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies psm, fileSystem, internalState @*/
 {
     const rpmts ts = psm->ts;
-    int rootFd = -1;
+    int rootFdno = -1;
     const char *n, *v, *r;
     rpmRC rc = RPMRC_OK;
     int i;
@@ -541,21 +484,25 @@ static rpmRC runLuaScript(rpmpsm psm, Header h, const char *sln,
 
     xx = headerNVR(h, &n, &v, &r);
 
+    /* Save the current working directory. */
+/*@-nullpass@*/
+    rootFdno = open(".", O_RDONLY, 0);
+/*@=nullpass@*/
+
+    /* Get into the chroot. */
     if (!rpmtsChrootDone(ts)) {
 	const char *rootDir = rpmtsRootDir(ts);
-	xx = chdir("/");
-/*@-nullpass@*/
-	rootFd = open(".", O_RDONLY, 0);
-/*@=nullpass@*/
-	if (rootFd >= 0) {
-	    /*@-superuser -noeffect @*/
-	    if (rootDir != NULL && strcmp(rootDir, "/") && *rootDir == '/')
-		xx = chroot(rootDir);
-	    /*@=superuser =noeffect @*/
+	/*@-superuser -noeffect @*/
+	if (rootDir != NULL && strcmp(rootDir, "/") && *rootDir == '/') {
+	    xx = Chroot(rootDir);
+	/*@=superuser =noeffect @*/
 	    xx = rpmtsSetChrootDone(ts, 1);
 	}
     }
 
+    /* All lua scripts run with CWD == "/". */
+    xx = Chdir("/");
+
     /* Create arg variable */
     rpmluaPushTable(lua, "arg");
     var = rpmluavNew();
@@ -596,16 +543,20 @@ static rpmRC runLuaScript(rpmpsm psm, Header h, const char *sln,
 
     rpmluaDelVar(lua, "arg");
 
-    if (rootFd >= 0) {
+    /* Get out of chroot. */
+    if (rpmtsChrootDone(ts)) {
 	const char *rootDir = rpmtsRootDir(ts);
-	xx = fchdir(rootFd);
-	xx = close(rootFd);
+	xx = fchdir(rootFdno);
 	/*@-superuser -noeffect @*/
-	if (rootDir != NULL && strcmp(rootDir, "/") && *rootDir == '/')
-	    xx = chroot(".");
+	if (rootDir != NULL && strcmp(rootDir, "/") && *rootDir == '/') {
+	    xx = Chroot(".");
 	/*@=superuser =noeffect @*/
-	xx = rpmtsSetChrootDone(ts, 0);
-    }
+	    xx = rpmtsSetChrootDone(ts, 0);
+	}
+    } else
+	xx = fchdir(rootFdno);
+
+    xx = close(rootFdno);
 
     return rc;
 }
@@ -920,10 +871,10 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
 		!(rootDir[0] == '/' && rootDir[1] == '\0'))
 	    {
 		/*@-superuser -noeffect @*/
-		xx = chroot(rootDir);
+		xx = Chroot(rootDir);
 		/*@=superuser =noeffect @*/
 	    }
-	    xx = chdir("/");
+	    xx = Chdir("/");
 	    rpmMessage(RPMMESS_DEBUG, _("%s: %s(%s-%s-%s.%s)\texecv(%s) pid %d\n"),
 			psm->stepName, sln, n, v, r, a,
 			argv[0], (unsigned)getpid());
@@ -1017,18 +968,16 @@ static rpmRC runInstScript(rpmpsm psm)
     rpmfi fi = psm->fi;
     HGE_t hge = fi->hge;
     HFD_t hfd = (fi->hfd ? fi->hfd : headerFreeData);
-    void ** progArgv;
+    void ** progArgv = NULL;
     int progArgc;
+    const char * argv0 = NULL;
     const char ** argv;
     rpmTagType ptt, stt;
     const char * script;
     rpmRC rc = RPMRC_OK;
     int xx;
 
-    /*
-     * headerGetEntry() sets the data pointer to NULL if the entry does
-     * not exist.
-     */
+assert(fi->h != NULL);
     xx = hge(fi->h, psm->scriptTag, &stt, (void **) &script, NULL);
     xx = hge(fi->h, psm->progTag, &ptt, (void **) &progArgv, &progArgc);
     if (progArgv == NULL && script == NULL)
@@ -1043,11 +992,15 @@ static rpmRC runInstScript(rpmpsm psm)
     }
     /*@=branchstate@*/
 
+    if (argv[0][0] == '%')
+	argv[0] = argv0 = rpmExpand(argv[0], NULL);
+
     if (fi->h != NULL)	/* XXX can't happen */
     rc = runScript(psm, fi->h, tag2sln(psm->scriptTag), progArgc, argv,
 		script, psm->scriptArg, -1);
 
 exit:
+    argv0 = _free(argv0);
     progArgv = hfd(progArgv, ptt);
     script = hfd(script, stt);
     return rc;
@@ -1070,7 +1023,7 @@ static rpmRC handleOneTrigger(const rpmpsm psm,
 	/*@modifies psm, sourceH, triggeredH, *triggersAlreadyRun,
 		rpmGlobalMacroContext, fileSystem, internalState @*/
 {
-    int scareMem = 1;
+    int scareMem = 0;
     const rpmts ts = psm->ts;
     rpmfi fi = psm->fi;
     HGE_t hge = fi->hge;
@@ -1224,6 +1177,7 @@ static rpmRC runImmedTriggers(rpmpsm psm)
     int numTriggerIndices;
     unsigned char * triggersRun;
     rpmRC rc = RPMRC_OK;
+    size_t nb;
 
     if (fi->h == NULL)	return rc;	/* XXX can't happen */
 
@@ -1234,8 +1188,8 @@ static rpmRC runImmedTriggers(rpmpsm psm)
 	)
 	return rc;
 
-    triggersRun = alloca(sizeof(*triggersRun) * numTriggerIndices);
-    memset(triggersRun, 0, sizeof(*triggersRun) * numTriggerIndices);
+    nb = sizeof(*triggersRun) * numTriggerIndices;
+    triggersRun = memset(alloca(nb), 0, nb);
 
     {	Header sourceH = NULL;
 	int i;
@@ -1261,7 +1215,8 @@ static rpmRC runImmedTriggers(rpmpsm psm)
     return rc;
 }
 
-/*@observer@*/ static const char * pkgStageString(pkgStage a)
+/*@observer@*/
+static const char * pkgStageString(pkgStage a)
 	/*@*/
 {
     switch(a) {
@@ -1380,6 +1335,197 @@ rpmpsm rpmpsmNew(rpmts ts, rpmte te, rpmfi fi)
     return rpmpsmLink(psm, msg);
 }
 
+/**
+ * Load a transaction id from a header.
+ * @param h		header
+ * @param tag		tag to load
+ * @return		tag value (0 on failure)
+ */
+static uint_32 hLoadTID(Header h, int_32 tag)
+	/*@*/
+{
+    uint_32 val = 0;
+    int_32 typ = 0;
+    void * ptr = NULL;
+    int_32 cnt = 0;
+
+    if (headerGetEntry(h, tag, &typ, &ptr, &cnt)
+     && typ == RPM_INT32_TYPE && ptr != NULL && cnt == 1)
+	val = *(uint_32 *)ptr;
+    ptr = headerFreeData(ptr, typ);
+    return val;
+}
+
+/**
+ * Copy a tag from a source to a target header.
+ * @param sh		source header
+ * @param th		target header
+ * @param tag		tag to copy
+ * @return		0 always
+ */
+static int hCopyTag(Header sh, Header th, int_32 tag)
+	/*@modifies th @*/
+{
+    int_32 typ = 0;
+    void * ptr = NULL;
+    int_32 cnt = 0;
+    int xx = 1;
+
+    if (headerGetEntry(sh, tag, &typ, &ptr, &cnt) && cnt > 0)
+	xx = headerAddEntry(th, tag, typ, ptr, cnt);
+assert(xx);
+    ptr = headerFreeData(ptr, typ);
+    return 0;
+}
+
+/**
+ * Save backward link(s) of an upgrade chain into a header.
+ * @param h		header
+ * @param *blink	backward links
+ * @return		0 always
+ */
+static int hSaveBlinks(Header h, const struct rpmChainLink_s * blink)
+	/*@modifies h @*/
+{
+    /*@observer@*/
+    static const char * chain_end = RPMTE_CHAIN_END;
+    int ac;
+    int xx = 1;
+
+    ac = argvCount(blink->NEVRA);
+    if (ac > 0)
+	xx = headerAddEntry(h, RPMTAG_BLINKNEVRA, RPM_STRING_ARRAY_TYPE,
+			argvData(blink->NEVRA), ac);
+    else     /* XXX Add an explicit chain terminator on 1st install. */
+	xx = headerAddEntry(h, RPMTAG_BLINKNEVRA, RPM_STRING_ARRAY_TYPE,
+			&chain_end, 1);
+assert(xx);
+    
+    ac = argvCount(blink->Pkgid);
+    if (ac > 0) 
+	xx = headerAddEntry(h, RPMTAG_BLINKPKGID, RPM_STRING_ARRAY_TYPE,
+			argvData(blink->Pkgid), ac);
+    else     /* XXX Add an explicit chain terminator on 1st install. */
+	xx = headerAddEntry(h, RPMTAG_BLINKPKGID, RPM_STRING_ARRAY_TYPE,
+			&chain_end, 1);
+assert(xx);
+
+    ac = argvCount(blink->Hdrid);
+    if (ac > 0)
+	xx = headerAddEntry(h, RPMTAG_BLINKHDRID, RPM_STRING_ARRAY_TYPE,
+			argvData(blink->Hdrid), ac);
+    else     /* XXX Add an explicit chain terminator on 1st install. */
+	xx = headerAddEntry(h, RPMTAG_BLINKNEVRA, RPM_STRING_ARRAY_TYPE,
+			&chain_end, 1);
+assert(xx);
+
+    return 0;
+}
+
+/**
+ * Save forward link(s) of an upgrade chain into a header.
+ * @param h		header
+ * @param *flink	forward links
+ * @return		0 always
+ */
+static int hSaveFlinks(Header h, const struct rpmChainLink_s * flink)
+	/*@modifies h @*/
+{
+#ifdef	NOTYET
+    /*@observer@*/
+    static const char * chain_end = RPMTE_CHAIN_END;
+#endif
+    int ac;
+    int xx = 1;
+
+    /* Save backward links into header upgrade chain. */
+    ac = argvCount(flink->NEVRA);
+    if (ac > 0)
+	xx = headerAddEntry(h, RPMTAG_FLINKNEVRA, RPM_STRING_ARRAY_TYPE,
+			argvData(flink->NEVRA), ac);
+#ifdef	NOTYET	/* XXX is an explicit flink terminator needed? */
+    else     /* XXX Add an explicit chain terminator on 1st install. */
+	xx = headerAddEntry(h, RPMTAG_FLINKNEVRA, RPM_STRING_ARRAY_TYPE,
+			&chain_end, 1);
+#endif
+assert(xx);
+
+    ac = argvCount(flink->Pkgid);
+    if (ac > 0) 
+	xx = headerAddEntry(h, RPMTAG_FLINKPKGID, RPM_STRING_ARRAY_TYPE,
+			argvData(flink->Pkgid), ac);
+#ifdef	NOTYET	/* XXX is an explicit flink terminator needed? */
+    else     /* XXX Add an explicit chain terminator on 1st install. */
+	xx = headerAddEntry(h, RPMTAG_FLINKPKGID, RPM_STRING_ARRAY_TYPE,
+			&chain_end, 1);
+#endif
+assert(xx);
+
+    ac = argvCount(flink->Hdrid);
+    if (ac > 0)
+	xx = headerAddEntry(h, RPMTAG_FLINKHDRID, RPM_STRING_ARRAY_TYPE,
+			argvData(flink->Hdrid), ac);
+#ifdef	NOTYET	/* XXX is an explicit flink terminator needed? */
+    else     /* XXX Add an explicit chain terminator on 1st install. */
+	xx = headerAddEntry(h, RPMTAG_FLINKNEVRA, RPM_STRING_ARRAY_TYPE,
+			&chain_end, 1);
+#endif
+assert(xx);
+
+    return 0;
+}
+
+/**
+ * Add per-transaction data to an install header.
+ * @param ts		transaction set
+ * @param te		transaction element
+ * @param fi		file info set
+ * @return		0 always
+ */
+static int populateInstallHeader(const rpmts ts, const rpmte te, rpmfi fi)
+	/*@modifies fi @*/
+{
+    uint_32 tscolor = rpmtsColor(ts);
+    uint_32 tecolor = rpmteColor(te);
+    int_32 installTime = (int_32) time(NULL);
+    int fc = rpmfiFC(fi);
+    const char * origin;
+    int xx = 1;
+
+assert(fi->h != NULL);
+    if (fi->fstates != NULL && fc > 0)
+	xx = headerAddEntry(fi->h, RPMTAG_FILESTATES, RPM_CHAR_TYPE,
+				fi->fstates, fc);
+assert(xx);
+
+    xx = headerAddEntry(fi->h, RPMTAG_INSTALLTIME, RPM_INT32_TYPE,
+			&installTime, 1);
+assert(xx);
+
+    xx = headerAddEntry(fi->h, RPMTAG_INSTALLCOLOR, RPM_INT32_TYPE,
+			&tscolor, 1);
+assert(xx);
+
+    /* XXX FIXME: add preferred color at install. */
+
+    xx = headerAddEntry(fi->h, RPMTAG_PACKAGECOLOR, RPM_INT32_TYPE,
+				&tecolor, 1);
+assert(xx);
+
+    /* Add the header's origin (i.e. URL) */
+    origin = headerGetOrigin(fi->h);
+    if (origin != NULL)
+	xx = headerAddEntry(fi->h, RPMTAG_PACKAGEORIGIN, RPM_STRING_TYPE,
+				origin, 1);
+assert(xx);
+
+    /* XXX Don't clobber forward/backward upgrade chain on rollbacks */
+    if (rpmtsType(ts) != RPMTRANS_TYPE_ROLLBACK)
+	xx = hSaveBlinks(fi->h, &te->blink);
+
+    return 0;
+}
+
 static void * rpmpsmThread(void * arg)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies arg, rpmGlobalMacroContext, fileSystem, internalState @*/
@@ -1450,6 +1596,9 @@ assert(psm->te != NULL);
 	    int fc = rpmfiFC(fi);
 	    const char * hdrid;
 
+	    /* Add per-transaction data to install header. */
+	    xx = populateInstallHeader(ts, psm->te, fi);
+
 	    psm->scriptArg = psm->npkgs_installed + 1;
 
 assert(psm->mi == NULL);
@@ -1527,17 +1676,27 @@ psm->te->h = headerLink(fi->h);
 	}
 	if (psm->goal == PSM_PKGSAVE) {
 	    /* Open output package for writing. */
-	    {	const char * bfmt = rpmGetPath("%{_repackage_name_fmt}", NULL);
-		const char * pkgbn =
-			headerSprintf(fi->h, bfmt, rpmTagTable, rpmHeaderFormats, NULL);
+	    {	char tiddn[32];
+		const char * bfmt;
+		const char * pkgdn;
+		const char * pkgbn;
 
+		xx = snprintf(tiddn, sizeof(tiddn), "%d", rpmtsGetTid(ts));
+		bfmt = rpmGetPath(tiddn, "/", "%{_repackage_name_fmt}", NULL);
+		pkgbn = headerSprintf(fi->h, bfmt,
+					rpmTagTable, rpmHeaderFormats, NULL);
 		bfmt = _free(bfmt);
 		psm->pkgURL = rpmGenPath("%{?_repackage_root}",
 					 "%{?_repackage_dir}",
 					pkgbn);
 		pkgbn = _free(pkgbn);
 		(void) urlPath(psm->pkgURL, &psm->pkgfn);
-		psm->fd = Fopen(psm->pkgfn, "w.ufdio");
+		pkgdn = dirname(xstrdup(psm->pkgfn));
+		rc = rpmMkdirPath(pkgdn, "_repackage_dir");
+		pkgdn = _free(pkgdn);
+		if (rc == RPMRC_FAIL)
+		    break;
+		psm->fd = Fopen(psm->pkgfn, "w");
 		if (psm->fd == NULL || Ferror(psm->fd)) {
 		    rc = RPMRC_FAIL;
 		    break;
@@ -1665,22 +1824,17 @@ psm->te->h = headerLink(fi->h);
 	    /*@=nullstate@*/
 
 	    /* Write the lead section into the package. */
-	    {	int archnum = -1;
-		int osnum = -1;
-		struct rpmlead lead;
-
-#ifndef	DYING
-		rpmGetArchInfo(NULL, &archnum);
-		rpmGetOsInfo(NULL, &osnum);
-#endif
+	    {	struct rpmlead lead;
 
 		memset(&lead, 0, sizeof(lead));
 		/* XXX Set package version conditioned on noDirTokens. */
 		lead.major = 3;
 		lead.minor = 0;
 		lead.type = RPMLEAD_BINARY;
-		lead.archnum = archnum;
-		lead.osnum = osnum;
+
+		/* XXX DIEDIEDIE: legacy values were not 0. */
+		lead.archnum = 0;
+		lead.osnum = 0;
 		lead.signature_type = RPMSIGTYPE_HEADERSIG;
 
 		strncpy(lead.name, rpmteNEVR(psm->te), sizeof(lead.name));
@@ -1711,7 +1865,6 @@ psm->te->h = headerLink(fi->h);
 	    /* Add remove transaction id to header. */
 	    if (psm->oh != NULL)
 	    {	int_32 tid = rpmtsGetTid(ts);
-		int ac;
 
 		xx = hae(psm->oh, RPMTAG_REMOVETID, RPM_INT32_TYPE,
 				&tid, 1);
@@ -1721,20 +1874,14 @@ psm->te->h = headerLink(fi->h);
 		    xx = hae(psm->oh, RPMTAG_PACKAGEORIGIN, RPM_STRING_TYPE,
 				origin, 1);
 
-assert(psm->te != NULL);
-		ac = argvCount(psm->te->flink.NEVRA);
-		if (ac > 0)
-		    xx = hae(psm->oh, RPMTAG_FLINKNEVRA, RPM_STRING_ARRAY_TYPE,
-				argvData(psm->te->flink.NEVRA), ac);
-		ac = argvCount(psm->te->flink.Pkgid);
-		if (ac > 0)
-		    xx = hae(psm->oh, RPMTAG_FLINKPKGID, RPM_STRING_ARRAY_TYPE,
-				argvData(psm->te->flink.Pkgid), ac);
-		ac = argvCount(psm->te->flink.Hdrid);
-		if (ac > 0)
-		    xx = hae(psm->oh, RPMTAG_FLINKHDRID, RPM_STRING_ARRAY_TYPE,
-				argvData(psm->te->flink.Hdrid), ac);
+		/* Copy upgrade chain link tags. */
+		xx = hCopyTag(fi->h, psm->oh, RPMTAG_INSTALLTID);
+		xx = hCopyTag(fi->h, psm->oh, RPMTAG_BLINKPKGID);
+		xx = hCopyTag(fi->h, psm->oh, RPMTAG_BLINKHDRID);
+		xx = hCopyTag(fi->h, psm->oh, RPMTAG_BLINKNEVRA);
 
+assert(psm->te != NULL);
+		xx = hSaveFlinks(psm->oh, &psm->te->flink);
 	    }
 
 	    /* Write the metadata section into the package. */
@@ -1885,45 +2032,10 @@ assert(psm->te != NULL);
 	if (rpmtsFlags(ts) & RPMTRANS_FLAG_TEST)	break;
 
 	if (psm->goal == PSM_PKGINSTALL) {
-	    int_32 installTime = (int_32) time(NULL);
-	    int fc = rpmfiFC(fi);
-	    const char * origin;
-	    int ac;
-
-	    if (fi->h == NULL) break;	/* XXX can't happen */
-	    if (fi->fstates != NULL && fc > 0)
-		xx = hae(fi->h, RPMTAG_FILESTATES, RPM_CHAR_TYPE,
-				fi->fstates, fc);
-
-	    xx = hae(fi->h, RPMTAG_INSTALLTIME, RPM_INT32_TYPE,
-				&installTime, 1);
-
-	    xx = hae(fi->h, RPMTAG_INSTALLCOLOR, RPM_INT32_TYPE,
-				&tscolor, 1);
-
-	    /* Add the header's origin (i.e. URL) */
-	    origin = headerGetOrigin(fi->h);
-	    if (origin != NULL)
-		xx = hae(fi->h, RPMTAG_PACKAGEORIGIN, RPM_STRING_TYPE,
-				origin, 1);
-	
-assert(psm->te != NULL);
-	    ac = argvCount(psm->te->blink.NEVRA);
-	    if (ac > 0)
-		xx = hae(fi->h, RPMTAG_BLINKNEVRA, RPM_STRING_ARRAY_TYPE,
-				argvData(psm->te->blink.NEVRA), ac);
-	    ac = argvCount(psm->te->blink.Pkgid);
-	    if (ac > 0)
-		xx = hae(fi->h, RPMTAG_BLINKPKGID, RPM_STRING_ARRAY_TYPE,
-				argvData(psm->te->blink.Pkgid), ac);
-	    ac = argvCount(psm->te->blink.Hdrid);
-	    if (ac > 0)
-		xx = hae(fi->h, RPMTAG_BLINKHDRID, RPM_STRING_ARRAY_TYPE,
-				argvData(psm->te->blink.Hdrid), ac);
 
 	    /*
-	     * If this package has already been installed, remove it from
-	     * the database before adding the new one.
+	     * If this header has already been installed, remove it from
+	     * the database before adding the new header.
 	     */
 	    if (fi->record && !(rpmtsFlags(ts) & RPMTRANS_FLAG_APPLYONLY)) {
 		rc = rpmpsmNext(psm, PSM_RPMDB_REMOVE);
@@ -1972,6 +2084,10 @@ assert(psm->te != NULL);
 		/* Run triggers in other package(s) this package sets off. */
 		rc = rpmpsmNext(psm, PSM_TRIGGERS);
 		if (rc) break;
+
+		/* Run triggers in this package other package(s) set off. */
+		rc = rpmpsmNext(psm, PSM_IMMED_TRIGGERS);
+		if (rc) break;
 	    }
 
 	    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_APPLYONLY))
@@ -2099,10 +2215,10 @@ psm->te->h = headerFree(psm->te->h);
 		_gr_loaded++;
 	    }
 
-	    xx = chdir("/");
+	    xx = Chdir("/");
 	    /*@-superuser@*/
 	    if (rootDir != NULL && strcmp(rootDir, "/") && *rootDir == '/')
-		rc = chroot(rootDir);
+		rc = Chroot(rootDir);
 	    /*@=superuser@*/
 	    psm->chrootDone = 1;
 	    (void) rpmtsSetChrootDone(ts, 1);
@@ -2115,12 +2231,12 @@ psm->te->h = headerFree(psm->te->h);
 	    const char * currDir = rpmtsCurrDir(ts);
 	    /*@-superuser@*/
 	    if (rootDir != NULL && strcmp(rootDir, "/") && *rootDir == '/')
-		rc = chroot(".");
+		rc = Chroot(".");
 	    /*@=superuser@*/
 	    psm->chrootDone = 0;
 	    (void) rpmtsSetChrootDone(ts, 0);
 	    if (currDir != NULL)	/* XXX can't happen */
-		xx = chdir(currDir);
+		xx = Chdir(currDir);
 	}
 	break;
     case PSM_SCRIPT:	/* Run current package scriptlets. */
@@ -2154,6 +2270,8 @@ psm->te->h = headerFree(psm->te->h);
 	    t = stpcpy(t, ".gzdio");
 	if (!strcmp(payload_compressor, "bzip2"))
 	    t = stpcpy(t, ".bzdio");
+	if (!strcmp(payload_compressor, "lzma"))
+	    t = stpcpy(t, ".lzdio");
 
 	/*@-branchstate@*/
 	if (!hge(fi->h, RPMTAG_PAYLOADFORMAT, NULL,
@@ -2185,25 +2303,23 @@ assert(psm->mi == NULL);
 	if (fi->h == NULL)	break;	/* XXX can't happen */
 
 	/* Add header to db, doing header check if requested */
-	(void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_DBADD), 0);
-	if (!(rpmtsVSFlags(ts) & RPMVSF_NOHDRCHK))
-	    rc = rpmdbAdd(rpmtsGetRdb(ts), rpmtsGetTid(ts), fi->h,
-				ts, headerCheck);
-	else
-	    rc = rpmdbAdd(rpmtsGetRdb(ts), rpmtsGetTid(ts), fi->h,
-				NULL, NULL);
-	(void) rpmswExit(rpmtsOp(ts, RPMTS_OP_DBADD), 0);
+	/* XXX rollback headers propagate the previous transaction id. */
+	{   uint_32 tid = ((rpmtsType(ts) == RPMTRANS_TYPE_ROLLBACK)
+		? hLoadTID(fi->h, RPMTAG_INSTALLTID) : rpmtsGetTid(ts));
+	    (void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_DBADD), 0);
+	    if (!(rpmtsVSFlags(ts) & RPMVSF_NOHDRCHK))
+		rc = rpmdbAdd(rpmtsGetRdb(ts), tid, fi->h, ts, headerCheck);
+	    else
+		rc = rpmdbAdd(rpmtsGetRdb(ts), tid, fi->h, NULL, NULL);
+	    (void) rpmswExit(rpmtsOp(ts, RPMTS_OP_DBADD), 0);
+	}
 
 	if (rc != RPMRC_OK) break;
 
 assert(psm->te != NULL);
 	/* Mark non-rollback elements as installed. */
-	if (
-	    rpmtsType(ts) != RPMTRANS_TYPE_ROLLBACK &&
-	    rpmtsType(ts) != RPMTRANS_TYPE_AUTOROLLBACK)
-	{
+	if (rpmtsType(ts) != RPMTRANS_TYPE_ROLLBACK)
 	    psm->te->installed = 1;
-	}
 
 	/* Set the database instance for (possible) rollbacks. */
 	rpmteSetDBInstance(psm->te, headerGetInstance(fi->h));
@@ -2211,7 +2327,6 @@ assert(psm->te != NULL);
 	break;
     case PSM_RPMDB_REMOVE:
     {	
-
 	if (rpmtsFlags(ts) & RPMTRANS_FLAG_TEST)	break;
 
 	(void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_DBREMOVE), 0);
@@ -2221,6 +2336,9 @@ assert(psm->te != NULL);
 
 	if (rc != RPMRC_OK) break;
 
+	/* Forget the offset of a successfully removed header. */
+	psm->te->u.removed.dboffset = 0;
+
     }	break;
 
     default:
