@@ -7,6 +7,7 @@
 #include <rpmlib.h>
 
 #include "rpmal.h"
+#define	_RPMDS_INTERNAL
 #include "rpmds.h"
 #include "rpmfi.h"
 
@@ -20,6 +21,7 @@ int _rpmal_debug = 0;
 /*@access alKey @*/
 /*@access alNum @*/
 /*@access rpmal @*/
+/*@access rpmds @*/
 /*@access availablePackage @*/
 
 /*@access fnpyKey @*/	/* XXX suggestedKeys array */
@@ -357,7 +359,7 @@ fprintf(stderr, "--- die[%5d] %p [%3d] %s\n", (int)(die - al->dirs), die, die->d
 		if (i < die->numFiles) {
 /*@-modfilesys@*/
 if (_rpmal_debug)
-fprintf(stderr, "\t%p[%3d] memmove(%p:%p,%p:%p,0x%x) %s <- %s\n", die->files, die->numFiles, fie, fie->baseName, fie+1, (fie+1)->baseName, ((die->numFiles - i) * sizeof(*fie)), fie->baseName, (fie+1)->baseName);
+fprintf(stderr, "\t%p[%3d] memmove(%p:%p,%p:%p,0x%x) %s <- %s\n", die->files, die->numFiles, fie, fie->baseName, fie+1, (fie+1)->baseName, (unsigned) ((die->numFiles - i) * sizeof(*fie)), fie->baseName, (fie+1)->baseName);
 /*@=modfilesys@*/
 
 /*@-bounds@*/
@@ -639,7 +641,7 @@ void rpmalAddProvides(rpmal al, alKey pkgKey, rpmds provides, uint_32 tscolor)
     if (rpmdsInit(provides) != NULL)
     while (rpmdsNext(provides) >= 0) {
 
-	if ((Name = rpmdsN(provides)) == NULL)
+	if ((Name = provides->N[provides->i]) == NULL)
 	    continue;	/* XXX can't happen */
 
 	/* Ignore colored provides not in our rainbow. */
@@ -651,7 +653,9 @@ void rpmalAddProvides(rpmal al, alKey pkgKey, rpmds provides, uint_32 tscolor)
 	ai->k++;
 
 	aie->pkgKey = pkgKey;
+/*@-assignexpose@*/
 	aie->entry = Name;
+/*@=assignexpose@*/
 	aie->entryLen = strlen(Name);
 	ix = rpmdsIx(provides);
 
@@ -815,6 +819,7 @@ rpmalAllSatisfiesDepend(const rpmal al, const rpmds ds, alKey * keyp)
 	ret = rpmalAllFileSatisfiesDepend(al, ds, keyp);
 	if (ret != NULL && *ret != NULL)
 	    return ret;
+	ret = _free(ret);
 	/* ... then, look for files "provided" by package. */
     }
 
