@@ -7,6 +7,8 @@
 
 #include "rpmio_internal.h"
 #include <rpmcli.h>
+#define	_RPMEVR_INTERNAL	/* XXX RPMSENSE_KEYRING */
+#include <rpmevr.h>
 
 #include "rpmdb.h"
 
@@ -50,7 +52,7 @@ static int manageFile(/*@out@*/ FD_t *fdp,
 
     /* open a file and set *fdp */
     if (*fdp == NULL && fnp != NULL && *fnp != NULL) {
-	fd = Fopen(*fnp, ((flags & O_WRONLY) ? "w.ufdio" : "r.ufdio"));
+	fd = Fopen(*fnp, ((flags & O_WRONLY) ? "w" : "r"));
 	if (fd == NULL || Ferror(fd)) {
 	    rpmError(RPMERR_OPEN, _("%s: open failed: %s\n"), *fnp,
 		Fstrerror(fd));
@@ -116,6 +118,11 @@ static int copyFile(FD_t *sfdp, const char **sfnp,
 	rpmError(RPMERR_FREAD, _("%s: Fread failed: %s\n"), *sfnp, Fstrerror(*sfdp));
 	goto exit;
     }
+    if (Fflush(*tfdp) != 0) {
+	rpmError(RPMERR_FWRITE, _("%s: Fflush failed: %s\n"), *tfnp,
+	    Fstrerror(*tfdp));
+	goto exit;
+    }
 
     rc = 0;
 
@@ -1086,7 +1093,7 @@ int rpmcliSign(rpmts ts, QVA_t qva, const char ** argv)
     while ((arg = *argv++) != NULL) {
 	FD_t fd;
 
-	if ((fd = Fopen(arg, "r.ufdio")) == NULL
+	if ((fd = Fopen(arg, "r")) == NULL
 	 || Ferror(fd)
 	 || rpmVerifySignatures(qva, ts, fd, arg))
 	    res++;
