@@ -8,6 +8,7 @@
 #include "rpmlib.h"
 #include "rpmurl.h"
 #include "rpmmacro.h"
+#include "rpmps.h"
 #include "argv.h"
 
 /** \ingroup rpmcli
@@ -34,6 +35,12 @@ extern int ftsOpts;
 /*@unchecked@*/
 extern struct poptOption		rpmcliFtsPoptTable[];
 
+/*@unchecked@*/
+extern int global_depFlags;
+
+/*@unchecked@*/
+extern struct poptOption		rpmcliDepFlagsPoptTable[];
+
 /*@unchecked@*/ /*@observer@*/ /*@null@*/
 extern const char * rpmcliPipeOutput;
 
@@ -43,6 +50,10 @@ extern const char * rpmcliRcfile;
 /*@unchecked@*/ /*@observer@*/ /*@null@*/
 extern const char * rpmcliRootDir;
 
+/*@unchecked@*/ /*@observer@*/ /*@null@*/
+extern const char * rpmcliTargets;
+/*@=redecl@*/
+
 /**
  * Initialize most everything needed by an rpm CLI executable context.
  * @param argc			no. of args
@@ -53,7 +64,7 @@ extern const char * rpmcliRootDir;
 /*@null@*/
 poptContext
 rpmcliInit(int argc, char *const argv[], struct poptOption * optionsTable)
-	/*@globals rpmCLIMacroContext, rpmGlobalMacroContext, h_errno, stderr, 
+	/*@globals rpmRcfiles, rpmCLIMacroContext, rpmGlobalMacroContext, h_errno, stderr, 
 		fileSystem, internalState @*/
 	/*@modifies rpmCLIMacroContext, rpmGlobalMacroContext, stderr, 
 		fileSystem, internalState @*/;
@@ -64,8 +75,8 @@ rpmcliInit(int argc, char *const argv[], struct poptOption * optionsTable)
  */
 /*@mayexit@*/
 void rpmcliConfigured(void)
-	/*@globals rpmCLIMacroContext, rpmGlobalMacroContext, h_errno,
-		fileSystem, internalState @*/
+	/*@globals rpmRcfiles, rpmCLIMacroContext,
+		rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies rpmCLIMacroContext, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
 
@@ -76,7 +87,10 @@ void rpmcliConfigured(void)
  */
 poptContext
 rpmcliFini(/*@only@*/ /*@null@*/ poptContext optCon)
-	/*@modifies optCon @*/;
+	/*@globals rpmTags, rpmGlobalMacroContext,
+		fileSystem, internalState @*/
+	/*@modifies optCon, rpmTags, rpmGlobalMacroContext,
+		fileSystem, internalState @*/;
 
 /**
  * Common/global popt tokens used for command line option tables.
@@ -89,6 +103,7 @@ rpmcliFini(/*@only@*/ /*@null@*/ poptContext optCon)
 #define	RPMCLI_POPT_NODIGEST		-1030
 #define	RPMCLI_POPT_NOHDRCHK		-1031
 #define	RPMCLI_POPT_NOCONTEXTS		-1032
+#define	RPMCLI_POPT_TARGETPLATFORM	-1033
 
 /* ==================================================================== */
 /** \name RPMQV */
@@ -115,7 +130,8 @@ typedef enum rpmQVSources_e {
     RPMQV_TID,		/*!< ... from install transaction id (time stamp). */
     RPMQV_HDLIST,	/*!< ... from system hdlist. */
     RPMQV_FTSWALK,	/*!< ... from fts(3) walk. */
-    RPMQV_WHATNEEDS	/*!< ... from requires using contained provides. */
+    RPMQV_WHATNEEDS,	/*!< ... from requires using contained provides. */
+    RPMQV_SPECSRPM	/*!< ... srpm from spec file parse (query only). */
 } rpmQVSources;
 
 /** \ingroup rpmcli
@@ -259,55 +275,6 @@ typedef	int (*QSpecF_t) (rpmts ts, QVA_t qva, const char * arg)
 	/*@modifies ts, qva, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
 
-/** \ingroup rpmcli
- * Describe query/verify/signature command line operation.
- */
-#if !defined(SWIG)
-struct rpmQVKArguments_s {
-    rpmQVSources qva_source;	/*!< Identify CLI arg type. */
-    int 	qva_sourceCount;/*!< Exclusive option check (>1 is error). */
-    rpmQueryFlags qva_flags;	/*!< Bit(s) to control operation. */
-    rpmfileAttrs qva_fflags;	/*!< Bit(s) to filter on attribute. */
-/*@only@*/ /*@null@*/
-    rpmdbMatchIterator qva_mi;	/*!< Match iterator on selected headers. */
-/*@refccounted@*/ /*@relnull@*/
-    rpmgi qva_gi;		/*!< Generalized iterator on args. */
-    rpmRC qva_rc;		/*!< Current return code. */
-
-/*@null@*/
-    QVF_t qva_showPackage;	/*!< Function to display iterator matches. */
-/*@null@*/
-    QSpecF_t qva_specQuery;	/*!< Function to query spec file. */
-/*@unused@*/
-    int qva_verbose;		/*!< (unused) */
-/*@only@*/ /*@null@*/
-    const char * qva_queryFormat;/*!< Format for headerSprintf(). */
-    int sign;			/*!< Is a passphrase needed? */
-    int trust;			/*!< Trust metric when importing pubkeys. */
-/*@observer@*/
-    const char * passPhrase;	/*!< Pass phrase. */
-/*@observer@*/ /*@null@*/
-    const char * qva_prefix;	/*!< Path to top of install tree. */
-    char qva_mode;
-		/*!<
-		- 'q'	from --query, -q
-		- 'Q'	from --querytags
-		- 'V'	from --verify, -V
-		- 'A'	from --addsign
-		- 'I'	from --import
-		- 'K'	from --checksig, -K
-		- 'R'	from --resign
-		*/
-    char qva_char;		/*!< (unused) always ' ' */
-    char * targets;		/*!< Target platform(s), comma separated. */
-};
-#endif
-
-/** \ingroup rpmcli
- */
-/*@unchecked@*/
-extern struct rpmQVKArguments_s rpmQVKArgs;
-
 /** \ingroup rpmcli
  */
 /*@unchecked@*/
@@ -463,58 +430,29 @@ int rpmcliVerify(rpmts ts, QVA_t qva, /*@null@*/ const char ** argv)
 	/*@modifies ts, qva, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
 
-/*@}*/
-/* ==================================================================== */
-/** \name RPMBT */
-/*@{*/
-
-/** \ingroup rpmcli
- * Describe build command line request.
- */
-struct rpmBuildArguments_s {
-    rpmQueryFlags qva_flags;	/*!< Bit(s) to control verification. */
-    int buildAmount;		/*!< Bit(s) to control operation. */
-/*@null@*/
-    const char * buildRootOverride; /*!< from --buildroot */
-/*@null@*/
-    char * targets;		/*!< Target platform(s), comma separated. */
-/*@observer@*/
-    const char * passPhrase;	/*!< Pass phrase. */
-/*@only@*/ /*@null@*/
-    const char * cookie;	/*!< NULL for binary, ??? for source, rpm's */
-    int force;			/*!< from --force */
-    int noBuild;		/*!< from --nobuild */
-    int noDeps;			/*!< from --nodeps */
-    int noLang;			/*!< from --nolang */
-    int shortCircuit;		/*!< from --short-circuit */
-    int sign;			/*!< from --sign */
-    int useCatalog;		/*!< from --usecatalog */
-    char buildMode;		/*!< Build mode (one of "btBC") */
-    char buildChar;		/*!< Build stage (one of "abcilps ") */
-/*@observer@*/ /*@null@*/
-    const char * rootdir;
-};
-
-/** \ingroup rpmcli
- */
-typedef	struct rpmBuildArguments_s *	BTA_t;
-
-/** \ingroup rpmcli
- */
-/*@unchecked@*/
-extern struct rpmBuildArguments_s	rpmBTArgs;
-
-/** \ingroup rpmcli
- */
-/*@unchecked@*/
-extern struct poptOption		rpmBuildPoptTable[];
-
 /*@}*/
 /* ==================================================================== */
 /** \name RPMEIU */
 /*@{*/
 /* --- install/upgrade/erase modes */
 
+/** \ingroup rpmcli
+ * Bit(s) to control rpmInstall() and rpmErase() operation.
+ */
+typedef enum rpmInstallInterfaceFlags_e {
+    INSTALL_NONE	= 0,
+    INSTALL_PERCENT	= (1 <<  0),	/*!< from --percent */
+    INSTALL_HASH	= (1 <<  1),	/*!< from --hash */
+    INSTALL_NODEPS	= (1 <<  2),	/*!< from --nodeps */
+    INSTALL_NOORDER	= (1 <<  3),	/*!< from --noorder */
+    INSTALL_LABEL	= (1 <<  4),	/*!< from --verbose (notify) */
+    INSTALL_UPGRADE	= (1 <<  5),	/*!< from --upgrade */
+    INSTALL_FRESHEN	= (1 <<  6),	/*!< from --freshen */
+    INSTALL_INSTALL	= (1 <<  7),	/*!< from --install */
+    INSTALL_ERASE	= (1 <<  8),	/*!< from --erase */
+    INSTALL_ALLMATCHES	= (1 <<  9)	/*!< from --allmatches (erase) */
+} rpmInstallInterfaceFlags;
+
 /*@unchecked@*/
 extern int rpmcliPackagesTotal;
 /*@unchecked@*/
@@ -572,39 +510,15 @@ int rpmInstallSource(rpmts ts, const char * arg,
 	/*@modifies ts, *specFilePtr, *cookie, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
 
-/** \ingroup rpmcli
- * Describe database command line requests.
- */
-struct rpmInstallArguments_s {
-    rpmtransFlags transFlags;
-    rpmprobFilterFlags probFilter;
-    rpmInstallInterfaceFlags installInterfaceFlags;
-    rpmEraseInterfaceFlags eraseInterfaceFlags;
-    rpmQueryFlags qva_flags;	/*!< from --nodigest/--nosignature */
-    uint_32 arbtid;		/*!< from --arbgoal */
-    uint_32 rbtid;		/*!< from --rollback */
-    uint_32 *rbtidExcludes;	/*!< from --rollback */
-    int numrbtidExcludes;	/*!< from --rollback */
-    int numRelocations;
-    int noDeps;
-    int incldocs;
-/*@owned@*/ /*@null@*/
-    rpmRelocation relocations;
-/*@null@*/
-    const char * prefix;
-/*@observer@*/ /*@null@*/
-    const char * rootdir;
-};
-
 /** \ingroup rpmcli
  * Install/upgrade/freshen binary rpm package.
+ * @todo Use rpmdsCompare rather than rpmVersionCompare.
  * @param ts		transaction set
  * @param ia		mode flags and parameters
  * @param fileArgv	array of package file names (NULL terminated)
  * @return		0 on success
  */
-int rpmInstall(rpmts ts, struct rpmInstallArguments_s * ia,
-		/*@null@*/ const char ** fileArgv)
+int rpmInstall(rpmts ts, QVA_t ia, /*@null@*/ const char ** fileArgv)
 	/*@globals rpmcliPackagesTotal, rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState@*/
 	/*@modifies ts, ia, rpmcliPackagesTotal, rpmGlobalMacroContext,
@@ -617,8 +531,7 @@ int rpmInstall(rpmts ts, struct rpmInstallArguments_s * ia,
  * @param argv		array of package file names (NULL terminated)
  * @return		0 on success
  */
-int rpmErase(rpmts ts, struct rpmInstallArguments_s * ia,
-		/*@null@*/ const char ** argv)
+int rpmErase(rpmts ts, QVA_t ia, /*@null@*/ const char ** argv)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, ia, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
@@ -725,18 +638,12 @@ IDTX IDTXglob(rpmts ts, const char * globstr, rpmTag tag, uint_32 rbtid)
  * @param argv		array of arguments (NULL terminated)
  * @return		0 on success
  */
-int rpmRollback(rpmts ts, struct rpmInstallArguments_s * ia,
-		/*@null@*/ const char ** argv)
+int rpmRollback(rpmts ts, QVA_t ia, /*@null@*/ const char ** argv)
 	/*@globals rpmcliPackagesTotal, rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState @*/
 	/*@modifies ts, ia, rpmcliPackagesTotal, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
 
-/** \ingroup rpmcli
- */
-/*@unchecked@*/
-extern struct rpmInstallArguments_s rpmIArgs;
-
 /** \ingroup rpmcli
  */
 /*@unchecked@*/
@@ -746,21 +653,6 @@ extern struct poptOption rpmInstallPoptTable[];
 /* ==================================================================== */
 /** \name RPMDB */
 /*@{*/
-/* --- database modes */
-
-/** \ingroup rpmcli
- * Describe database command line requests.
- */
-struct rpmDatabaseArguments_s {
-    int init;			/*!< from --initdb */
-    int rebuild;		/*!< from --rebuilddb */
-    int verify;			/*!< from --verifydb */
-};
-
-/** \ingroup rpmcli
- */
-/*@unchecked@*/
-extern struct rpmDatabaseArguments_s rpmDBArgs;
 
 /** \ingroup rpmcli
  */
@@ -823,6 +715,135 @@ int rpmcliSign(rpmts ts, QVA_t qva, /*@null@*/ const char ** argv)
 
 /*@}*/
 
+/** \ingroup rpmcli
+ * Command line option information.
+ */
+#if !defined(SWIG)
+struct rpmQVKArguments_s {
+    rpmQVSources qva_source;	/*!< Identify CLI arg type. */
+    int 	qva_sourceCount;/*!< Exclusive option check (>1 is error). */
+    rpmQueryFlags qva_flags;	/*!< Bit(s) to control operation. */
+    rpmfileAttrs qva_fflags;	/*!< Bit(s) to filter on attribute. */
+/*@only@*/ /*@null@*/
+    rpmdbMatchIterator qva_mi;	/*!< Match iterator on selected headers. */
+/*@refccounted@*/ /*@relnull@*/
+    rpmgi qva_gi;		/*!< Generalized iterator on args. */
+    rpmRC qva_rc;		/*!< Current return code. */
+
+/*@null@*/
+    QVF_t qva_showPackage;	/*!< Function to display iterator matches. */
+    int qva_showOK;		/*!< No. of successes. */
+    int qva_showFAIL;		/*!< No. of failures. */
+/*@null@*/
+    QSpecF_t qva_specQuery;	/*!< Function to query spec file. */
+/*@unused@*/
+    int qva_verbose;		/*!< (unused) */
+/*@only@*/ /*@null@*/
+    const char * qva_queryFormat;/*!< Format for headerSprintf(). */
+    int sign;			/*!< Is a passphrase needed? */
+    int trust;			/*!< Trust metric when importing pubkeys. */
+/*@observer@*/
+    const char * passPhrase;	/*!< Pass phrase. */
+/*@owned@*/ /*@null@*/
+    const char * qva_prefix;	/*!< Path to top of install tree. */
+    char qva_mode;
+		/*!<
+		- 'q'	from --query, -q
+		- 'Q'	from --querytags
+		- 'V'	from --verify, -V
+		- 'A'	from --addsign
+		- 'I'	from --import
+		- 'K'	from --checksig, -K
+		- 'R'	from --resign
+		*/
+    char qva_char;		/*!< (unused) always ' ' */
+
+    /* install/erase mode arguments */
+    rpmdepFlags depFlags;
+    rpmtransFlags transFlags;
+    rpmprobFilterFlags probFilter;
+    rpmInstallInterfaceFlags installInterfaceFlags;
+    uint_32 arbtid;		/*!< from --arbgoal */
+    uint_32 rbtid;		/*!< from --rollback */
+    uint_32 *rbtidExcludes;	/*!< from --rollback */
+    int numrbtidExcludes;	/*!< from --rollback */
+    int numRelocations;
+    int noDeps;
+    int incldocs;
+    int no_rollback_links;
+/*@owned@*/ /*@null@*/
+    rpmRelocation relocations;
+
+    /* database mode arguments */
+    int init;			/*!< from --initdb */
+    int rebuild;		/*!< from --rebuilddb */
+    int verify;			/*!< from --verifydb */
+
+    /* rollback vectors */
+    int (*rbCheck) (rpmts ts);
+    int (*rbOrder) (rpmts ts);
+    int (*rbRun) (rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet);
+};
+#endif
+
+/** \ingroup rpmcli
+ */
+/*@unchecked@*/
+extern struct rpmQVKArguments_s rpmQVKArgs;
+
+/** \ingroup rpmcli
+ */
+/*@unchecked@*/
+extern struct rpmQVKArguments_s rpmIArgs;
+
+/** \ingroup rpmcli
+ */
+/*@unchecked@*/
+extern struct rpmQVKArguments_s rpmDBArgs;
+
+/* ==================================================================== */
+/** \name RPMBT */
+/*@{*/
+
+/** \ingroup rpmcli
+ * Describe build command line request.
+ */
+struct rpmBuildArguments_s {
+    rpmQueryFlags qva_flags;	/*!< Bit(s) to control verification. */
+    int buildAmount;		/*!< Bit(s) to control operation. */
+/*@observer@*/
+    const char * passPhrase;	/*!< Pass phrase. */
+/*@only@*/ /*@null@*/
+    const char * cookie;	/*!< NULL for binary, ??? for source, rpm's */
+    int force;			/*!< from --force */
+    int noBuild;		/*!< from --nobuild */
+    int noDeps;			/*!< from --nodeps */
+    int noLang;			/*!< from --nolang */
+    int shortCircuit;		/*!< from --short-circuit */
+    int sign;			/*!< from --sign */
+    int useCatalog;		/*!< from --usecatalog */
+    char buildMode;		/*!< Build mode (one of "btBC") */
+    char buildChar;		/*!< Build stage (one of "abcilps ") */
+/*@observer@*/ /*@null@*/
+    const char * rootdir;
+};
+
+/** \ingroup rpmcli
+ */
+typedef	struct rpmBuildArguments_s *	BTA_t;
+
+/** \ingroup rpmcli
+ */
+/*@unchecked@*/
+extern struct rpmBuildArguments_s	rpmBTArgs;
+
+/** \ingroup rpmcli
+ */
+/*@unchecked@*/
+extern struct poptOption		rpmBuildPoptTable[];
+
+/*@}*/
+
 #ifdef __cplusplus
 }
 #endif
