@@ -6,6 +6,9 @@
  * Structure(s) used for dependency tag sets.
  */
 
+#include "rpmevr.h"
+#define	_RPMNS_INTERNAL
+#include "rpmns.h"
 #include "rpmps.h"
 
 /**
@@ -24,7 +27,7 @@ extern int _rpmds_nopromote;
 
 #if defined(_RPMDS_INTERNAL)
 /** \ingroup rpmds
- * A package dependency set.
+ * A dependency set.
  */
 struct rpmds_s {
 /*@observer@*/
@@ -33,11 +36,11 @@ struct rpmds_s {
     const char * DNEVR;		/*!< Formatted dependency string. */
 /*@refcounted@*/ /*@null@*/
     Header h;			/*!< Header for dependency set (or NULL) */
-/*@only@*/
+/*@only@*/ /*@relnull@*/
     const char ** N;		/*!< Name. */
-/*@only@*/ /*@null@*/
+/*@only@*/ /*@relnull@*/
     const char ** EVR;		/*!< Epoch-Version-Release. */
-/*@only@*/ /*@null@*/
+/*@only@*/ /*@relnull@*/
     int_32 * Flags;		/*!< Bit(s) identifying context/comparison. */
 /*@only@*/ /*@null@*/
     uint_32 * Color;		/*!< Bit(s) calculated from file color(s). */
@@ -45,6 +48,12 @@ struct rpmds_s {
     int_32 * Refs;		/*!< No. of file refs. */
 /*@only@*/ /*@null@*/
     int_32 * Result;		/*!< Dependency check result. */
+/*@null@*/
+    int (*EVRparse) (const char *evrstr, EVR_t evr);	 /* EVR parsing. */
+    int (*EVRcmp) (const char *a, const char *b);	 /* EVR comparison. */
+    struct rpmns_s ns;		/*!< Name (split). */
+/*@only@*/ /*@null@*/
+    const char * A;		/*!< Arch (from containing package). */
     int_32 BT;			/*!< Package build time tie breaker. */
     rpmTag tagN;		/*!< Header tag. */
     rpmTagType Nt, EVRt, Ft;	/*!< Tag data types. */
@@ -58,22 +67,43 @@ struct rpmds_s {
 };
 #endif	/* _RPMDS_INTERNAL */
 
-/**
- * Container to carry provides/requires/conflicts/obsoletes dependencies.
+#if defined(_RPMPRCO_INTERNAL)
+/** \ingroup rpmds
+ * Container for provides/requires/conflicts/obsoletes dependency set(s).
  */
-typedef struct rpmPRCO_s * rpmPRCO;
-#if !defined(SWIG)
 struct rpmPRCO_s {
-/*@dependent@*/ /*@null@*/
+/*@dependent@*/ /*@relnull@*/
     rpmds * Pdsp;		/*!< Provides: collector. */
-/*@dependent@*/ /*@null@*/
+/*@dependent@*/ /*@relnull@*/
     rpmds * Rdsp;		/*!< Requires: collector. */
-/*@dependent@*/ /*@null@*/
+/*@dependent@*/ /*@relnull@*/
     rpmds * Cdsp;		/*!< Conflicts: collector. */
-/*@dependent@*/ /*@null@*/
+/*@dependent@*/ /*@relnull@*/
     rpmds * Odsp;		/*!< Obsoletes: collector. */
+/*@dependent@*/ /*@relnull@*/
+    rpmds * Tdsp;		/*!< Triggers collector. */
+/*@dependent@*/ /*@relnull@*/
+    rpmds * Ddsp;		/*!< Dirnames collector. */
+/*@dependent@*/ /*@relnull@*/
+    rpmds * Ldsp;		/*!< Linktos collector. */
+/*@refcounted@*/ /*@null@*/
+    rpmds this;		/*!< N = EVR */
+/*@refcounted@*/ /*@null@*/
+    rpmds P;		/*!< Provides: */
+/*@refcounted@*/ /*@null@*/
+    rpmds R;		/*!< Requires: */
+/*@refcounted@*/ /*@null@*/
+    rpmds C;		/*!< Conflicts: */
+/*@refcounted@*/ /*@null@*/
+    rpmds O;		/*!< Obsoletes: */
+/*@refcounted@*/ /*@null@*/
+    rpmds T;		/*!< Triggers */
+/*@refcounted@*/ /*@null@*/
+    rpmds D;		/*!< Dirnames */
+/*@refcounted@*/ /*@null@*/
+    rpmds L;		/*!< Linktos */
 };
-#endif
+#endif	/* _RPMPRCO_INTERNAL */
 
 #ifdef __cplusplus
 extern "C" {
@@ -125,10 +155,9 @@ rpmds XrpmdsLink (/*@null@*/ rpmds ds, /*@null@*/ const char * msg,
  */
 /*@null@*/
 rpmds rpmdsFree(/*@killref@*/ /*@only@*/ /*@null@*/ rpmds ds)
-	/*@modifies ds@*/;
+	/*@modifies ds @*/;
 /**
  * Create and load a dependency set.
- * @deprecated Only scareMem = 0 will be permitted.
  * @param h		header
  * @param tagN		type of dependency
  * @param flags		scareMem(0x1), nofilter(0x2)
@@ -139,15 +168,24 @@ rpmds rpmdsNew(Header h, rpmTag tagN, int flags)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies h, rpmGlobalMacroContext, fileSystem, internalState @*/;
 
+/**
+ * Return N string, expanded if necessary.
+ * @param ds		dependency set
+ * @return		new N string (malloc'ed)
+ */
+const char * rpmdsNewN(rpmds ds)
+	/*@globals rpmGlobalMacroContext, h_errno @*/
+	/*@modifies ds, rpmGlobalMacroContext @*/;
+
 /**
  * Return new formatted dependency string.
  * @param dspfx		formatted dependency string prefix
  * @param ds		dependency set
  * @return		new formatted dependency (malloc'ed)
  */
-/*@only@*/
-char * rpmdsNewDNEVR(const char * dspfx, const rpmds ds)
-	/*@*/;
+char * rpmdsNewDNEVR(const char * dspfx, rpmds ds)
+	/*@globals rpmGlobalMacroContext, h_errno @*/
+	/*@modifies ds, rpmGlobalMacroContext @*/;
 
 /**
  * Create, load and initialize a dependency for this header. 
@@ -212,7 +250,7 @@ extern const char * rpmdsDNEVR(/*@null@*/ const rpmds ds)
  * @return		current dependency name, NULL on invalid
  */
 /*@observer@*/ /*@null@*/
-extern const char * rpmdsN(/*@null@*/ const rpmds ds)
+extern const char * rpmdsN(/*@null@*/ rpmds ds)
 	/*@*/;
 
 /**
@@ -240,6 +278,15 @@ int_32 rpmdsFlags(/*@null@*/ const rpmds ds)
 rpmTag rpmdsTagN(/*@null@*/ const rpmds ds)
 	/*@*/;
 
+/**
+ * Return current dependency arch.
+ * @param ds		dependency set
+ * @return		current dependency arch, NULL on invalid
+ */
+/*@observer@*/ /*@relnull@*/
+extern const char * rpmdsA(/*@null@*/ const rpmds ds)
+	/*@*/;
+
 /**
  * Return dependency build time.
  * @param ds		dependency set
@@ -257,6 +304,14 @@ time_t rpmdsBT(/*@null@*/ const rpmds ds)
 time_t rpmdsSetBT(/*@null@*/ const rpmds ds, time_t BT)
 	/*@modifies ds @*/;
 
+/**
+ * Return dependency class type.
+ * @param ds		dependency set
+ * @return		dependency class type
+ */
+nsType rpmdsNSType(/*@null@*/ const rpmds ds)
+	/*@*/;
+
 /**
  * Return current "Don't promote Epoch:" flag.
  *
@@ -282,6 +337,28 @@ int rpmdsNoPromote(/*@null@*/ const rpmds ds)
 int rpmdsSetNoPromote(/*@null@*/ rpmds ds, int nopromote)
 	/*@modifies ds @*/;
 
+/**
+ * Set EVR parsing function.
+ * @param ds		dependency set
+ * @param EVRparse	EVR parsing function (NULL uses default)
+ * @return		previous EVR parsing function
+ */
+/*@null@*/
+void * rpmdsSetEVRparse(/*@null@*/ rpmds ds,
+		/*@null@*/ int (*EVRparse)(const char *everstr, EVR_t evr))
+	/*@modifies ds @*/;
+
+/**
+ * Set EVR comparison function.
+ * @param ds		dependency set
+ * @param EVRcmp	EVR comparison function (NULL uses default)
+ * @return		previous EVR comparison function
+ */
+/*@null@*/
+void * rpmdsSetEVRcmp(/*@null@*/ rpmds ds,
+		/*@null@*/ int (*EVRcmp)(const char *a, const char *b))
+	/*@modifies ds @*/;
+
 /**
  * Return current dependency color.
  * @param ds		dependency set
@@ -411,14 +488,14 @@ int rpmdsRpmlib(rpmds * dsp, /*@null@*/ void * tblp)
 	/*@modifies *dsp @*/;
 
 /**
- * Load sysinfo provides into a dependency set.
- * @retval *dsp		(loaded) depedency set
+ * Load sysinfo dependencies into a dependency set.
+ * @retval *PRCO	provides/requires/conflicts/obsoletes depedency set(s)
  * @param fn		path to file (NULL uses /etc/rpm/sysinfo)
  * @return		0 on success
  */
-int rpmdsSysinfo(rpmds * dsp, /*@null@*/ const char * fn)
+int rpmdsSysinfo(rpmPRCO PRCO, /*@null@*/ const char * fn)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-	/*@modifies *dsp, rpmGlobalMacroContext, h_errno,
+	/*@modifies PRCO, rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState @*/;
 
 /**
@@ -441,6 +518,34 @@ int rpmdsGetconf(rpmds * dsp, /*@null@*/ const char * path)
 int rpmdsMergePRCO(void * context, rpmds ds)
 	/*@modifies context, ds @*/;
 
+/**
+ * Free dependency set(s) container.
+ * @param PRCO		dependency set(s) container
+ * @return		NULL
+ */
+/*@null@*/
+rpmPRCO rpmdsFreePRCO(/*@only@*/ /*@null@*/ rpmPRCO PRCO)
+	/*@modifies PRCO @*/;
+
+/**
+ * Create dependency set(s) container.
+ * @param h		header
+ * @return		0 on success
+ */
+rpmPRCO rpmdsNewPRCO(/*@null@*/ Header h)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies h, rpmGlobalMacroContext, fileSystem, internalState @*/;
+
+/**
+ * Retrieve a dependency set from container.
+ * @param PRCO		dependency set(s) container
+ * @param tagN		type of dependency set
+ * @return		dependency set (or NULL)
+ */
+/*@null@*/
+rpmds rpmdsFromPRCO(/*@null@*/ rpmPRCO PRCO, rpmTag tagN)
+	/*@*/;
+
 /**
  * Extract ELF dependencies from a file.
  * @param fn		file name
@@ -467,6 +572,33 @@ int rpmdsLdconfig(rpmPRCO PRCO, /*@null@*/ const char * fn)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies *PRCO, rpmGlobalMacroContext, fileSystem, internalState @*/;
 
+#if defined(__sun)
+/**
+ * Given a colon-separated list of directories, search each of those
+ * directories for (ELF or ELF64) shared objects, and load the provided
+ * shared objects into a dependency set.
+ * @todo Add dependency colors, and attach to file.
+ * @retval *PRCO	provides/requires/conflicts/obsoletes depedency set(s)
+ * @param rldp		:-sep string of dirs (NULL uses /lib:/usr/lib)
+ * @return		0 on success
+ */
+int rpmdsRldpath(rpmPRCO PRCO, /*@null@*/ const char * rldp)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies *PRCO, rpmGlobalMacroContext, fileSystem, internalState @*/;
+
+/**
+ * Use Solaris' crle(1) command to find the ELF (or ELF64) loader path.
+ * calls rpmdsRldpath once it has the loader path.
+ * @todo Add dependency colors, and attach to file.
+ * @retval *PRCO	provides/requires/conflicts/obsoletes depedency set(s)
+ * @param fn		unused.
+ * @return		0 on success
+ */
+int rpmdsCrle(rpmPRCO PRCO, /*@null@*/ const char * fn)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies *PRCO, rpmGlobalMacroContext, fileSystem, internalState @*/;
+#endif
+
 /**
  * Load uname(2) provides into a dependency set.
  * @retval *dsp		(loaded) depedency set
@@ -532,6 +664,15 @@ int rpmdsAnyMatchesDep (const Header h, const rpmds req, int nopromote)
 int rpmdsNVRMatchesDep(const Header h, const rpmds req, int nopromote)
 	/*@*/;
 
+/**
+ * Negate return code for negated comparisons.
+ * @param ds		dependency set
+ * @param rc		postive return code
+ * @return		return code
+ */
+int rpmdsNegateRC(const rpmds ds, int rc)
+	/*@*/;
+
 #if !defined(SWIG)
 /**
  * Return current dependency type name.
