@@ -138,13 +138,15 @@ const char * rpmfiFN(rpmfi fi)
 
     /*@-branchstate@*/
     if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+	const char *dn;
 	char * t;
 	if (fi->fn == NULL)
 	    fi->fn = xmalloc(fi->fnlen);
 	FN = t = fi->fn;
+	(void) urlPath(fi->dnl[fi->dil[fi->i]], &dn);
 /*@-boundswrite@*/
 	*t = '\0';
-	t = stpcpy(t, fi->dnl[fi->dil[fi->i]]);
+	t = stpcpy(t, dn);
 	t = stpcpy(t, fi->bnl[fi->i]);
 /*@=boundswrite@*/
     }
@@ -152,9 +154,9 @@ const char * rpmfiFN(rpmfi fi)
     return FN;
 }
 
-int_32 rpmfiFFlags(rpmfi fi)
+uint_32 rpmfiFFlags(rpmfi fi)
 {
-    int_32 FFlags = 0;
+    uint_32 FFlags = 0;
 
     if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
 /*@-boundsread@*/
@@ -165,9 +167,24 @@ int_32 rpmfiFFlags(rpmfi fi)
     return FFlags;
 }
 
-int_32 rpmfiVFlags(rpmfi fi)
+uint_32 rpmfiSetFFlags(rpmfi fi, uint_32 FFlags)
 {
-    int_32 VFlags = 0;
+    uint_32 oFFlags = 0;
+
+    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+/*@-boundsread@*/
+	if (fi->fflags != NULL && fi->h == NULL) {
+	    oFFlags = fi->fflags[fi->i];
+	    *((uint_32 *)(fi->fflags + fi->i)) = FFlags;
+	}
+/*@=boundsread@*/
+    }
+    return oFFlags;
+}
+
+uint_32 rpmfiVFlags(rpmfi fi)
+{
+    uint_32 VFlags = 0;
 
     if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
 /*@-boundsread@*/
@@ -178,6 +195,21 @@ int_32 rpmfiVFlags(rpmfi fi)
     return VFlags;
 }
 
+uint_32 rpmfiSetVFlags(rpmfi fi, uint_32 VFlags)
+{
+    uint_32 oVFlags = 0;
+
+    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+/*@-boundsread@*/
+	if (fi->vflags != NULL && fi->h == NULL) {
+	    oVFlags = fi->vflags[fi->i];
+	    *((uint_32 *)(fi->vflags + fi->i)) = VFlags;
+	}
+/*@=boundsread@*/
+    }
+    return oVFlags;
+}
+
 int_16 rpmfiFMode(rpmfi fi)
 {
     int_16 fmode = 0;
@@ -204,6 +236,21 @@ rpmfileState rpmfiFState(rpmfi fi)
     return fstate;
 }
 
+rpmfileState rpmfiSetFState(rpmfi fi, rpmfileState fstate)
+{
+    int_32 ofstate = 0;
+
+    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+/*@-boundsread@*/
+	if (fi->fstates != NULL) {
+	    ofstate = fi->fstates[fi->i];
+	    fi->fstates[fi->i] = fstate;
+	}
+/*@=boundsread@*/
+    }
+    return ofstate;
+}
+
 const unsigned char * rpmfiDigest(rpmfi fi, int * algop, size_t * lenp)
 {
     unsigned char * digest = NULL;
@@ -223,19 +270,6 @@ const unsigned char * rpmfiDigest(rpmfi fi, int * algop, size_t * lenp)
     return digest;
 }
 
-const unsigned char * rpmfiMD5(rpmfi fi)
-{
-    unsigned char * digest = NULL;
-
-    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
-/*@-boundsread@*/
-	if (fi->digests != NULL)
-	    digest = fi->digests + (fi->digestlen * fi->i);
-/*@=boundsread@*/
-    }
-    return digest;
-}
-
 const char * rpmfiFLink(rpmfi fi)
 {
     const char * flink = NULL;
@@ -340,11 +374,11 @@ const char * rpmfiFContext(rpmfi fi)
     return fcontext;
 }
 
-int_32 rpmfiFDepends(rpmfi fi, const int_32 ** fddictp)
+int_32 rpmfiFDepends(rpmfi fi, const uint_32 ** fddictp)
 {
     int fddictx = -1;
     int fddictn = 0;
-    const int_32 * fddict = NULL;
+    const uint_32 * fddict = NULL;
 
     if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
 /*@-boundsread@*/
@@ -574,7 +608,7 @@ fileAction rpmfiDecideFate(const rpmfi ofi, rpmfi nfi, int skipMissing)
 {
     const char * fn = rpmfiFN(nfi);
     int newFlags = rpmfiFFlags(nfi);
-    char buffer[1024];
+    char buffer[1024+1];
     fileTypes dbWhat, newWhat, diskWhat;
     struct stat sb;
     int save = (newFlags & RPMFILE_NOREPLACE) ? FA_ALTNAME : FA_SAVE;
@@ -604,7 +638,7 @@ fileAction rpmfiDecideFate(const rpmfi ofi, rpmfi nfi, int skipMissing)
     if (newWhat == XDIR)
 	return FA_CREATE;
 
-    if (diskWhat != newWhat)
+    if (diskWhat != newWhat && dbWhat != REG && dbWhat != LINK)
 	return save;
     else if (newWhat != dbWhat && diskWhat != dbWhat)
 	return save;
@@ -626,12 +660,13 @@ fileAction rpmfiDecideFate(const rpmfi ofi, rpmfi nfi, int skipMissing)
 	size_t nlen = 0;
 	const unsigned char * ndigest;
 	odigest = rpmfiDigest(ofi, &oalgo, &olen);
-	/* XXX avoid digest on sparse /var/log/lastlog file. */
-	if (strcmp(fn, "/var/log/lastlog"))
-	if (dodigest(oalgo, fn, buffer, 0, NULL))
-	    return FA_CREATE;	/* assume file has been removed */
-	if (odigest && !memcmp(odigest, buffer, olen))
-	    return FA_CREATE;	/* unmodified config file, replace. */
+	if (diskWhat == REG) {
+	    if (!(newFlags & RPMFILE_SPARSE))
+	    if (dodigest(oalgo, fn, buffer, 0, NULL))
+		return FA_CREATE;	/* assume file has been removed */
+	    if (odigest && !memcmp(odigest, buffer, olen))
+		return FA_CREATE;	/* unmodified config file, replace. */
+	}
 	ndigest = rpmfiDigest(nfi, &nalgo, &nlen);
 /*@-nullpass@*/
 	if (odigest && ndigest && oalgo == nalgo && olen == nlen
@@ -640,11 +675,14 @@ fileAction rpmfiDecideFate(const rpmfi ofi, rpmfi nfi, int skipMissing)
 /*@=nullpass@*/
     } else /* dbWhat == LINK */ {
 	const char * oFLink, * nFLink;
-	if (readlink(fn, buffer, sizeof(buffer) - 1) == -1)
-	    return FA_CREATE;	/* assume file has been removed */
 	oFLink = rpmfiFLink(ofi);
-	if (oFLink && !strcmp(oFLink, buffer))
-	    return FA_CREATE;	/* unmodified config file, replace. */
+	if (diskWhat == LINK) {
+	    if (readlink(fn, buffer, sizeof(buffer) - 1) == -1)
+		return FA_CREATE;	/* assume file has been removed */
+	    buffer[sizeof(buffer)-1] = '\0';
+	    if (oFLink && !strcmp(oFLink, buffer))
+		return FA_CREATE;	/* unmodified config file, replace. */
+	}
 	nFLink = rpmfiFLink(nfi);
 /*@-nullpass@*/
 	if (oFLink && nFLink && !strcmp(oFLink, nFLink))
@@ -1241,11 +1279,12 @@ static inline unsigned char nibble(char c)
     if (hge((_h), (_tag), NULL, (void **) &(_data), NULL)) \
 	_data = xstrdup(_data)
 
-rpmfi rpmfiNew(const rpmts ts, Header h, rpmTag tagN, int scareMem)
+rpmfi rpmfiNew(const rpmts ts, Header h, rpmTag tagN, int flags)
 {
+    HFD_t hfd = headerFreeData;
+    int scareMem = (flags & 0x1);
     HGE_t hge =
 	(scareMem ? (HGE_t) headerGetEntryMinMemory : (HGE_t) headerGetEntry);
-    HFD_t hfd = headerFreeData;
     rpmte p;
     rpmfi fi = NULL;
     const char * Type;
@@ -1256,6 +1295,9 @@ rpmfi rpmfiNew(const rpmts ts, Header h, rpmTag tagN, int scareMem)
     int xx;
     int i;
 
+#ifdef	NOTYET			/* XXX transaction.c still needs scareMem 1 */
+assert(scareMem == 0);		/* XXX always allocate memory */
+#endif
     if (tagN == RPMTAG_BASENAMES) {
 	Type = "Files";
     } else {
@@ -1283,12 +1325,7 @@ rpmfi rpmfiNew(const rpmts ts, Header h, rpmTag tagN, int scareMem)
     if (fi->fsm == NULL)
 	fi->fsm = newFSM();
 
-    {	int_32 rtid = 0;
-	if (hge(h, RPMTAG_REMOVETID, NULL, (void **) &rtid, NULL) && rtid > 0)
-	    fi->fsm->repackaged = 1;
-	else
-	    fi->fsm->repackaged = 0;
-    }
+    fi->fsm->repackaged = (headerIsEntry(h, RPMTAG_REMOVETID) ? 1 : 0);
 
     /* 0 means unknown */
     xx = hge(h, RPMTAG_ARCHIVESIZE, NULL, (void **) &uip, NULL);
@@ -1452,9 +1489,8 @@ assert(dalgo == fi->fdigestalgos[i]);
 	}
 
 	/* XXX test for incompatible arch triggering autorelocation is dumb. */
-	if (newPath != NULL && *newPath != '\0' && i == p->nrelocs
-	 && p->archScore == 0)
-	{
+	/* XXX DIEDIEDIE: used to test '... && p->archScore == 0' */
+	if (newPath != NULL && *newPath != '\0' && i == p->nrelocs) {
 
 	    p->relocs =
 		xrealloc(p->relocs, (p->nrelocs + 2) * sizeof(*p->relocs));
@@ -1799,7 +1835,7 @@ void rpmfiBuildFDeps(Header h, rpmTag tagN,
     char deptype = 'R';
     char mydt;
     const char * DNEVR;
-    const int_32 * ddict;
+    const uint_32 * ddict;
     unsigned ix;
     int ndx;
 
