@@ -314,17 +314,35 @@ extern const char * rpmfiFN(/*@null@*/ rpmfi fi)
  * @param fi		file info set
  * @return		current file flags, 0 on invalid
  */
-int_32 rpmfiFFlags(/*@null@*/ rpmfi fi)
+uint_32 rpmfiFFlags(/*@null@*/ rpmfi fi)
 	/*@*/;
 
+/**
+ * Set current file flags in file info set.
+ * @param fi		file info set
+ * @param FFlags	new file flags
+ * @return		previous file flags, 0 on invalid
+ */
+uint_32 rpmfiSetFFlags(/*@null@*/ rpmfi fi, uint_32 FFlags)
+	/*@modifies fi @*/;
+
 /**
  * Return current file verify flags from file info set.
  * @param fi		file info set
  * @return		current file verify flags, 0 on invalid
  */
-int_32 rpmfiVFlags(/*@null@*/ rpmfi fi)
+uint_32 rpmfiVFlags(/*@null@*/ rpmfi fi)
 	/*@*/;
 
+/**
+ * Set current file verify flags in file info set.
+ * @param fi		file info set
+ * @param VFlags	new file verify flags
+ * @return		previous file verify flags, 0 on invalid
+ */
+uint_32 rpmfiSetVFlags(/*@null@*/ rpmfi fi, uint_32 VFlags)
+	/*@modifies fi @*/;
+
 /**
  * Return current file mode from file info set.
  * @param fi		file info set
@@ -341,6 +359,15 @@ int_16 rpmfiFMode(/*@null@*/ rpmfi fi)
 rpmfileState rpmfiFState(/*@null@*/ rpmfi fi)
 	/*@*/;
 
+/**
+ * Set current file state in file info set.
+ * @param fi		file info set
+ * @param fstate	new file state
+ * @return		previous file state, 0 on invalid
+ */
+rpmfileState rpmfiSetFState(/*@null@*/ rpmfi fi, rpmfileState fstate)
+	/*@modifies fi @*/;
+
 /**
  * Return current file (binary) digest from file info set.
  * @param fi		file info set
@@ -354,16 +381,6 @@ extern const unsigned char * rpmfiDigest(/*@null@*/ rpmfi fi,
 		/*@out@*/ /*@null@*/ size_t * lenp)
 	/*@modifies *algop, *lenp @*/;
 
-/**
- * Return current file (binary) md5 digest from file info set.
- * @deprecated Use rpmfiDigest instead.
- * @param fi		file info set
- * @return		current file md5 digest, NULL on invalid
- */
-/*@observer@*/ /*@null@*/
-extern const unsigned char * rpmfiMD5(/*@null@*/ rpmfi fi)
-	/*@*/;
-
 /**
  * Return current file linkto (i.e. symlink(2) target) from file info set.
  * @param fi		file info set
@@ -442,7 +459,7 @@ extern const char * rpmfiFContext(/*@null@*/ rpmfi fi)
  * @return		no. of file depends entries, 0 on invalid
  */
 int_32 rpmfiFDepends(/*@null@*/ rpmfi fi,
-		/*@out@*/ /*@null@*/ const int_32 ** fddictp)
+		/*@out@*/ /*@null@*/ const uint_32 ** fddictp)
 	/*@modifies *fddictp @*/;
 
 /**
@@ -528,15 +545,14 @@ rpmfi rpmfiFree(/*@killref@*/ /*@only@*/ /*@null@*/ rpmfi fi)
 
 /**
  * Create and load a file info set.
- * @deprecated Only scareMem = 0 will be permitted.
  * @param ts		transaction set (NULL skips path relocation)
  * @param h		header
  * @param tagN		RPMTAG_BASENAMES
- * @param scareMem	Use pointers to refcounted header memory?
+ * @param flags		scareMem(0x1), nofilter(0x2)
  * @return		new file info set
  */
 /*@null@*/
-rpmfi rpmfiNew(/*@null@*/ const rpmts ts, Header h, rpmTag tagN, int scareMem)
+rpmfi rpmfiNew(/*@null@*/ const rpmts ts, Header h, rpmTag tagN, int flags)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem @*/
 	/*@modifies ts, h, rpmGlobalMacroContext, fileSystem @*/;
 
