@@ -73,6 +73,10 @@ struct rpmgi_s {
     FTS * ftsp;
 /*@relnull@*/
     FTSENT * fts;
+/*@null@*/
+    rpmRC (*walkPathFilter) (rpmgi gi);
+/*@null@*/
+    rpmRC (*stash) (rpmgi gi, Header h);
 
 /*@refs@*/
     int nrefs;			/*!< Reference count. */
@@ -184,6 +188,14 @@ Header rpmgiHeader(/*@null@*/ rpmgi gi)
 rpmts rpmgiTs(/*@null@*/ rpmgi gi)
         /*@*/;
 
+/**
+ * Escape isspace(3) characters in string.
+ * @param s		string
+ * @return		escaped string
+ */
+const char * rpmgiEscapeSpaces(const char * s)
+	/*@*/;
+
 /**
  * Load iterator args.
  * @param gi		generalized iterator
