@@ -7,21 +7,20 @@
 #include <rpmcli.h>
 
 #include "rpmdb.h"
-#include "rpmds.h"
+#ifdef	NOTYET
+#include "rpmds.h"		/* XXX ts->suggests, +foo -foo =foo args */
+#endif
 
-#define	_RPMTE_INTERNAL		/* XXX findErases needs rpmte internals. */
-#include "rpmte.h"		/* XXX: rpmteChain */
-#define	_RPMTS_INTERNAL		/* ts->goal, ts->dbmode, ts->suggests */
+#include "rpmte.h"		/* XXX rpmtsPrint() */
+#define	_RPMTS_INTERNAL		/* XXX ts->goal, ts->suggests */
 #include "rpmts.h"
 
 #include "manifest.h"
-#include "misc.h"	/* XXX for rpmGlob() */
+#include "misc.h"		/* XXX rpmGlob() */
+#include "rpmgi.h"		/* XXX rpmgiEscapeSpaces */
 #include "debug.h"
 
-/*@access rpmts @*/	/* XXX ts->goal, ts->dbmode */
-/*@access rpmte @*/	/* XXX p->hdrid, p->pkgid, p->NEVRA */
-/*@access IDTX @*/
-/*@access IDT @*/
+/*@access rpmts @*/	/* XXX ts->goal */
 
 /*@unchecked@*/
 int rpmcliPackagesTotal = 0;
@@ -120,7 +119,7 @@ void * rpmShowProgress(/*@null@*/ const void * arg,
 	if (filename == NULL || filename[0] == '\0')
 	    return NULL;
 /*@=boundsread@*/
-	fd = Fopen(filename, "r.ufdio");
+	fd = Fopen(filename, "r");
 	/*@-type@*/ /* FIX: still necessary? */
 	if (fd == NULL || Ferror(fd)) {
 	    rpmError(RPMERR_OPEN, _("open of %s failed: %s\n"), filename,
@@ -285,9 +284,7 @@ struct rpmEIU {
 
 /** @todo Generalize --freshen policies. */
 /*@-bounds@*/
-int rpmInstall(rpmts ts,
-		struct rpmInstallArguments_s * ia,
-		const char ** fileArgv)
+int rpmInstall(rpmts ts, QVA_t ia, const char ** fileArgv)
 {
     struct rpmEIU * eiu = memset(alloca(sizeof(*eiu)), 0, sizeof(*eiu));
     rpmps ps;
@@ -296,6 +293,7 @@ int rpmInstall(rpmts ts,
 /*@only@*/ /*@null@*/ const char * fileURL = NULL;
     int stopInstall = 0;
     const char ** av = NULL;
+    const char *fn;
     rpmVSFlags vsflags, ovsflags, tvsflags;
     int ac = 0;
     int rc;
@@ -314,7 +312,7 @@ int rpmInstall(rpmts ts,
     if (!(ia->transFlags & RPMTRANS_FLAG_NOCONTEXTS)) {
 	rpmsx sx = rpmtsREContext(ts);
 	if (sx == NULL) {
-	    const char *fn = rpmGetPath("%{?_install_file_context_path}", NULL);
+	    fn = rpmGetPath("%{?_install_file_context_path}", NULL);
 	    if (fn != NULL && *fn != '\0') {
 		sx = rpmsxNew(fn);
 		(void) rpmtsSetREContext(ts, sx);
@@ -324,6 +322,7 @@ int rpmInstall(rpmts ts,
 	sx = rpmsxFree(sx);
     }
     (void) rpmtsSetFlags(ts, ia->transFlags);
+    (void) rpmtsSetDFlags(ts, ia->depFlags);
 
     /* Display and set autorollback goal. */
     if (rpmExpandNumeric("%{?_rollback_transaction_on_failure}")) {
@@ -369,7 +368,9 @@ int rpmInstall(rpmts ts,
     for (eiu->fnp = fileArgv; *eiu->fnp != NULL; eiu->fnp++) {
     /*@=temptrans@*/
 	av = _free(av);	ac = 0;
-	rc = rpmGlob(*eiu->fnp, &ac, &av);
+	fn = rpmgiEscapeSpaces(*eiu->fnp);
+	rc = rpmGlob(fn, &ac, &av);
+	fn = _free(fn);
 	if (rc || ac == 0) {
 	    rpmError(RPMERR_OPEN, _("File not found by glob: %s\n"), *eiu->fnp);
 	    continue;
@@ -487,7 +488,7 @@ if (fileURL[0] == '=') {
 	(void) urlPath(*eiu->fnp, &fileName);
 
 	/* Try to read the header from a package file. */
-	eiu->fd = Fopen(*eiu->fnp, "r.ufdio");
+	eiu->fd = Fopen(*eiu->fnp, "r");
 	if (eiu->fd == NULL || Ferror(eiu->fd)) {
 	    rpmError(RPMERR_OPEN, _("open of %s failed: %s\n"), *eiu->fnp,
 			Fstrerror(eiu->fd));
@@ -597,13 +598,12 @@ if (fileURL[0] == '=') {
 	case 0:
 	    rpmMessage(RPMMESS_DEBUG, _("\tadded binary package [%d]\n"),
 			eiu->numRPMS);
+	    eiu->numRPMS++;
 	    /*@switchbreak@*/ break;
 	case 1:
-	    rpmMessage(RPMMESS_ERROR,
-			    _("error reading from file %s\n"), *eiu->fnp);
-	    eiu->numFailed++;
-	    goto exit;
-	    /*@notreached@*/ /*@switchbreak@*/ break;
+	    rpmMessage(RPMMESS_WARNING,
+			    _("package file %s was skipped\n"), *eiu->fnp);
+	    /*@switchbreak@*/ break;
 	case 2:
 	    rpmMessage(RPMMESS_ERROR,
 			    _("file %s requires a newer version of RPM\n"),
@@ -613,7 +613,6 @@ if (fileURL[0] == '=') {
 	    /*@notreached@*/ /*@switchbreak@*/ break;
 	}
 
-	eiu->numRPMS++;
 	continue;
 
 maybe_manifest:
@@ -723,7 +722,7 @@ maybe_manifest:
 	if (eiu->sourceURL != NULL)
 	for (i = 0; i < eiu->numSRPMS; i++) {
 	    if (eiu->sourceURL[i] == NULL) continue;
-	    eiu->fd = Fopen(eiu->sourceURL[i], "r.ufdio");
+	    eiu->fd = Fopen(eiu->sourceURL[i], "r");
 	    if (eiu->fd == NULL || Ferror(eiu->fd)) {
 		rpmMessage(RPMMESS_ERROR, _("cannot open file %s: %s\n"),
 			   eiu->sourceURL[i], Fstrerror(eiu->fd));
@@ -762,8 +761,7 @@ exit:
 }
 /*@=bounds@*/
 
-int rpmErase(rpmts ts, struct rpmInstallArguments_s * ia,
-		const char ** argv)
+int rpmErase(rpmts ts, QVA_t ia, const char ** argv)
 {
     int count;
     const char ** arg;
@@ -788,6 +786,7 @@ int rpmErase(rpmts ts, struct rpmInstallArguments_s * ia,
 	ia->transFlags |= RPMTRANS_FLAG_REPACKAGE;
 
     (void) rpmtsSetFlags(ts, ia->transFlags);
+    (void) rpmtsSetDFlags(ts, ia->depFlags);
 
     /* Display and set autorollback goal. */
     if (rpmExpandNumeric("%{?_rollback_transaction_on_failure}")) {
@@ -801,7 +800,7 @@ int rpmErase(rpmts ts, struct rpmInstallArguments_s * ia,
 
 #ifdef	NOTYET	/* XXX no callbacks on erase yet */
     {	int notifyFlags;
-	notifyFlags = ia->eraseInterfaceFlags | (rpmIsVerbose() ? INSTALL_LABEL : 0 );
+	notifyFlags = ia->installInterfaceFlags | (rpmIsVerbose() ? INSTALL_LABEL : 0 );
 	xx = rpmtsSetNotifyCallback(ts,
 			rpmShowProgress, (void *) ((long)notifyFlags));
     }
@@ -823,7 +822,7 @@ int rpmErase(rpmts ts, struct rpmInstallArguments_s * ia,
 	    while ((h = rpmdbNextIterator(mi)) != NULL) {
 		unsigned int recOffset = rpmdbGetIteratorOffset(mi);
 
-		if (!(count++ == 0 || (ia->eraseInterfaceFlags & UNINSTALL_ALLMATCHES))) {
+		if (!(count++ == 0 || (ia->installInterfaceFlags & INSTALL_ALLMATCHES))) {
 		    rpmMessage(RPMMESS_ERROR, _("\"%s\" specifies multiple packages\n"),
 			*arg);
 		    numFailed++;
@@ -840,7 +839,7 @@ int rpmErase(rpmts ts, struct rpmInstallArguments_s * ia,
 
     if (numFailed) goto exit;
 
-    if (!(ia->eraseInterfaceFlags & UNINSTALL_NODEPS)) {
+    if (!(ia->installInterfaceFlags & INSTALL_NODEPS)) {
 
 	if (rpmtsCheck(ts)) {
 	    numFailed = numPackages;
@@ -864,8 +863,7 @@ int rpmErase(rpmts ts, struct rpmInstallArguments_s * ia,
 	}
     }
 
-    if (!stopUninstall) {
-	(void) rpmtsSetFlags(ts, (rpmtsFlags(ts) | RPMTRANS_FLAG_REVERSE));
+    if (numPackages > 0 && !stopUninstall) {
 
 	/* Drop added/available package indices and dependency sets. */
 	rpmtsClean(ts);
@@ -892,7 +890,7 @@ int rpmInstallSource(rpmts ts, const char * arg,
     int rc;
 
 
-    fd = Fopen(arg, "r.ufdio");
+    fd = Fopen(arg, "r");
     if (fd == NULL || Ferror(fd)) {
 	rpmMessage(RPMMESS_ERROR, _("cannot open %s: %s\n"), arg, Fstrerror(fd));
 	if (fd != NULL) (void) Fclose(fd);
@@ -923,583 +921,3 @@ int rpmInstallSource(rpmts ts, const char * arg,
 
     return rc;
 }
-
-/*@unchecked@*/
-static int reverse = -1;
-
-/**
- */
-static int IDTintcmp(const void * a, const void * b)
-	/*@*/
-{
-    /*@-castexpose@*/
-    return ( reverse * (((IDT)a)->val.u32 - ((IDT)b)->val.u32) );
-    /*@=castexpose@*/
-}
-
-IDTX IDTXfree(IDTX idtx)
-{
-    if (idtx) {
-	int i;
-	if (idtx->idt)
-	for (i = 0; i < idtx->nidt; i++) {
-	    IDT idt = idtx->idt + i;
-	    idt->h = headerFree(idt->h);
-	    idt->key = _free(idt->key);
-	}
-	idtx->idt = _free(idtx->idt);
-	idtx = _free(idtx);
-    }
-    return NULL;
-}
-
-IDTX IDTXnew(void)
-{
-    IDTX idtx = xcalloc(1, sizeof(*idtx));
-    idtx->delta = 10;
-    idtx->size = sizeof(*((IDT)0));
-    return idtx;
-}
-
-IDTX IDTXgrow(IDTX idtx, int need)
-{
-    if (need < 0) return NULL;
-    if (idtx == NULL)
-	idtx = IDTXnew();
-    if (need == 0) return idtx;
-
-    if ((idtx->nidt + need) > idtx->alloced) {
-	while (need > 0) {
-	    idtx->alloced += idtx->delta;
-	    need -= idtx->delta;
-	}
-	idtx->idt = xrealloc(idtx->idt, (idtx->alloced * idtx->size) );
-    }
-    return idtx;
-}
-
-IDTX IDTXsort(IDTX idtx)
-{
-    if (idtx != NULL && idtx->idt != NULL && idtx->nidt > 0)
-	qsort(idtx->idt, idtx->nidt, idtx->size, IDTintcmp);
-    return idtx;
-}
-
-IDTX IDTXload(rpmts ts, rpmTag tag, uint_32 rbtid)
-{
-    IDTX idtx = NULL;
-    rpmdbMatchIterator mi;
-    HGE_t hge = (HGE_t) headerGetEntry;
-    Header h;
-
-    /*@-branchstate@*/
-    mi = rpmtsInitIterator(ts, tag, NULL, 0);
-#ifdef	NOTYET
-    (void) rpmdbSetIteratorRE(mi, RPMTAG_NAME, RPMMIRE_DEFAULT, '!gpg-pubkey');
-#endif
-    while ((h = rpmdbNextIterator(mi)) != NULL) {
-	rpmTagType type = RPM_NULL_TYPE;
-	int_32 count = 0;
-	int_32 * tidp;
-
-	tidp = NULL;
-	if (!hge(h, tag, &type, (void **)&tidp, &count) || tidp == NULL)
-	    continue;
-
-	if (type == RPM_INT32_TYPE && (*tidp == 0 || *tidp == -1))
-	    continue;
-
-	/* Don't bother with headers installed prior to the rollback goal. */
-	if (*tidp < rbtid)
-	    continue;
-
-	idtx = IDTXgrow(idtx, 1);
-	if (idtx == NULL || idtx->idt == NULL)
-	    continue;
-
-	{   IDT idt;
-	    /*@-nullderef@*/
-	    idt = idtx->idt + idtx->nidt;
-	    /*@=nullderef@*/
-	    idt->done = 0;
-	    idt->h = headerLink(h);
-	    idt->key = NULL;
-	    idt->instance = rpmdbGetIteratorOffset(mi);
-	    idt->val.u32 = *tidp;
-	}
-	idtx->nidt++;
-    }
-    mi = rpmdbFreeIterator(mi);
-    /*@=branchstate@*/
-
-    return IDTXsort(idtx);
-}
-
-IDTX IDTXglob(rpmts ts, const char * globstr, rpmTag tag, uint_32 rbtid)
-{
-    HGE_t hge = (HGE_t) headerGetEntry;		/* XXX MinMem? <shrug> */
-    IDTX idtx = NULL;
-    Header h;
-    int_32 * tidp;
-    FD_t fd;
-    const char ** av = NULL;
-    int ac = 0;
-    rpmRC rpmrc;
-    int xx;
-    int i;
-
-    av = NULL;	ac = 0;
-    xx = rpmGlob(globstr, &ac, &av);
-
-/*@-branchstate@*/
-    if (xx == 0)
-    for (i = 0; i < ac; i++) {
-	rpmTagType type;
-	int_32 count;
-	int isSource;
-
-	fd = Fopen(av[i], "r.ufdio");
-	if (fd == NULL || Ferror(fd)) {
-	    rpmError(RPMERR_OPEN, _("open of %s failed: %s\n"), av[i],
-	    		Fstrerror(fd));
-	    if (fd != NULL) (void) Fclose(fd);
-	    continue;
-	}
-
-	rpmrc = rpmReadPackageFile(ts, fd, av[i], &h);
-	(void) Fclose(fd);
-	switch (rpmrc) {
-	default:
-	    goto bottom;
-	    /*@notreached@*/ /*@switchbreak@*/ break;
-	case RPMRC_NOTTRUSTED:
-	case RPMRC_NOKEY:
-	case RPMRC_OK:
-	    isSource = (headerIsEntry(h, RPMTAG_SOURCERPM) == 0);
-	    if (isSource)
-		goto bottom;
-	    /*@switchbreak@*/ break;
-	}
-
-{ const char * origin = headerGetOrigin(h);
-assert(origin != NULL);
-assert(!strcmp(av[i], origin));
-}
-	tidp = NULL;
-	if (!hge(h, tag, &type, (void **) &tidp, &count) || tidp == NULL)
-	    goto bottom;
-
-	/* Don't bother with headers installed prior to the rollback goal. */
-	if (*tidp < rbtid)
-	    goto bottom;
-
-	idtx = IDTXgrow(idtx, 1);
-	if (idtx == NULL || idtx->idt == NULL)
-	    goto bottom;
-
-	{   IDT idt;
-	    idt = idtx->idt + idtx->nidt;
-	    idt->done = 0;
-	    idt->h = headerLink(h);
-	    idt->key = av[i];
-	    av[i] = NULL;
-	    idt->instance = 0;
-	    idt->val.u32 = *tidp;
-	}
-	idtx->nidt++;
-bottom:
-	h = headerFree(h);
-    }
-/*@=branchstate@*/
-
-    for (i = 0; i < ac; i++)
-	av[i] = _free(av[i]);
-    av = _free(av);	ac = 0;
-
-    return IDTXsort(idtx);
-}
-
-/**
- * Search for string B in argv array AV.
- * @param lname		type of link
- * @param AV		argv array
- * @param AC		no. of args
- * @param B		string
- * @return		1 if found, 0 not found, -1 error
- */
-static int cmpArgvStr(const char *lname, const char ** AV, int AC,
-		/*@null@*/ const char * B)
-	/*@*/
-{
-    const char * A;
-    int i;
-
-    if (AV != NULL && AC > 0 && B == NULL) {
-	rpmError(RPMERR_OPEN, _("Missing repackaged package(s) detected:\n"));
-	for (i = 0; i < AC && (A = AV[i]) != NULL; i++)
-	    rpmError(RPMERR_OPEN, _("    %s[%d]: %s\n"), lname, i, A);
-	return -1;
-    }
-
-    if (AV != NULL && B != NULL)
-    for (i = 0; i < AC && (A = AV[i]) != NULL; i++) {
-	if (*A && *B && !strcmp(A, B))
-	    return 1;
-    }
-    return 0;
-}
-
-/**
- * Find (and add to transaction set) all erase elements with matching blink.
- * In addition, recreate any added transaction element linkages.
- *
- * XXX rp->h should have FLINK{HDRID,PKGID,NEVRA} populated.
- * XXX ip->h should have BLINK{HDRID,PKGID,NEVRA} populated.
- * XXX p = ts->teInstall is added transaction element from rp->h.
- *
- * @param ts		transaction set (ts->teInstall set to last added pkg)
- * @param p		most recently added install element (NULL skips linking)
- * @param thistid	current transaction id
- * @param ip		currently installed package(s) to be erased
- * @param niids		no. of currently installed package(s)
- * @return		-1 on error, otherwise no. of erase elemnts added
- */
-static int findErases(rpmts ts, /*@null@*/ rpmte p, unsigned thistid,
-		/*@null@*/ IDT ip, int niids)
-	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-	/*@modifies ts, p, ip, rpmGlobalMacroContext, fileSystem, internalState @*/
-{
-    int rc = 0;
-    int xx;
-
-    /* Erase the previously installed packages for this transaction. 
-     * Provided this transaction is not excluded from the rollback.
-     */
-    while (ip != NULL && ip->val.u32 == thistid) {
-
-	if (ip->done)
-	    goto bottom;
-
-	{
-	    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
-	    const char ** flinkPkgid = NULL;
-	    const char ** flinkHdrid = NULL;
-	    const char ** flinkNEVRA = NULL;
-	    rpmTagType pt, ht, nt;
-	    int_32 pn, hn, nn;
-	    int bingo;
-
-	    xx = hge(ip->h, RPMTAG_BLINKPKGID, &pt, (void **)&flinkPkgid, &pn);
-	    xx = hge(ip->h, RPMTAG_BLINKHDRID, &ht, (void **)&flinkHdrid, &hn);
-	    xx = hge(ip->h, RPMTAG_BLINKNEVRA, &nt, (void **)&flinkNEVRA, &nn);
-
-	    /*
-	     * Link data may be missing and can have multiple entries.
-	     * Try hdrid, pkgid, finally NEVRA, argv vs. str compares.
-	     */
-	    bingo = cmpArgvStr("Hdrid", flinkHdrid, hn, (p ? p->hdrid : NULL));
-	    if (!bingo)
-		bingo = cmpArgvStr("Pkgid", flinkPkgid, pn, (p ? p->pkgid : NULL));
-	    if (!bingo)
-		bingo = cmpArgvStr("NEVRA", flinkNEVRA, nn, (p ? p->NEVRA : NULL));
-	    flinkPkgid = headerFreeData(flinkPkgid, pt);
-	    flinkHdrid = headerFreeData(flinkHdrid, ht);
-	    flinkNEVRA = headerFreeData(flinkNEVRA, nt);
-
-	    if (bingo < 0) {
-		rc = -1;
-		goto exit;
-	    }
-
-	    if (!bingo)
-		goto bottom;
-	}
-
-	rpmMessage(RPMMESS_DEBUG, "\t--- erase h#%u\n", ip->instance);
-
-	rc = rpmtsAddEraseElement(ts, ip->h, ip->instance);
-	if (rc != 0)
-	    goto exit;
-
-	/* Cross link the transaction elements to mimic --upgrade. */
-	if (p != NULL) {
-	    rpmte q = ts->teErase;
-	    xx = rpmteChain(p, q, ip->h, "Rollback");
-	}
-
-#ifdef	NOTYET
-	ip->instance = 0;
-#endif
-	ip->done = 1;
-
-bottom:
-
-	/* Go to the next header in the rpmdb */
-	niids--;
-	if (niids > 0)
-	    ip++;
-	else
-	    ip = NULL;
-    }
-
-exit:
-    return rc;
-}
-
-/** @todo Transaction handling, more, needs work. */
-int rpmRollback(rpmts ts, struct rpmInstallArguments_s * ia, const char ** argv)
-{
-    int ifmask= (INSTALL_UPGRADE|INSTALL_FRESHEN|INSTALL_INSTALL|INSTALL_ERASE);
-    unsigned thistid = 0xffffffff;
-    unsigned prevtid;
-    time_t tid;
-    IDTX itids = NULL;
-    IDTX rtids = NULL;
-    IDT rp;
-    int nrids = 0;
-    IDT ip;
-    int niids = 0;
-    int rc = 0;
-    int vsflags, ovsflags;
-    int numAdded;
-    int numRemoved;
-    int excluded;
-    rpmps ps;
-    int _unsafe_rollbacks = 0;
-    rpmtransFlags transFlags = ia->transFlags;
-
-    if (argv != NULL && *argv != NULL) {
-	rc = -1;
-	goto exit;
-    }
-
-    _unsafe_rollbacks = rpmExpandNumeric("%{?_unsafe_rollbacks}");
-
-    vsflags = rpmExpandNumeric("%{?_vsflags_erase}");
-    if (ia->qva_flags & VERIFY_DIGEST)
-	vsflags |= _RPMVSF_NODIGESTS;
-    if (ia->qva_flags & VERIFY_SIGNATURE)
-	vsflags |= _RPMVSF_NOSIGNATURES;
-    if (ia->qva_flags & VERIFY_HDRCHK)
-	vsflags |= RPMVSF_NOHDRCHK;
-    vsflags |= RPMVSF_NEEDPAYLOAD;	/* XXX no legacy signatures */
-    ovsflags = rpmtsSetVSFlags(ts, vsflags);
-
-    (void) rpmtsSetFlags(ts, transFlags);
-
-    /*  Make the transaction a rollback transaction.  In a rollback
-     *  a best effort is what we want 
-     */
-    rpmtsSetType(ts, RPMTRANS_TYPE_ROLLBACK);
-
-    itids = IDTXload(ts, RPMTAG_INSTALLTID, ia->rbtid);
-    if (itids != NULL) {
-	ip = itids->idt;
-	niids = itids->nidt;
-    } else {
-	ip = NULL;
-	niids = 0;
-    }
-
-    {	const char * globstr = rpmExpand("%{_repackage_dir}/*.rpm", NULL);
-	if (globstr == NULL || *globstr == '%') {
-	    globstr = _free(globstr);
-	    rc = -1;
-	    goto exit;
-	}
-	rtids = IDTXglob(ts, globstr, RPMTAG_REMOVETID, ia->rbtid);
-
-	if (rtids != NULL) {
-	    rp = rtids->idt;
-	    nrids = rtids->nidt;
-	} else {
-	    rp = NULL;
-	    nrids = 0;
-	}
-	globstr = _free(globstr);
-    }
-
-    {	int notifyFlags, xx;
-	notifyFlags = ia->installInterfaceFlags | (rpmIsVerbose() ? INSTALL_LABEL : 0 );
-	xx = rpmtsSetNotifyCallback(ts,
-			rpmShowProgress, (void *) ((long)notifyFlags));
-    }
-
-    /* Run transactions until rollback goal is achieved. */
-    do {
-	prevtid = thistid;
-	rc = 0;
-	rpmcliPackagesTotal = 0;
-	numAdded = 0;
-	numRemoved = 0;
-	ia->installInterfaceFlags &= ~ifmask;
-
-	/* Find larger of the remaining install/erase transaction id's. */
-	thistid = 0;
-	if (ip != NULL && ip->val.u32 > thistid)
-	    thistid = ip->val.u32;
-	if (rp != NULL && rp->val.u32 > thistid)
-	    thistid = rp->val.u32;
-
-	/* If we've achieved the rollback goal, then we're done. */
-	if (thistid == 0 || thistid < ia->rbtid)
-	    break;
-
-	/* If we've reached the (configured) rollback goal, then we're done. */
-	if (_unsafe_rollbacks && thistid <= _unsafe_rollbacks)
-	    break;
-
-	/* Make sure this tid is not excluded */
-	excluded = 0;	/* Assume its not */
-	if (ia->rbtidExcludes != NULL && ia->numrbtidExcludes > 0)
-	{
-	    uint_32 *excludedTID;
-	    for(excludedTID = ia->rbtidExcludes; 
-		excludedTID < ia->rbtidExcludes + ia->numrbtidExcludes;
-		excludedTID++) {
-		if (thistid == *excludedTID) {
-		    time_t ttid = (time_t)thistid;
-		    rpmMessage(RPMMESS_NORMAL,
-			_("Excluding TID from rollback: %-24.24s (0x%08x)\n"),
-				ctime(&ttid), thistid);
-		    excluded = 1;
-		    /*@innerbreak@*/ break;	
-		}
-	    }	
-	}
-
-	rpmtsEmpty(ts);
-	(void) rpmtsSetFlags(ts, transFlags);
-
-	/* Install the previously erased packages for this transaction. 
-	 * Provided this transaction is not excluded from the rollback.
-	 */
-	while (rp != NULL && rp->val.u32 == thistid) {
-	    if (!excluded && !rp->done) {
-		rpmMessage(RPMMESS_DEBUG, "\t+++ install %s\n",
-			(rp->key ? rp->key : "???"));
-
-/*@-abstract@*/
-		rc = rpmtsAddInstallElement(ts, rp->h, (fnpyKey)rp->key,
-			       0, ia->relocations);
-/*@=abstract@*/
-		if (rc != 0)
-		    goto exit;
-
-		numAdded++;
-		rpmcliPackagesTotal++;
-		if (!(ia->installInterfaceFlags & ifmask))
-		    ia->installInterfaceFlags |= INSTALL_UPGRADE;
-
-		/* Re-add linked (i.e. from upgrade/obsoletes) erasures. */
-		rc = findErases(ts, ts->teInstall, thistid, ip, niids);
-		if (rc < 0)
-		    goto exit;
-#ifdef	NOTYET
-	    	rp->h = headerFree(rp->h);
-#endif
-		rp->done = 1;
-	    }
-
-	    /* Go to the next repackaged package */
-	    nrids--;
-	    if (nrids > 0)
-		rp++;
-	    else
-		rp = NULL;
-	}
-
-	/* Re-add pure (i.e. not from upgrade/obsoletes) erasures. */
-	rc = findErases(ts, NULL, thistid, ip, niids);
-	if (rc < 0)
-	    goto exit;
-
-	/* Check that all erasures have been re-added. */
-	while (ip != NULL && ip->val.u32 == thistid) {
-assert(excluded || ip->done);
-	    if (!excluded) {
-		numRemoved++;
-
-		if (_unsafe_rollbacks)
-		    rpmcliPackagesTotal++;
-
-		if (!(ia->installInterfaceFlags & ifmask)) {
-		    ia->installInterfaceFlags |= INSTALL_ERASE;
-		    (void) rpmtsSetFlags(ts, (transFlags | RPMTRANS_FLAG_REVERSE));
-		}
-	    }
-
-	    /* Go to the next header in the rpmdb */
-	    niids--;
-	    if (niids > 0)
-		ip++;
-	    else
-		ip = NULL;
-	}
-
-	/* If this transaction is excluded then continue */
-	if (excluded)
-	    continue;
-
-	/* Anything to do? */
-	if (rpmcliPackagesTotal <= 0)
-	    break;
-
-	tid = (time_t)thistid;
-	rpmMessage(RPMMESS_NORMAL,
-		_("Rollback packages (+%d/-%d) to %-24.24s (0x%08x):\n"),
-			numAdded, numRemoved, ctime(&tid), tid);
-
-	rc = rpmtsCheck(ts);
-	ps = rpmtsProblems(ts);
-	if (rc != 0 && rpmpsNumProblems(ps) > 0) {
-	    rpmMessage(RPMMESS_ERROR, _("Failed dependencies:\n"));
-	    rpmpsPrint(NULL, ps);
-	    ps = rpmpsFree(ps);
-	    goto exit;
-	}
-	ps = rpmpsFree(ps);
-
-	rc = rpmtsOrder(ts);
-	if (rc != 0)
-	    goto exit;
-
-	/* Drop added/available package indices and dependency sets. */
-	rpmtsClean(ts);
-
-	rc = rpmtsRun(ts, NULL, (ia->probFilter|RPMPROB_FILTER_OLDPACKAGE));
-	ps = rpmtsProblems(ts);
-	if (rc > 0 && rpmpsNumProblems(ps) > 0)
-	    rpmpsPrint(stderr, ps);
-	ps = rpmpsFree(ps);
-	if (rc)
-	    goto exit;
-
-	/* Clean up after successful rollback. */
-	if (rtids && !rpmIsDebug()) {
-	    int i;
-	    rpmMessage(RPMMESS_NORMAL, _("Cleaning up repackaged packages:\n"));
-	    if (rtids->idt)
-	    for (i = 0; i < rtids->nidt; i++) {
-		IDT rrp = rtids->idt + i;
-		if (rrp->val.u32 != thistid)
-		    /*@innercontinue@*/ continue;
-		if (rrp->key) {	/* XXX can't happen */
-		    rpmMessage(RPMMESS_NORMAL, _("\tRemoving %s:\n"), rrp->key);
-		    (void) unlink(rrp->key);	/* XXX: Should check rc??? */
-		}
-	    }
-	}
-
-
-    } while (1);
-
-exit:
-    rtids = IDTXfree(rtids);
-    itids = IDTXfree(itids);
-
-    rpmtsEmpty(ts);
-    (void) rpmtsSetFlags(ts, transFlags);
-
-    return rc;
-}
