@@ -14,6 +14,10 @@
 #include "header.h"
 #include "popt.h"
 
+#define RPM_FORMAT_VERSION 4
+#define RPM_MAJOR_VERSION 4
+#define RPM_MINOR_VERSION 9
+
 /**
  * Package read return codes.
  */
@@ -89,17 +93,22 @@ typedef /*@abstract@*/ void * alKey;
 typedef /*@abstract@*/ int alNum;
 /*@=mutrep@*/
 
-/** \ingroup rpmts rpmds 
+/** \ingroup rpmds 
  * Dependency tag sets from a header, so that a header can be discarded early.
  */
 typedef /*@abstract@*/ /*@refcounted@*/ struct rpmds_s * rpmds;
 
-/** \ingroup rpmts rpmfi
+/** \ingroup rpmds 
+ * Container for commonly extracted dependency set(s).
+ */
+typedef struct rpmPRCO_s * rpmPRCO;
+
+/** \ingroup rpmfi
  * File info tag sets from a header, so that a header can be discarded early.
  */
 typedef /*@abstract@*/ /*@refcounted@*/ struct rpmfi_s * rpmfi;
 
-/** \ingroup rpmts rpmte
+/** \ingroup rpmte
  * An element of a transaction set, i.e. a TR_ADDED or TR_REMOVED package.
  */
 typedef /*@abstract@*/ struct rpmte_s * rpmte;
@@ -114,43 +123,11 @@ typedef /*@abstract@*/ /*@refcounted@*/ struct rpmdb_s * rpmdb;
  */
 typedef /*@abstract@*/ struct _rpmdbMatchIterator * rpmdbMatchIterator;
 
-/** \ingroup rpmcli rpmgi
+/** \ingroup rpmgi
  * Generalized iterator.
  */
 typedef /*@abstract@*/ /*@refcounted@*/ struct rpmgi_s * rpmgi;
 
-/** \ingroup header
- * Return name, version, release strings from header.
- * @param h		header
- * @retval *np		name pointer (or NULL)
- * @retval *vp		version pointer (or NULL)
- * @retval *rp		release pointer (or NULL)
- * @return		0 always
- */
-int headerNVR(Header h,
-		/*@null@*/ /*@out@*/ const char ** np,
-		/*@null@*/ /*@out@*/ const char ** vp,
-		/*@null@*/ /*@out@*/ const char ** rp)
-	/*@modifies *np, *vp, *rp @*/;
-
-/** \ingroup header
- * Return name, epoch, version, release, arch strings from header.
- * @param h		header
- * @retval *np		name pointer (or NULL)
- * @retval *ep		epoch pointer (or NULL)
- * @retval *vp		version pointer (or NULL)
- * @retval *rp		release pointer (or NULL)
- * @retval *ap		arch pointer (or NULL)
- * @return		0 always
- */
-int headerNEVRA(Header h,
-		/*@null@*/ /*@out@*/ const char ** np,
-		/*@null@*/ /*@out@*/ /*@unused@*/ const char ** ep,
-		/*@null@*/ /*@out@*/ const char ** vp,
-		/*@null@*/ /*@out@*/ const char ** rp,
-		/*@null@*/ /*@out@*/ const char ** ap)
-	/*@modifies *np, *vp, *rp, *ap @*/;
-
 /** \ingroup header
  * Translate and merge legacy signature tags into header.
  * @todo Remove headerSort() through headerInitIterator() modifies sig.
@@ -232,16 +209,16 @@ typedef enum rpmTag_e {
 /* XXX 2nd underscore prevents tagTable generation */
     RPMTAG_SIG_BASE		= HEADER_SIGBASE,
     RPMTAG_SIGSIZE		= RPMTAG_SIG_BASE+1,	/* i */
-    RPMTAG_SIGLEMD5_1		= RPMTAG_SIG_BASE+2,	/*!< internal - obsolete */
+    RPMTAG_SIGLEMD5_1		= RPMTAG_SIG_BASE+2,	/* internal - obsolete */
     RPMTAG_SIGPGP		= RPMTAG_SIG_BASE+3,	/* x */
-    RPMTAG_SIGLEMD5_2		= RPMTAG_SIG_BASE+4,	/*!< internal - obsolete */
+    RPMTAG_SIGLEMD5_2		= RPMTAG_SIG_BASE+4,	/* x internal - obsolete */
     RPMTAG_SIGMD5	        = RPMTAG_SIG_BASE+5,	/* x */
 #define	RPMTAG_PKGID	RPMTAG_SIGMD5			/* x */
     RPMTAG_SIGGPG	        = RPMTAG_SIG_BASE+6,	/* x */
-    RPMTAG_SIGPGP5	        = RPMTAG_SIG_BASE+7,	/*!< internal - obsolete */
+    RPMTAG_SIGPGP5	        = RPMTAG_SIG_BASE+7,	/* internal - obsolete */
 
-    RPMTAG_BADSHA1_1		= RPMTAG_SIG_BASE+8,	/*!< internal - obsolete */
-    RPMTAG_BADSHA1_2		= RPMTAG_SIG_BASE+9,	/*!< internal - obsolete */
+    RPMTAG_BADSHA1_1		= RPMTAG_SIG_BASE+8,	/* internal - obsolete */
+    RPMTAG_BADSHA1_2		= RPMTAG_SIG_BASE+9,	/* internal - obsolete */
     RPMTAG_PUBKEYS		= RPMTAG_SIG_BASE+10,	/* s[] */
     RPMTAG_DSAHEADER		= RPMTAG_SIG_BASE+11,	/* x */
     RPMTAG_RSAHEADER		= RPMTAG_SIG_BASE+12,	/* x */
@@ -270,7 +247,7 @@ typedef enum rpmTag_e {
     RPMTAG_PACKAGER		= 1015,	/* s */
     RPMTAG_GROUP		= 1016,	/* s{} */
 /*@-enummemuse@*/
-    RPMTAG_CHANGELOG		= 1017, /*!< s[] internal */
+    RPMTAG_CHANGELOG		= 1017, /* s[] internal */
 /*@=enummemuse@*/
     RPMTAG_SOURCE		= 1018,	/* s[] */
     RPMTAG_PATCH		= 1019,	/* s[] */
@@ -285,8 +262,8 @@ typedef enum rpmTag_e {
     RPMTAG_FILESIZES		= 1028,	/* i[] */
     RPMTAG_FILESTATES		= 1029, /* c[] */
     RPMTAG_FILEMODES		= 1030,	/* h[] */
-    RPMTAG_FILEUIDS		= 1031, /*!< internal */
-    RPMTAG_FILEGIDS		= 1032, /*!< internal */
+    RPMTAG_FILEUIDS		= 1031, /* i[] internal */
+    RPMTAG_FILEGIDS		= 1032, /* i[] internal */
     RPMTAG_FILERDEVS		= 1033,	/* h[] */
     RPMTAG_FILEMTIMES		= 1034, /* i[] */
     RPMTAG_FILEDIGESTS		= 1035,	/* s[] */
@@ -294,15 +271,15 @@ typedef enum rpmTag_e {
     RPMTAG_FILELINKTOS		= 1036,	/* s[] */
     RPMTAG_FILEFLAGS		= 1037,	/* i[] */
 /*@-enummemuse@*/
-    RPMTAG_ROOT			= 1038, /*!< internal - obsolete */
+    RPMTAG_ROOT			= 1038, /* internal - obsolete */
 /*@=enummemuse@*/
     RPMTAG_FILEUSERNAME		= 1039,	/* s[] */
     RPMTAG_FILEGROUPNAME	= 1040,	/* s[] */
 /*@-enummemuse@*/
-    RPMTAG_EXCLUDE		= 1041, /*!< internal - obsolete */
-    RPMTAG_EXCLUSIVE		= 1042, /*!< internal - obsolete */
+    RPMTAG_EXCLUDE		= 1041, /* internal - obsolete */
+    RPMTAG_EXCLUSIVE		= 1042, /* internal - obsolete */
 /*@=enummemuse@*/
-    RPMTAG_ICON			= 1043, /* internal - obsolete */
+    RPMTAG_ICON			= 1043, /* x */
     RPMTAG_SOURCERPM		= 1044,	/* s */
     RPMTAG_FILEVERIFYFLAGS	= 1045,	/* i[] */
     RPMTAG_ARCHIVESIZE		= 1046,	/* i */
@@ -313,21 +290,21 @@ typedef enum rpmTag_e {
     RPMTAG_REQUIRENAME		= 1049,	/* s[] */
 #define	RPMTAG_REQUIRES RPMTAG_REQUIRENAME	/* s[] */
     RPMTAG_REQUIREVERSION	= 1050,	/* s[] */
-    RPMTAG_NOSOURCE		= 1051, /*!< internal */
-    RPMTAG_NOPATCH		= 1052, /*!< internal */
+    RPMTAG_NOSOURCE		= 1051, /* i internal */
+    RPMTAG_NOPATCH		= 1052, /* i internal */
     RPMTAG_CONFLICTFLAGS	= 1053, /* i[] */
     RPMTAG_CONFLICTNAME		= 1054,	/* s[] */
 #define	RPMTAG_CONFLICTS RPMTAG_CONFLICTNAME	/* s[] */
 #define	RPMTAG_C	RPMTAG_CONFLICTNAME	/* s[] */
     RPMTAG_CONFLICTVERSION	= 1055,	/* s[] */
-    RPMTAG_DEFAULTPREFIX	= 1056, /*!< internal - deprecated */
-    RPMTAG_BUILDROOT		= 1057, /*!< internal */
-    RPMTAG_INSTALLPREFIX	= 1058, /*!< internal - deprecated */
-    RPMTAG_EXCLUDEARCH		= 1059,
-    RPMTAG_EXCLUDEOS		= 1060,
-    RPMTAG_EXCLUSIVEARCH	= 1061,
-    RPMTAG_EXCLUSIVEOS		= 1062,
-    RPMTAG_AUTOREQPROV		= 1063, /*!< internal */
+    RPMTAG_DEFAULTPREFIX	= 1056, /* s internal - deprecated */
+    RPMTAG_BUILDROOT		= 1057, /* s internal */
+    RPMTAG_INSTALLPREFIX	= 1058, /* s internal - deprecated */
+    RPMTAG_EXCLUDEARCH		= 1059, /* s[] */
+    RPMTAG_EXCLUDEOS		= 1060, /* s[] */
+    RPMTAG_EXCLUSIVEARCH	= 1061, /* s[] */
+    RPMTAG_EXCLUSIVEOS		= 1062, /* s[] */
+    RPMTAG_AUTOREQPROV		= 1063, /* s internal */
     RPMTAG_RPMVERSION		= 1064,	/* s */
     RPMTAG_TRIGGERSCRIPTS	= 1065,	/* s[] */
     RPMTAG_TRIGGERNAME		= 1066,	/* s[] */
@@ -339,43 +316,43 @@ typedef enum rpmTag_e {
     RPMTAG_CHANGELOGNAME	= 1081,	/* s[] */
     RPMTAG_CHANGELOGTEXT	= 1082,	/* s[] */
 /*@-enummemuse@*/
-    RPMTAG_BROKENMD5		= 1083, /*!< internal - obsolete */
+    RPMTAG_BROKENMD5		= 1083, /* internal - obsolete */
 /*@=enummemuse@*/
-    RPMTAG_PREREQ		= 1084, /*!< internal */
+    RPMTAG_PREREQ		= 1084, /* internal */
     RPMTAG_PREINPROG		= 1085,	/* s */
     RPMTAG_POSTINPROG		= 1086,	/* s */
     RPMTAG_PREUNPROG		= 1087,	/* s */
     RPMTAG_POSTUNPROG		= 1088,	/* s */
-    RPMTAG_BUILDARCHS		= 1089,
+    RPMTAG_BUILDARCHS		= 1089, /* s[] */
     RPMTAG_OBSOLETENAME		= 1090,	/* s[] */
 #define	RPMTAG_OBSOLETES RPMTAG_OBSOLETENAME	/* s[] */
 #define	RPMTAG_O	RPMTAG_OBSOLETENAME	/* s[] */
     RPMTAG_VERIFYSCRIPTPROG	= 1091,	/* s */
     RPMTAG_TRIGGERSCRIPTPROG	= 1092,	/* s[] */
-    RPMTAG_DOCDIR		= 1093, /*!< internal */
+    RPMTAG_DOCDIR		= 1093, /* internal */
     RPMTAG_COOKIE		= 1094,	/* s */
     RPMTAG_FILEDEVICES		= 1095,	/* i[] */
     RPMTAG_FILEINODES		= 1096,	/* i[] */
     RPMTAG_FILELANGS		= 1097,	/* s[] */
     RPMTAG_PREFIXES		= 1098,	/* s[] */
     RPMTAG_INSTPREFIXES		= 1099,	/* s[] */
-    RPMTAG_TRIGGERIN		= 1100, /*!< internal */
-    RPMTAG_TRIGGERUN		= 1101, /*!< internal */
-    RPMTAG_TRIGGERPOSTUN	= 1102, /*!< internal */
-    RPMTAG_AUTOREQ		= 1103, /*!< internal */
-    RPMTAG_AUTOPROV		= 1104, /*!< internal */
+    RPMTAG_TRIGGERIN		= 1100, /* internal */
+    RPMTAG_TRIGGERUN		= 1101, /* internal */
+    RPMTAG_TRIGGERPOSTUN	= 1102, /* internal */
+    RPMTAG_AUTOREQ		= 1103, /* internal */
+    RPMTAG_AUTOPROV		= 1104, /* internal */
 /*@-enummemuse@*/
-    RPMTAG_CAPABILITY		= 1105, /*!< i legacy - obsolete */
+    RPMTAG_CAPABILITY		= 1105, /* i legacy - obsolete */
 /*@=enummemuse@*/
-    RPMTAG_SOURCEPACKAGE	= 1106, /*!< i legacy - obsolete */
+    RPMTAG_SOURCEPACKAGE	= 1106, /* i legacy - obsolete */
 /*@-enummemuse@*/
-    RPMTAG_OLDORIGFILENAMES	= 1107, /*!< internal - obsolete */
+    RPMTAG_OLDORIGFILENAMES	= 1107, /* internal - obsolete */
 /*@=enummemuse@*/
-    RPMTAG_BUILDPREREQ		= 1108, /*!< internal */
-    RPMTAG_BUILDREQUIRES	= 1109, /*!< internal */
-    RPMTAG_BUILDCONFLICTS	= 1110, /*!< internal */
+    RPMTAG_BUILDPREREQ		= 1108, /* internal */
+    RPMTAG_BUILDREQUIRES	= 1109, /* internal */
+    RPMTAG_BUILDCONFLICTS	= 1110, /* internal */
 /*@-enummemuse@*/
-    RPMTAG_BUILDMACROS		= 1111, /*!< internal - unused */
+    RPMTAG_BUILDMACROS		= 1111, /* internal - unused */
 /*@=enummemuse@*/
     RPMTAG_PROVIDEFLAGS		= 1112,	/* i[] */
     RPMTAG_PROVIDEVERSION	= 1113,	/* s[] */
@@ -384,25 +361,25 @@ typedef enum rpmTag_e {
     RPMTAG_DIRINDEXES		= 1116,	/* i[] */
     RPMTAG_BASENAMES		= 1117,	/* s[] */
     RPMTAG_DIRNAMES		= 1118,	/* s[] */
-    RPMTAG_ORIGDIRINDEXES	= 1119, /*!< i[] relocation */
-    RPMTAG_ORIGBASENAMES	= 1120, /*!< s[] relocation */
-    RPMTAG_ORIGDIRNAMES		= 1121, /*!< s[] relocation */
+    RPMTAG_ORIGDIRINDEXES	= 1119, /* i[] relocation */
+    RPMTAG_ORIGBASENAMES	= 1120, /* s[] relocation */
+    RPMTAG_ORIGDIRNAMES		= 1121, /* s[] relocation */
     RPMTAG_OPTFLAGS		= 1122,	/* s */
     RPMTAG_DISTURL		= 1123,	/* s */
     RPMTAG_PAYLOADFORMAT	= 1124,	/* s */
     RPMTAG_PAYLOADCOMPRESSOR	= 1125,	/* s */
     RPMTAG_PAYLOADFLAGS		= 1126,	/* s */
-    RPMTAG_INSTALLCOLOR		= 1127, /*!< i transaction color when installed */
+    RPMTAG_INSTALLCOLOR		= 1127, /* i transaction color when installed */
     RPMTAG_INSTALLTID		= 1128,	/* i */
     RPMTAG_REMOVETID		= 1129,	/* i */
 /*@-enummemuse@*/
-    RPMTAG_SHA1RHN		= 1130, /*!< internal - obsolete */
+    RPMTAG_SHA1RHN		= 1130, /* internal - obsolete */
 /*@=enummemuse@*/
-    RPMTAG_RHNPLATFORM		= 1131,	/* s */
+    RPMTAG_RHNPLATFORM		= 1131,	/* s deprecated */
     RPMTAG_PLATFORM		= 1132,	/* s */
-    RPMTAG_PATCHESNAME		= 1133, /*!< placeholder (SuSE) */
-    RPMTAG_PATCHESFLAGS		= 1134, /*!< placeholder (SuSE) */
-    RPMTAG_PATCHESVERSION	= 1135, /*!< placeholder (SuSE) */
+    RPMTAG_PATCHESNAME		= 1133, /* s[] deprecated placeholder (SuSE) */
+    RPMTAG_PATCHESFLAGS		= 1134, /* i[] deprecated placeholder (SuSE) */
+    RPMTAG_PATCHESVERSION	= 1135, /* s[] deprecated placeholder (SuSE) */
     RPMTAG_CACHECTIME		= 1136,	/* i */
     RPMTAG_CACHEPKGPATH		= 1137,	/* s */
     RPMTAG_CACHEPKGSIZE		= 1138,	/* i */
@@ -415,23 +392,23 @@ typedef enum rpmTag_e {
     RPMTAG_DEPENDSDICT		= 1145,	/* i[] */
     RPMTAG_SOURCEPKGID		= 1146,	/* x */
     RPMTAG_FILECONTEXTS		= 1147,	/* s[] */
-    RPMTAG_FSCONTEXTS		= 1148,	/*!< s[] extension */
-    RPMTAG_RECONTEXTS		= 1149,	/*!< s[] extension */
-    RPMTAG_POLICIES		= 1150,	/*!< s[] selinux *.te policy file. */
+    RPMTAG_FSCONTEXTS		= 1148,	/* s[] extension */
+    RPMTAG_RECONTEXTS		= 1149,	/* s[] extension */
+    RPMTAG_POLICIES		= 1150,	/* s[] selinux *.te policy file. */
     RPMTAG_PRETRANS		= 1151,	/* s */
     RPMTAG_POSTTRANS		= 1152,	/* s */
     RPMTAG_PRETRANSPROG		= 1153,	/* s */
     RPMTAG_POSTTRANSPROG	= 1154,	/* s */
     RPMTAG_DISTTAG		= 1155,	/* s */
-    RPMTAG_SUGGESTSNAME		= 1156,	/* s[] extension placeholder */
+    RPMTAG_SUGGESTSNAME		= 1156,	/* s[] extension */
 #define	RPMTAG_SUGGESTS RPMTAG_SUGGESTSNAME	/* s[] */
-    RPMTAG_SUGGESTSVERSION	= 1157,	/* s[] extension placeholder */
-    RPMTAG_SUGGESTSFLAGS	= 1158,	/* i[]   extension placeholder */
+    RPMTAG_SUGGESTSVERSION	= 1157,	/* s[] extension */
+    RPMTAG_SUGGESTSFLAGS	= 1158,	/* i[] extension */
     RPMTAG_ENHANCESNAME		= 1159,	/* s[] extension placeholder */
 #define	RPMTAG_ENHANCES RPMTAG_ENHANCESNAME	/* s[] */
     RPMTAG_ENHANCESVERSION	= 1160,	/* s[] extension placeholder */
-    RPMTAG_ENHANCESFLAGS	= 1161,	/* i[]   extension placeholder */
-    RPMTAG_PRIORITY		= 1162, /* i[]   extension placeholder */
+    RPMTAG_ENHANCESFLAGS	= 1161,	/* i[] extension placeholder */
+    RPMTAG_PRIORITY		= 1162, /* i[] extension placeholder */
     RPMTAG_CVSID		= 1163, /* s */
 #define	RPMTAG_SVNID	RPMTAG_CVSID	/* s */
     RPMTAG_BLINKPKGID		= 1164, /* s[] */
@@ -441,16 +418,30 @@ typedef enum rpmTag_e {
     RPMTAG_FLINKHDRID		= 1168, /* s[] */
     RPMTAG_FLINKNEVRA		= 1169, /* s[] */
     RPMTAG_PACKAGEORIGIN	= 1170, /* s */
-    RPMTAG_TRIGGERPREIN		= 1171, /*!< internal */
-    RPMTAG_BUILDSUGGESTS	= 1172, /*!< internal */
-    RPMTAG_BUILDENHANCES	= 1173, /*!< internal */
-    RPMTAG_SCRIPTSTATES		= 1174, /*!< i[] scriptlet exit codes */
-    RPMTAG_SCRIPTMETRICS	= 1175, /*!< i[] scriptlet execution times */
-    RPMTAG_BUILDCPUCLOCK	= 1176, /*!< i */
-    RPMTAG_FILEDIGESTALGOS	= 1177, /*!< i[] */
-    RPMTAG_VARIANTS		= 1178, /*!< s[] */
-    RPMTAG_XMAJOR		= 1179, /*!< i */
-    RPMTAG_XMINOR		= 1180, /*!< i */
+    RPMTAG_TRIGGERPREIN		= 1171, /* internal */
+    RPMTAG_BUILDSUGGESTS	= 1172, /* internal */
+    RPMTAG_BUILDENHANCES	= 1173, /* internal */
+    RPMTAG_SCRIPTSTATES		= 1174, /* i[] scriptlet exit codes */
+    RPMTAG_SCRIPTMETRICS	= 1175, /* i[] scriptlet execution times */
+    RPMTAG_BUILDCPUCLOCK	= 1176, /* i */
+    RPMTAG_FILEDIGESTALGOS	= 1177, /* i[] */
+    RPMTAG_VARIANTS		= 1178, /* s[] */
+    RPMTAG_XMAJOR		= 1179, /* i */
+    RPMTAG_XMINOR		= 1180, /* i */
+    RPMTAG_REPOTAG		= 1181,	/* s */
+    RPMTAG_KEYWORDS		= 1182,	/* s[] */
+    RPMTAG_BUILDPLATFORMS	= 1183,	/* s[] */
+    RPMTAG_PACKAGECOLOR		= 1184, /* i */
+    RPMTAG_PACKAGEPREFCOLOR	= 1185, /* i (unimplemented) */
+    RPMTAG_XATTRSDICT		= 1186, /* s[] (unimplemented) */
+    RPMTAG_FILEXATTRSX		= 1187, /* i[] (unimplemented) */
+    RPMTAG_DEPATTRSDICT		= 1188, /* s[] (unimplemented) */
+    RPMTAG_CONFLICTATTRSX	= 1189, /* i[] (unimplemented) */
+    RPMTAG_OBSOLETEATTRSX	= 1190, /* i[] (unimplemented) */
+    RPMTAG_PROVIDEATTRSX	= 1191, /* i[] (unimplemented) */
+    RPMTAG_REQUIREATTRSX	= 1192, /* i[] (unimplemented) */
+    RPMTAG_BUILDPROVIDES	= 1193, /* internal */
+    RPMTAG_BUILDOBSOLETES	= 1194, /* internal */
 
 /*@-enummemuse@*/
     RPMTAG_FIRSTFREE_TAG	/*!< internal */
@@ -514,128 +505,34 @@ typedef	enum rpmfileAttrs_e {
 /*@=enummemuse@*/
     RPMFILE_CONFIG	= (1 <<  0),	/*!< from %%config */
     RPMFILE_DOC		= (1 <<  1),	/*!< from %%doc */
-    RPMFILE_ICON	= (1 <<  2),	/*!< from %%donotuse. */
+    RPMFILE_ICON	= (1 <<  2),	/*!< from Icon: */
     RPMFILE_MISSINGOK	= (1 <<  3),	/*!< from %%config(missingok) */
     RPMFILE_NOREPLACE	= (1 <<  4),	/*!< from %%config(noreplace) */
-    RPMFILE_SPECFILE	= (1 <<  5),	/*!< @todo (unnecessary) marks 1st file in srpm. */
+    RPMFILE_SPECFILE	= (1 <<  5),	/*!< the specfile (srpm only). */
     RPMFILE_GHOST	= (1 <<  6),	/*!< from %%ghost */
     RPMFILE_LICENSE	= (1 <<  7),	/*!< from %%license */
     RPMFILE_README	= (1 <<  8),	/*!< from %%readme */
     RPMFILE_EXCLUDE	= (1 <<  9),	/*!< from %%exclude, internal */
-    RPMFILE_UNPATCHED	= (1 << 10),	/*!< placeholder (SuSE) */
+    RPMFILE_UNPATCHED	= (1 << 10),	/*!< (deprecated) placeholder (SuSE) */
     RPMFILE_PUBKEY	= (1 << 11),	/*!< from %%pubkey */
-    RPMFILE_POLICY	= (1 << 12)	/*!< from %%policy */
+    RPMFILE_POLICY	= (1 << 12),	/*!< from %%policy */
+    RPMFILE_EXISTS	= (1 << 13),	/*!< did lstat(fn, st) succeed? */
+    RPMFILE_SPARSE	= (1 << 14),	/*!< was ((512*st->st_blocks) < st->st_size) ? */
+    RPMFILE_TYPED	= (1 << 15),	/*!< (unimplemented) from %%spook */
+    RPMFILE_SOURCE	= (1 << 16),	/*!< from SourceN: (srpm only). */
+    RPMFILE_PATCH	= (1 << 17)	/*!< from PatchN: (srpm only). */
 } rpmfileAttrs;
 
+#define	RPMFILE_SPOOK	(RPMFILE_GHOST|RPMFILE_TYPED)
 #define	RPMFILE_ALL	~(RPMFILE_NONE)
 
-/**
- * Dependency Attributes.
- */
-typedef	enum rpmsenseFlags_e {
-    RPMSENSE_ANY	= 0,
-/*@-enummemuse@*/
-    RPMSENSE_SERIAL	= (1 << 0),	/*!< (obsolete). */
-/*@=enummemuse@*/
-    RPMSENSE_LESS	= (1 << 1),
-    RPMSENSE_GREATER	= (1 << 2),
-    RPMSENSE_EQUAL	= (1 << 3),
-    RPMSENSE_PROVIDES	= (1 << 4), /* only used internally by builds */
-    RPMSENSE_CONFLICTS	= (1 << 5), /* only used internally by builds */
-    RPMSENSE_PREREQ	= (1 << 6),	/*!< (obsolete). */
-    RPMSENSE_OBSOLETES	= (1 << 7), /* only used internally by builds */
-    RPMSENSE_INTERP	= (1 << 8),	/*!< Interpreter used by scriptlet. */
-    RPMSENSE_SCRIPT_PRE	= (1 << 9),	/*!< %pre dependency. */
-    RPMSENSE_SCRIPT_POST = (1 << 10),	/*!< %post dependency. */
-    RPMSENSE_SCRIPT_PREUN = (1 << 11),	/*!< %preun dependency. */
-    RPMSENSE_SCRIPT_POSTUN = (1 << 12), /*!< %postun dependency. */
-    RPMSENSE_SCRIPT_VERIFY = (1 << 13),	/*!< %verify dependency. */
-    RPMSENSE_FIND_REQUIRES = (1 << 14), /*!< find-requires dependency. */
-    RPMSENSE_FIND_PROVIDES = (1 << 15), /*!< find-provides dependency. */
-
-    RPMSENSE_TRIGGERIN	= (1 << 16),	/*!< %triggerin dependency. */
-    RPMSENSE_TRIGGERUN	= (1 << 17),	/*!< %triggerun dependency. */
-    RPMSENSE_TRIGGERPOSTUN = (1 << 18),	/*!< %triggerpostun dependency. */
-    RPMSENSE_MISSINGOK	= (1 << 19),	/*!< suggests/enhances hint. */
-    RPMSENSE_SCRIPT_PREP = (1 << 20),	/*!< %prep build dependency. */
-    RPMSENSE_SCRIPT_BUILD = (1 << 21),	/*!< %build build dependency. */
-    RPMSENSE_SCRIPT_INSTALL = (1 << 22),/*!< %install build dependency. */
-    RPMSENSE_SCRIPT_CLEAN = (1 << 23),	/*!< %clean build dependency. */
-    RPMSENSE_RPMLIB = (1 << 24),	/*!< rpmlib(feature) dependency. */
-    RPMSENSE_TRIGGERPREIN = (1 << 25),	/*!< %triggerprein dependency. */
-    RPMSENSE_KEYRING	= (1 << 26),
-    RPMSENSE_PATCHES	= (1 << 27),
-    RPMSENSE_CONFIG	= (1 << 28),
-    RPMSENSE_PROBE	= (1 << 29),
-    RPMSENSE_PACKAGE	= (1 << 30)
-} rpmsenseFlags;
-
-#define	RPMSENSE_SENSEMASK	0x0e	 /* Mask to get senses, ie serial, */
-                                         /* less, greater, equal.          */
-
-#define	RPMSENSE_TRIGGER	\
-	(RPMSENSE_TRIGGERPREIN | RPMSENSE_TRIGGERIN | RPMSENSE_TRIGGERUN | RPMSENSE_TRIGGERPOSTUN)
-
-#define	_ALL_REQUIRES_MASK	(\
-    RPMSENSE_INTERP | \
-    RPMSENSE_SCRIPT_PRE | \
-    RPMSENSE_SCRIPT_POST | \
-    RPMSENSE_SCRIPT_PREUN | \
-    RPMSENSE_SCRIPT_POSTUN | \
-    RPMSENSE_SCRIPT_VERIFY | \
-    RPMSENSE_FIND_REQUIRES | \
-    RPMSENSE_MISSINGOK | \
-    RPMSENSE_SCRIPT_PREP | \
-    RPMSENSE_SCRIPT_BUILD | \
-    RPMSENSE_SCRIPT_INSTALL | \
-    RPMSENSE_SCRIPT_CLEAN | \
-    RPMSENSE_RPMLIB | \
-    RPMSENSE_KEYRING | \
-    RPMSENSE_PACKAGE )
-
-#define	_notpre(_x)		((_x) & ~RPMSENSE_PREREQ)
-#define	_INSTALL_ONLY_MASK \
-    _notpre(RPMSENSE_SCRIPT_PRE|RPMSENSE_SCRIPT_POST|RPMSENSE_RPMLIB|RPMSENSE_KEYRING)
-#define	_ERASE_ONLY_MASK  \
-    _notpre(RPMSENSE_SCRIPT_PREUN|RPMSENSE_SCRIPT_POSTUN)
-
-#define	isInstallPreReq(_x)	((_x) & _INSTALL_ONLY_MASK)
-#define	isErasePreReq(_x)	((_x) & _ERASE_ONLY_MASK)
-
 /* ==================================================================== */
 /** \name RPMRC */
 /*@{*/
 
-/* Stuff for maintaining "variables" like SOURCEDIR, BUILDDIR, etc */
-#define	RPMVAR_OPTFLAGS			3
-#define	RPMVAR_INCLUDE			43
-#define	RPMVAR_MACROFILES		49
-
-#define	RPMVAR_NUM			55	/* number of RPMVAR entries */
-
-/** \ingroup rpmrc
- * Return value of an rpmrc variable.
- * @deprecated Use rpmExpand() with appropriate macro expression.
- * @todo Eliminate from API.
- */
-/*@-redecl@*/
-/*@observer@*/ /*@null@*/
-extern const char * rpmGetVar(int var)
-	/*@*/;
-/*@=redecl@*/
-
-/** \ingroup rpmrc
- * Set value of an rpmrc variable.
- * @deprecated Use rpmDefineMacro() to change appropriate macro instead.
- * @todo Eliminate from API.
- */
-void rpmSetVar(int var, const char * val)
-	/*@globals internalState@*/
-	/*@modifies internalState @*/;
-
 /** \ingroup rpmrc
  * Build and install arch/os table identifiers.
- * @todo Eliminate from API.
+ * @deprecated Eliminate from API.
  */
 enum rpm_machtable_e {
     RPM_MACHTABLE_INSTARCH	= 0,	/*!< Install platform architecture. */
@@ -645,6 +542,13 @@ enum rpm_machtable_e {
 };
 #define	RPM_MACHTABLE_COUNT	4	/*!< No. of arch/os tables. */
 
+/** \ingroup rpmrc
+ * The default rpmrc files to be read.
+ * @deprecated The rpmrc files are no longer parsed by default.
+ */
+/*@observer@*/ /*@checked@*/
+extern const char *rpmRcfiles;
+
 /** \ingroup rpmrc
  * Read macro configuration file(s) for a target.
  * @param file		colon separated files to read (NULL uses default)
@@ -653,12 +557,14 @@ enum rpm_machtable_e {
  */
 int rpmReadConfigFiles(/*@null@*/ const char * file,
 		/*@null@*/ const char * target)
-	/*@globals rpmGlobalMacroContext, rpmCLIMacroContext, h_errno,
+	/*@globals rpmRcfiles,
+		rpmGlobalMacroContext, rpmCLIMacroContext, h_errno,
 		fileSystem, internalState @*/
 	/*@modifies rpmGlobalMacroContext, rpmCLIMacroContext,
 		fileSystem, internalState @*/;
 
 /** \ingroup rpmrc
+ * @deprecated Expand %{_host_cpu}/%{_target_cpu} for the canonical/current cpu.
  * Return current arch name and/or number.
  * @todo Generalize to extract arch component from target_platform macro.
  * @retval name		address of arch name (or NULL)
@@ -669,6 +575,7 @@ void rpmGetArchInfo( /*@null@*/ /*@out@*/ const char ** name,
 	/*@modifies *name, *num @*/;
 
 /** \ingroup rpmrc
+ * @deprecated Expand %{_host_os}/%{_target_os} for the canonical/current os.
  * Return current os name and/or number.
  * @todo Generalize to extract os component from target_platform macro.
  * @retval name		address of os name (or NULL)
@@ -679,13 +586,13 @@ void rpmGetOsInfo( /*@null@*/ /*@out@*/ const char ** name,
 	/*@modifies *name, *num @*/;
 
 /** \ingroup rpmrc
+ * @deprecated Use rpmPlatformScore instead.
  * Return arch/os score of a name.
  * An arch/os score measures the "nearness" of a name to the currently
  * running (or defined) platform arch/os. For example, the score of arch
  * "i586" on an i686 platform is (usually) 2. The arch/os score is used
  * to select one of several otherwise identical packages using the arch/os
  * tags from the header as hints of the intended platform for the package.
- * @todo Rewrite to use RE's against config.guess target platform output.
  *
  * @param type		any of the RPM_MACHTABLE_* constants
  * @param name		name
@@ -694,6 +601,25 @@ void rpmGetOsInfo( /*@null@*/ /*@out@*/ const char ** name,
 int rpmMachineScore(int type, const char * name)
 	/*@*/;
 
+/*@only@*/ /*@null@*/ /*@unchecked@*/
+extern void * platpat;
+/*@unchecked@*/
+extern int nplatpat;
+
+/** \ingroup rpmrc
+ * Return score of a platform string.
+ * A platform score measures the "nearness" of a platform string wrto
+ * configured platform patterns. The returned score is the line number
+ * of the 1st pattern in /etc/rpm/platform that matches the input string.
+ *
+ * @param platform	cpu-vendor-os platform string
+ * @param mi_re		pattern array (NULL uses /etc/rpm/platform patterns)
+ * @param mi_nre	no. of patterns
+ * @return		platform score (0 is no match, lower is preferred)
+ */
+int rpmPlatformScore(const char * platform, /*@null@*/ void * mi_re, int mi_nre)
+	/*@modifies mi_re @*/;
+
 /** \ingroup rpmrc
  * Display current rpmrc (and macro) configuration.
  * @param fp		output file handle
@@ -728,26 +654,13 @@ void rpmSetMachine(/*@null@*/ const char * arch, /*@null@*/ const char * os)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies rpmGlobalMacroContext, fileSystem, internalState @*/;
 
-/** \ingroup rpmrc
- * Return current arch/os names.
- * @deprecated Use rpmExpand on _target_* macros.
- * @todo Eliminate from API.
- *
- * @retval arch		address of arch name (or NULL)
- * @retval os		address of os name (or NULL)
- */
-/*@unused@*/
-void rpmGetMachine( /*@null@*/ /*@out@*/ const char **arch,
-		/*@null@*/ /*@out@*/ const char **os)
-	/*@modifies *arch, *os @*/;
-
 /** \ingroup rpmrc
  * Destroy rpmrc arch/os compatibility tables.
  * @todo Eliminate from API.
  */
 void rpmFreeRpmrc(void)
-	/*@globals internalState @*/
-	/*@modifies internalState @*/;
+	/*@globals platpat, nplatpat, internalState @*/
+	/*@modifies platpat, nplatpat, internalState @*/;
 
 /*@}*/
 /* ==================================================================== */
@@ -862,6 +775,7 @@ struct rpmRelocation_s {
 
 /**
  * Compare headers to determine which header is "newer".
+ * @deprecated Use rpmdsCompare instead.
  * @param first		1st header
  * @param second	2nd header
  * @return		result of comparison
@@ -996,8 +910,27 @@ rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
 	/*@modifies ts, fd, *specFilePtr, *cookie, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
 
+/** \ingroup rpmts
+ * Bit(s) to control rpmtsCheck() and rpmtsOrder() operation.
+ * @todo Move to rpmts.h.
+ */
+typedef enum rpmdepFlags_e {
+    RPMDEPS_FLAG_NONE		= 0,
+    RPMDEPS_FLAG_NOUPGRADE	= (1 <<  0),	/*!< from --noupgrade */
+    RPMDEPS_FLAG_NOREQUIRES	= (1 <<  1),	/*!< from --norequires */
+    RPMDEPS_FLAG_NOCONFLICTS	= (1 <<  2),	/*!< from --noconflicts */
+    RPMDEPS_FLAG_NOOBSOLETES	= (1 <<  3),	/*!< from --noobsoletes */
+    RPMDEPS_FLAG_NOPARENTDIRS	= (1 <<  4),	/*!< from --noparentdirs */
+    RPMDEPS_FLAG_NOLINKTOS	= (1 <<  5),	/*!< from --nolinktos */
+    RPMDEPS_FLAG_ANACONDA	= (1 <<  6),	/*!< from --anaconda */
+    RPMDEPS_FLAG_NOSUGGEST	= (1 <<  7),	/*!< from --nosuggest */
+    RPMDEPS_FLAG_ADDINDEPS	= (1 <<  8),	/*!< from --aid */
+    RPMDEPS_FLAG_DEPLOOPS	= (1 <<  9)	/*!< from --deploops */
+} rpmdepFlags;
+
 /** \ingroup rpmts
  * Bit(s) to control rpmtsRun() operation.
+ * @todo Move to rpmts.h.
  */
 typedef enum rpmtransFlags_e {
     RPMTRANS_FLAG_NONE		= 0,
@@ -1023,7 +956,7 @@ typedef enum rpmtransFlags_e {
 /*@-enummemuse@*/
     RPMTRANS_FLAG_UNDO		= (1 << 14),
 /*@=enummemuse@*/
-    RPMTRANS_FLAG_REVERSE	= (1 << 15),
+    /* 15 unused */
 
     RPMTRANS_FLAG_NOTRIGGERPREIN= (1 << 16),	/*!< from --notriggerprein */
     RPMTRANS_FLAG_NOPRE		= (1 << 17),	/*!< from --nopre */
@@ -1038,12 +971,11 @@ typedef enum rpmtransFlags_e {
 /*@=enummemuse@*/
     RPMTRANS_FLAG_APPLYONLY	= (1 << 25),
 
-    RPMTRANS_FLAG_ANACONDA	= (1 << 26),	/*!< from --anaconda */
+    /* 26 unused */
     RPMTRANS_FLAG_NOFDIGESTS	= (1 << 27),	/*!< from --nofdigests */
-    RPMTRANS_FLAG_NOSUGGEST	= (1 << 28),	/*!< from --nosuggest */
-    RPMTRANS_FLAG_ADDINDEPS	= (1 << 29),	/*!< from --aid */
+    /* 28-29 unused */
     RPMTRANS_FLAG_NOCONFIGS	= (1 << 30),	/*!< from --noconfigs */
-    RPMTRANS_FLAG_DEPLOOPS	= (1 << 31)	/*!< from --deploops */
+    /* 31 unused */
 } rpmtransFlags;
 
 #define	_noTransScripts		\
@@ -1060,16 +992,6 @@ typedef enum rpmtransFlags_e {
     RPMTRANS_FLAG_NOTRIGGERPOSTUN \
   )
 
-/** \ingroup rpmts
- * Segmented string compare for version and/or release.
- *
- * @param a		1st string
- * @param b		2nd string
- * @return		+1 if a is "newer", 0 if equal, -1 if b is "newer"
- */
-int rpmvercmp(const char * a, const char * b)
-	/*@*/;
-
 /*@}*/
 
 #if !defined(SWIG)
@@ -1146,38 +1068,6 @@ struct rpmlead {
 /*@unused@*/ char reserved[16];	/*!< Pad to 96 bytes -- 8 byte aligned! */
 } ;
 
-/* ==================================================================== */
-/** \name RPMEIU */
-/*@{*/
-/* --- install/upgrade/erase modes */
-
-/** \ingroup rpmcli
- * Bit(s) to control rpmInstall() operation.
- * @todo Move to rpmcli.h
- */
-typedef enum rpmInstallInterfaceFlags_e {
-    INSTALL_NONE	= 0,
-    INSTALL_PERCENT	= (1 << 0),	/*!< from --percent */
-    INSTALL_HASH	= (1 << 1),	/*!< from --hash */
-    INSTALL_NODEPS	= (1 << 2),	/*!< from --nodeps */
-    INSTALL_NOORDER	= (1 << 3),	/*!< from --noorder */
-    INSTALL_LABEL	= (1 << 4),	/*!< from --verbose (notify) */
-    INSTALL_UPGRADE	= (1 << 5),	/*!< from --upgrade */
-    INSTALL_FRESHEN	= (1 << 6),	/*!< from --freshen */
-    INSTALL_INSTALL	= (1 << 7),	/*!< from --install */
-    INSTALL_ERASE	= (1 << 8)	/*!< from --erase */
-} rpmInstallInterfaceFlags;
-
-/** \ingroup rpmcli
- * Bit(s) to control rpmErase() operation.
- */
-typedef enum rpmEraseInterfaceFlags_e {
-    UNINSTALL_NONE	= 0,
-    UNINSTALL_NODEPS	= (1 << 0),	/*!< from --nodeps */
-    UNINSTALL_ALLMATCHES= (1 << 1)	/*!< from --allmatches */
-} rpmEraseInterfaceFlags;
-
-/*@}*/
 /* ==================================================================== */
 /** \name RPMK */
 /*@{*/
