@@ -24,11 +24,7 @@ int _rpmte_debug = 0;
 
 void rpmteCleanDS(rpmte te)
 {
-    te->this = rpmdsFree(te->this);
-    te->provides = rpmdsFree(te->provides);
-    te->requires = rpmdsFree(te->requires);
-    te->conflicts = rpmdsFree(te->conflicts);
-    te->obsoletes = rpmdsFree(te->obsoletes);
+    te->PRCO = rpmdsFreePRCO(te->PRCO);
 }
 
 /**
@@ -144,22 +140,11 @@ static void addTE(rpmts ts, rpmte p, Header h,
 
     arch = NULL;
     xx = hge(h, RPMTAG_ARCH, NULL, (void **)&arch, NULL);
-    if (arch != NULL) {
-	p->arch = xstrdup(arch);
-	p->archScore = rpmMachineScore(RPM_MACHTABLE_INSTARCH, arch);
-    } else {
-	p->arch = NULL;
-	p->archScore = 0;
-    }
+    p->arch = (arch != NULL ? xstrdup(arch) : NULL);
     os = NULL;
     xx = hge(h, RPMTAG_OS, NULL, (void **)&os, NULL);
-    if (os != NULL) {
-	p->os = xstrdup(os);
-	p->osScore = rpmMachineScore(RPM_MACHTABLE_INSTOS, os);
-    } else {
-	p->os = NULL;
-	p->osScore = 0;
-    }
+    p->os = (os != NULL ? xstrdup(os) : NULL);
+
     p->isSource = (headerIsEntry(h, RPMTAG_SOURCERPM) == 0);
 
     nb = strlen(p->NEVR) + 1;
@@ -216,13 +201,7 @@ static void addTE(rpmts ts, rpmte p, Header h,
 
     p->pkgFileSize = 0;
 
-    p->this = rpmdsThis(h, RPMTAG_PROVIDENAME, RPMSENSE_EQUAL);
-    p->provides = rpmdsNew(h, RPMTAG_PROVIDENAME, scareMem);
-    p->requires = rpmdsNew(h, RPMTAG_REQUIRENAME, scareMem);
-    p->conflicts = rpmdsNew(h, RPMTAG_CONFLICTNAME, scareMem);
-    p->obsoletes = rpmdsNew(h, RPMTAG_OBSOLETENAME, scareMem);
-    p->triggers = NULL;	/* XXX NOTYET */
-    p->dirnames = rpmdsNew(h, RPMTAG_DIRNAMES, scareMem);
+    p->PRCO = rpmdsNewPRCO(h);
 
     savep = rpmtsSetRelocateElement(ts, p);
     p->fi = rpmfiNew(ts, h, RPMTAG_BASENAMES, scareMem);
@@ -547,26 +526,7 @@ fnpyKey rpmteKey(rpmte te)
 
 rpmds rpmteDS(rpmte te, rpmTag tag)
 {
-    /*@-compdef -refcounttrans -retalias -retexpose -usereleased @*/
-    if (te == NULL)
-	return NULL;
-
-    if (tag == RPMTAG_NAME)
-	return te->this;
-    if (tag == RPMTAG_PROVIDENAME)
-	return te->provides;
-    if (tag == RPMTAG_REQUIRENAME)
-	return te->requires;
-    if (tag == RPMTAG_CONFLICTNAME)
-	return te->conflicts;
-    if (tag == RPMTAG_OBSOLETENAME)
-	return te->obsoletes;
-    if (tag == RPMTAG_TRIGGERNAME)
-	return te->triggers;
-    if (tag == RPMTAG_DIRNAMES)
-	return te->dirnames;
-    return NULL;
-    /*@=compdef =refcounttrans =retalias =retexpose =usereleased @*/
+    return (te != NULL ? rpmdsFromPRCO(te->PRCO, tag) : NULL);
 }
 
 rpmfi rpmteFI(rpmte te, rpmTag tag)
@@ -758,7 +718,7 @@ rpmtsi XrpmtsiInit(rpmts ts, const char * fn, unsigned int ln)
 
     tsi = xcalloc(1, sizeof(*tsi));
     tsi->ts = rpmtsLink(ts, "rpmtsi");
-    tsi->reverse = ((rpmtsFlags(ts) & RPMTRANS_FLAG_REVERSE) ? 1 : 0);
+    tsi->reverse = 0;
     tsi->oc = (tsi->reverse ? (rpmtsNElements(ts) - 1) : 0);
     tsi->ocsave = tsi->oc;
 /*@-modfilesys@*/
