@@ -96,8 +96,6 @@ struct rpmte_s {
     const char * arch;		/*!< Architecture hint. */
 /*@only@*/ /*@null@*/
     const char * os;		/*!< Operating system hint. */
-    int archScore;		/*!< (TR_ADDED) Arch score. */
-    int osScore;		/*!< (TR_ADDED) Os score. */
     int isSource;		/*!< (TR_ADDED) source rpm? */
 
     rpmte parent;		/*!< Parent transaction element. */
@@ -110,20 +108,9 @@ struct rpmte_s {
 /*@owned@*/
     tsortInfo tsi;		/*!< Dependency ordering chains. */
 
-/*@refcounted@*/ /*@null@*/
-    rpmds this;			/*!< This package's provided NEVR. */
-/*@refcounted@*/ /*@null@*/
-    rpmds provides;		/*!< Provides: dependencies. */
-/*@refcounted@*/ /*@null@*/
-    rpmds requires;		/*!< Requires: dependencies. */
-/*@refcounted@*/ /*@null@*/
-    rpmds conflicts;		/*!< Conflicts: dependencies. */
-/*@refcounted@*/ /*@null@*/
-    rpmds obsoletes;		/*!< Obsoletes: dependencies. */
-/*@refcounted@*/ /*@null@*/
-    rpmds triggers;		/*!< Triggers: dependencies. */
-/*@refcounted@*/ /*@null@*/
-    rpmds dirnames;		/*!< Dirnames: dependencies. */
+/*@null@*/
+    rpmPRCO PRCO;		/*!< Current dependencies. */
+
 /*@refcounted@*/ /*@null@*/
     rpmfi fi;			/*!< File information. */
 
@@ -596,6 +583,8 @@ void rpmteColorDS(rpmte te, rpmTag tag)
 int rpmteChain(rpmte p, rpmte q, Header oh, /*@null@*/ const char * msg)
 	/*@modifies p, q, oh @*/;
 
+#define	RPMTE_CHAIN_END	"CHAIN END"	/*!< End of chain marker. */
+
 /**
  * Return transaction element index.
  * @param tsi		transaction element iterator
