@@ -15,6 +15,7 @@
 #include "rpmds.h"
 #include "rpmfi.h"
 #include "rpmlock.h"
+#include "rpmns.h"
 
 #define	_RPMTE_INTERNAL		/* XXX te->h */
 #include "rpmte.h"
@@ -71,75 +72,6 @@ int _rpmts_debug = 0;
 /*@unchecked@*/
 int _rpmts_stats = 0;
 
-char * hGetNEVR(Header h, const char ** np)
-{
-    const char * n, * v, * r;
-    char * NVR, * t;
-
-    (void) headerNVR(h, &n, &v, &r);
-    NVR = t = xcalloc(1, strlen(n) + strlen(v) + strlen(r) + sizeof("--"));
-/*@-boundswrite@*/
-    t = stpcpy(t, n);
-    t = stpcpy(t, "-");
-    t = stpcpy(t, v);
-    t = stpcpy(t, "-");
-    t = stpcpy(t, r);
-    if (np)
-	*np = n;
-/*@=boundswrite@*/
-    return NVR;
-}
-
-char * hGetNEVRA(Header h, const char ** np)
-{
-    const char * n, * v, * r, * a;
-    char * NVRA, * t;
-
-    (void) headerNVR(h, &n, &v, &r);
-    /* XXX pubkeys have no arch. */
-/*@-branchstate@*/
-    a = NULL;
-    if (!headerGetEntry(h, RPMTAG_ARCH, NULL, (void **) &a, NULL) || a == NULL)
-	a = "pubkey";
-/*@=branchstate@*/
-    NVRA = t = xcalloc(1, strlen(n) + strlen(v) + strlen(r) + strlen(a) + sizeof("--."));
-/*@-boundswrite@*/
-    t = stpcpy(t, n);
-    t = stpcpy(t, "-");
-    t = stpcpy(t, v);
-    t = stpcpy(t, "-");
-    t = stpcpy(t, r);
-    t = stpcpy(t, ".");
-    t = stpcpy(t, a);
-    if (np)
-	*np = n;
-/*@=boundswrite@*/
-    return NVRA;
-}
-
-uint_32 hGetColor(Header h)
-{
-    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
-    uint_32 hcolor = 0;
-    uint_32 * fcolors;
-    int_32 ncolors;
-    int i;
-
-    fcolors = NULL;
-    ncolors = 0;
-    if (hge(h, RPMTAG_FILECOLORS, NULL, (void **)&fcolors, &ncolors)
-     && fcolors != NULL && ncolors > 0)
-    {
-/*@-boundsread@*/
-	for (i = 0; i < ncolors; i++)
-	    hcolor |= fcolors[i];
-/*@=boundsread@*/
-    }
-    hcolor &= 0x0f;
-
-    return hcolor;
-}
-
 rpmts XrpmtsUnlink(rpmts ts, const char * msg, const char * fn, unsigned ln)
 {
 /*@-modfilesys@*/
@@ -222,41 +154,6 @@ int rpmtsVerifyDB(rpmts ts)
     return rpmdbVerify(ts->rootDir);
 }
 
-/*@-boundsread@*/
-static int isArch(const char * arch)
-	/*@*/
-{
-    const char ** av;
-/*@-nullassign@*/
-    /*@observer@*/
-    static const char *arches[] = {
-	"i386", "i486", "i586", "i686", "athlon", "pentium3", "pentium4", "x86_64", "amd64", "ia32e",
-	"alpha", "alphaev5", "alphaev56", "alphapca56", "alphaev6", "alphaev67",
-	"sparc", "sun4", "sun4m", "sun4c", "sun4d", "sparcv8", "sparcv9",
-	"sparc64", "sun4u",
-	"mips", "mipsel", "IP",
-	"ppc", "ppciseries", "ppcpseries",
-	"ppc64", "ppc64iseries", "ppc64pseries",
-	"m68k",
-	"rs6000",
-	"ia64",
-	"armv3l", "armv4b", "armv4l",
-	"armv5teb", "armv5tel",
-	"s390", "i370", "s390x",
-	"sh", "xtensa",
-	"noarch",
-	NULL,
-    };
-/*@=nullassign@*/
-
-    for (av = arches; *av != NULL; av++) {
-	if (!strcmp(arch, *av))
-	    return 1;
-    }
-    return 0;
-}
-/*@=boundsread@*/
-
 /*@-compdef@*/ /* keyp might no be defined. */
 rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
 			const void * keyp, size_t keylen)
@@ -320,7 +217,7 @@ rpmdbMatchIterator rpmtsInitIterator(const rpmts ts, rpmTag rpmtag,
 	t = (char *) keyp;
 	t = strrchr(t, '.');
 	/* Is this a valid ".arch" suffix? */
-	if (t != NULL && isArch(t+1)) {
+	if (t != NULL && rpmnsArch(t+1)) {
 	   *t++ = '\0';
 	   arch = t;
 	}
@@ -554,25 +451,33 @@ static int sugcmp(const void * a, const void * b)
 int rpmtsSolve(rpmts ts, rpmds ds, /*@unused@*/ const void * data)
 {
     const char * errstr;
-    const char * str;
+    const char * str = NULL;
     const char * qfmt;
     rpmdbMatchIterator mi;
-    Header bh;
-    Header h;
-    size_t bhnamelen;
-    time_t bhtime;
+    Header bh = NULL;
+    Header h = NULL;
+    size_t bhnamelen = 0;
+    time_t bhtime = 0;
     rpmTag rpmtag;
     const char * keyp;
-    size_t keylen;
+    size_t keylen = 0;
     int rc = 1;	/* assume not found */
     int xx;
 
-    /* Make suggestions only for install Requires: */
+    /* Make suggestions only for installing Requires: */
     if (ts->goal != TSM_INSTALL)
 	return rc;
 
-    if (rpmdsTagN(ds) != RPMTAG_REQUIRENAME)
+    switch (rpmdsTagN(ds)) {
+    case RPMTAG_CONFLICTNAME:
+    default:
 	return rc;
+	/*@notreached@*/ break;
+    case RPMTAG_DIRNAMES:	/* XXX perhaps too many wrong answers */
+    case RPMTAG_REQUIRENAME:
+    case RPMTAG_FILELINKTOS:
+	break;
+    }
 
     keyp = rpmdsN(ds);
     if (keyp == NULL)
@@ -585,11 +490,7 @@ int rpmtsSolve(rpmts ts, rpmds ds, /*@unused@*/ const void * data)
 
     /* Look for a matching Provides: in suggested universe. */
     rpmtag = (*keyp == '/' ? RPMTAG_BASENAMES : RPMTAG_PROVIDENAME);
-    keylen = 0;
     mi = rpmdbInitIterator(ts->sdb, rpmtag, keyp, keylen);
-    bhnamelen = 0;
-    bhtime = 0;
-    bh = NULL;
     while ((h = rpmdbNextIterator(mi)) != NULL) {
 	const char * hname;
 	size_t hnamelen;
@@ -599,15 +500,17 @@ int rpmtsSolve(rpmts ts, rpmds ds, /*@unused@*/ const void * data)
 	if (rpmtag == RPMTAG_PROVIDENAME && !rpmdsAnyMatchesDep(h, ds, 1))
 	    continue;
 
-	/* XXX Prefer the shortest name if given alternatives. */
 	hname = NULL;
 	hnamelen = 0;
 	if (headerGetEntry(h, RPMTAG_NAME, NULL, (void **)&hname, NULL)) {
 	    if (hname)
 		hnamelen = strlen(hname);
 	}
-	if (bhnamelen > 0 && hnamelen > bhnamelen)
-	    continue;
+
+	/* XXX Prefer the shortest pkg N for basenames/provides resp. */
+	if (bhnamelen > 0)
+	    if (hnamelen > bhnamelen)
+		continue;
 
 	/* XXX Prefer the newest build if given alternatives. */
 	htime = 0;
@@ -617,6 +520,7 @@ int rpmtsSolve(rpmts ts, rpmds ds, /*@unused@*/ const void * data)
 	if (htime <= bhtime)
 	    continue;
 
+	/* Save new "best" candidate. */
 	bh = headerFree(bh);
 	bh = headerLink(h);
 	bhtime = htime;
@@ -628,7 +532,7 @@ int rpmtsSolve(rpmts ts, rpmds ds, /*@unused@*/ const void * data)
     if (bh == NULL)
 	goto exit;
 
-    /* Format the suggestion. */
+    /* Format the suggested resolution path. */
     qfmt = rpmExpand("%{?_solve_name_fmt}", NULL);
     if (qfmt == NULL || *qfmt == '\0')
 	goto exit;
@@ -636,16 +540,15 @@ int rpmtsSolve(rpmts ts, rpmds ds, /*@unused@*/ const void * data)
     bh = headerFree(bh);
     qfmt = _free(qfmt);
     if (str == NULL) {
-	rpmError(RPMERR_QFMT, _("incorrect format: %s\n"), errstr);
+	rpmError(RPMERR_QFMT, _("incorrect solve path format: %s\n"), errstr);
 	goto exit;
     }
 
-    if (ts->transFlags & RPMTRANS_FLAG_ADDINDEPS) {
+    if (ts->depFlags & RPMDEPS_FLAG_ADDINDEPS) {
 	FD_t fd;
 	rpmRC rpmrc;
 
-	h = headerFree(h);
-	fd = Fopen(str, "r.ufdio");
+	fd = Fopen(str, "r");
 	if (fd == NULL || Ferror(fd)) {
 	    rpmError(RPMERR_OPEN, _("open of %s failed: %s\n"), str,
 			Fstrerror(fd));
@@ -660,7 +563,6 @@ int rpmtsSolve(rpmts ts, rpmds ds, /*@unused@*/ const void * data)
 	xx = Fclose(fd);
 	switch (rpmrc) {
 	default:
-	    str = _free(str);
 	    break;
 	case RPMRC_NOTTRUSTED:
 	case RPMRC_NOKEY:
@@ -669,13 +571,12 @@ int rpmtsSolve(rpmts ts, rpmds ds, /*@unused@*/ const void * data)
 	        !rpmtsAddInstallElement(ts, h, (fnpyKey)str, 1, NULL))
 	    {
 		rpmMessage(RPMMESS_DEBUG, _("Adding: %s\n"), str);
-		rc = -1;
-		/* XXX str memory leak */
+		rc = -1;	/* XXX restart unsatisfiedDepends() */
 		break;
 	    }
-	    str = _free(str);
 	    break;
 	}
+	str = _free(str);
 	h = headerFree(h);
 	goto exit;
     }
@@ -685,7 +586,10 @@ int rpmtsSolve(rpmts ts, rpmds ds, /*@unused@*/ const void * data)
     if (ts->suggests != NULL && ts->nsuggests > 0) {
 	if (bsearch(&str, ts->suggests, ts->nsuggests,
 			sizeof(*ts->suggests), sugcmp))
+	{
+	    str = _free(str);
 	    goto exit;
+	}
     }
 
     /* Add a new (unique) suggestion. */
@@ -753,7 +657,7 @@ rpmps rpmtsProblems(rpmts ts)
     rpmps ps = NULL;
     if (ts) {
 	if (ts->probs)
-	    ps = rpmpsLink(ts->probs, NULL);
+	    ps = rpmpsLink(ts->probs, __FUNCTION__);
     }
     return ps;
 }
@@ -870,6 +774,8 @@ rpmts rpmtsFree(rpmts ts)
     rpmtsEmpty(ts);
 /*@=nullstate@*/
 
+    ts->PRCO = rpmdsFreePRCO(ts->PRCO);
+
     (void) rpmtsCloseDB(ts);
 
     (void) rpmtsCloseSDB(ts);
@@ -1116,7 +1022,7 @@ int rpmtsSetREContext(rpmts ts, rpmsx sx)
 
 int_32 rpmtsGetTid(rpmts ts)
 {
-    int_32 tid = 0;
+    int_32 tid = -1;	/* XXX -1 is time(2) error return. */
     if (ts != NULL) {
 	tid = ts->tid;
     }
@@ -1125,7 +1031,7 @@ int_32 rpmtsGetTid(rpmts ts)
 
 int_32 rpmtsSetTid(rpmts ts, int_32 tid)
 {
-    int_32 otid = 0;
+    int_32 otid = -1;	/* XXX -1 is time(2) error return. */
     if (ts != NULL) {
 	otid = ts->tid;
 	ts->tid = tid;
@@ -1221,6 +1127,13 @@ rpmdb rpmtsGetRdb(rpmts ts)
 /*@=compdef =refcounttrans =usereleased @*/
 }
 
+rpmPRCO rpmtsPRCO(rpmts ts)
+{
+/*@-compdef -retexpose -usereleased @*/
+    return (ts != NULL ? ts->PRCO : NULL);
+/*@=compdef =retexpose =usereleased @*/
+}
+
 int rpmtsInitDSI(const rpmts ts)
 {
     rpmDiskSpaceInfo dsi;
@@ -1230,6 +1143,8 @@ int rpmtsInitDSI(const rpmts ts)
 
     if (rpmtsFilterFlags(ts) & RPMPROB_FILTER_DISKSPACE)
 	return 0;
+    if (ts->filesystems != NULL)
+	return 0;
 
     rpmMessage(RPMMESS_DEBUG, _("mounted filesystems:\n"));
     rpmMessage(RPMMESS_DEBUG,
@@ -1275,32 +1190,48 @@ int rpmtsInitDSI(const rpmts ts)
 	dsi->dev = sb.st_dev;
 /* XXX figger out how to get this info for non-statvfs systems. */
 #if STATFS_IN_SYS_STATVFS
+	dsi->f_frsize = sfb.f_frsize;
 	dsi->f_fsid = sfb.f_fsid;
 	dsi->f_flag = sfb.f_flag;
+	dsi->f_favail = sfb.f_favail;
 	dsi->f_namemax = sfb.f_namemax;
+#else
+	dsi->f_fsid = sfb.f_fsid;
+	dsi->f_namemax = sfb.f_namelen;
 #endif
 
-	dsi->bsize = sfb.f_bsize;
+	dsi->f_bsize = sfb.f_bsize;
+	dsi->f_blocks = sfb.f_blocks;
+	dsi->f_bfree = sfb.f_bfree;
+	dsi->f_files = sfb.f_files;
+	dsi->f_ffree = sfb.f_ffree;
+
 	dsi->bneeded = 0;
 	dsi->ineeded = 0;
 #ifdef STATFS_HAS_F_BAVAIL
-	dsi->bavail = sfb.f_bavail;
+	dsi->f_bavail = sfb.f_bavail;
+	if (sfb.f_ffree > 0 && sfb.f_files > 0 && sfb.f_favail > 0)
+	    dsi->f_favail = sfb.f_favail;
+	else	/* XXX who knows what evil lurks here? */
+	    dsi->f_favail = !(sfb.f_ffree == 0 && sfb.f_files == 0)
+				? sfb.f_ffree : -1;
 #else
 /* FIXME: the statfs struct doesn't have a member to tell how many blocks are
  * available for non-superusers.  f_blocks - f_bfree is probably too big, but
  * it's about all we can do.
  */
-	dsi->bavail = sfb.f_blocks - sfb.f_bfree;
-#endif
+	dsi->f_bavail = sfb.f_blocks - sfb.f_bfree;
 	/* XXX Avoid FAT and other file systems that have not inodes. */
-	dsi->iavail = !(sfb.f_ffree == 0 && sfb.f_files == 0)
+	dsi->f_favail = !(sfb.f_ffree == 0 && sfb.f_files == 0)
 				? sfb.f_ffree : -1;
+#endif
+
 #if !defined(ST_RDONLY)
 #define	ST_RDONLY	1
 #endif
 	rpmMessage(RPMMESS_DEBUG, _("%5d 0x%08x %8u %12ld %12ld %s %s\n"),
-		i, (unsigned) dsi->dev, (unsigned) dsi->bsize,
-		(signed long) dsi->bavail, (signed long) dsi->iavail,
+		i, (unsigned) dsi->dev, (unsigned) dsi->f_bsize,
+		(signed long) dsi->f_bavail, (signed long) dsi->f_favail,
 		((dsi->f_flag & ST_RDONLY) ? "ro" : "rw"),
 		ts->filesystems[i]);
     }
@@ -1312,19 +1243,19 @@ void rpmtsUpdateDSI(const rpmts ts, dev_t dev,
 		fileAction action)
 {
     rpmDiskSpaceInfo dsi;
-    uint_32 bneeded;
+    unsigned long long bneeded;
 
     dsi = ts->dsi;
     if (dsi) {
-	while (dsi->bsize && dsi->dev != dev)
+	while (dsi->f_bsize && dsi->dev != dev)
 	    dsi++;
-	if (dsi->bsize == 0)
+	if (dsi->f_bsize == 0)
 	    dsi = NULL;
     }
     if (dsi == NULL)
 	return;
 
-    bneeded = BLOCK_ROUND(fileSize, dsi->bsize);
+    bneeded = BLOCK_ROUND(fileSize, dsi->f_bsize);
 
     switch (action) {
     case FA_BACKUP:
@@ -1341,7 +1272,7 @@ void rpmtsUpdateDSI(const rpmts ts, dev_t dev,
      */
     case FA_CREATE:
 	dsi->bneeded += bneeded;
-	dsi->bneeded -= BLOCK_ROUND(prevSize, dsi->bsize);
+	dsi->bneeded -= BLOCK_ROUND(prevSize, dsi->f_bsize);
 	/*@switchbreak@*/ break;
 
     case FA_ERASE:
@@ -1354,7 +1285,7 @@ void rpmtsUpdateDSI(const rpmts ts, dev_t dev,
     }
 
     if (fixupSize)
-	dsi->bneeded -= BLOCK_ROUND(fixupSize, dsi->bsize);
+	dsi->bneeded -= BLOCK_ROUND(fixupSize, dsi->f_bsize);
 }
 
 void rpmtsCheckDSIProblems(const rpmts ts, const rpmte te)
@@ -1377,18 +1308,18 @@ void rpmtsCheckDSIProblems(const rpmts ts, const rpmte te)
     ps = rpmtsProblems(ts);
     for (i = 0; i < ts->filesystemCount; i++, dsi++) {
 
-	if (dsi->bavail > 0 && adj_fs_blocks(dsi->bneeded) > dsi->bavail) {
+	if (dsi->f_bavail > 0 && adj_fs_blocks(dsi->bneeded) > dsi->f_bavail) {
 	    rpmpsAppend(ps, RPMPROB_DISKSPACE,
 			rpmteNEVR(te), rpmteKey(te),
 			ts->filesystems[i], NULL, NULL,
- 	   (adj_fs_blocks(dsi->bneeded) - dsi->bavail) * dsi->bsize);
+ 	   (adj_fs_blocks(dsi->bneeded) - dsi->f_bavail) * dsi->f_bsize);
 	}
 
-	if (dsi->iavail > 0 && adj_fs_blocks(dsi->ineeded) > dsi->iavail) {
+	if (dsi->f_favail > 0 && adj_fs_blocks(dsi->ineeded) > dsi->f_favail) {
 	    rpmpsAppend(ps, RPMPROB_DISKNODES,
 			rpmteNEVR(te), rpmteKey(te),
 			ts->filesystems[i], NULL, NULL,
- 	    (adj_fs_blocks(dsi->ineeded) - dsi->iavail));
+ 	    (adj_fs_blocks(dsi->ineeded) - dsi->f_favail));
 	}
 
 	if ((dsi->bneeded || dsi->ineeded) && (dsi->f_flag & ST_RDONLY)) {
@@ -1457,6 +1388,21 @@ rpmtransFlags rpmtsSetFlags(rpmts ts, rpmtransFlags transFlags)
     return otransFlags;
 }
 
+rpmdepFlags rpmtsDFlags(rpmts ts)
+{
+    return (ts != NULL ? ts->depFlags : 0);
+}
+
+rpmdepFlags rpmtsSetDFlags(rpmts ts, rpmdepFlags depFlags)
+{
+    rpmdepFlags odepFlags = 0;
+    if (ts != NULL) {
+	odepFlags = ts->depFlags;
+	ts->depFlags = depFlags;
+    }
+    return odepFlags;
+}
+
 Spec rpmtsSpec(rpmts ts)
 {
 /*@-compdef -retexpose -usereleased@*/
@@ -1504,6 +1450,11 @@ uint_32 rpmtsSetColor(rpmts ts, uint_32 color)
     return ocolor;
 }
 
+uint_32 rpmtsPrefColor(rpmts ts)
+{
+    return (ts != NULL ? ts->prefcolor : 0);
+}
+
 rpmop rpmtsOp(rpmts ts, rpmtsOpX opx)
 {
     rpmop op = NULL;
@@ -1528,6 +1479,7 @@ int rpmtsSetNotifyCallback(rpmts ts,
 rpmts rpmtsCreate(void)
 {
     rpmts ts;
+    int xx;
 
     ts = xcalloc(1, sizeof(*ts));
     memset(&ts->ops, 0, sizeof(ts->ops));
@@ -1542,6 +1494,14 @@ rpmts rpmtsCreate(void)
     ts->solveData = NULL;
     ts->nsuggests = 0;
     ts->suggests = NULL;
+
+    ts->PRCO = rpmdsNewPRCO(NULL);
+    {	const char * fn = rpmGetPath("%{?_rpmds_sysinfo_path}", NULL);
+	if (fn && *fn != '\0' && !rpmioAccess(fn, NULL, R_OK))
+	   xx = rpmdsSysinfo(ts->PRCO, NULL);
+	fn = _free(fn);
+    }
+
     ts->sdb = NULL;
     ts->sdbmode = O_RDONLY;
 
@@ -1553,6 +1513,8 @@ rpmts rpmtsCreate(void)
     ts->delta = 5;
 
     ts->color = rpmExpandNumeric("%{?_transaction_color}");
+    ts->prefcolor = rpmExpandNumeric("%{?_prefer_color}");
+    if (!ts->prefcolor) ts->prefcolor = 0x2;
 
     ts->numRemovedPackages = 0;
     ts->allocedRemovedPackages = ts->delta;
