@@ -112,17 +112,24 @@ extern int _cacheDependsRC;
 typedef	/*@abstract@*/ struct diskspaceInfo_s * rpmDiskSpaceInfo;
 
 /** \ingroup rpmts
+ * An internal copy of (linux) struct statvfs for portability, with extensions.
  */
 struct diskspaceInfo_s {
-    dev_t dev;			/*!< File system device number. */
+    unsigned long f_bsize;	/*!< File system block size. */
+    unsigned long f_frsize;	/*!< File system fragment size. */
+    unsigned long long f_blocks;/*!< File system size (in frsize units). */
+    unsigned long long f_bfree;	/*!< No. of free blocks. */
+    signed long long f_bavail;	/*!< No. of blocks available to non-root. */
+    unsigned long long f_files;	/*!< No. of inodes. */
+    unsigned long long f_ffree;	/*!< No. of free inodes. */
+    signed long long f_favail;	/*!< No. of inodes available to non-root. */
     unsigned long f_fsid;	/*!< File system id. */
     unsigned long f_flag;	/*!< Mount flags. */
     unsigned long f_namemax;	/*!< Maximum filename length. */
-    signed long bneeded;	/*!< No. of blocks needed. */
-    signed long ineeded;	/*!< No. of inodes needed. */
-    int bsize;			/*!< File system block size. */
-    signed long long bavail;	/*!< No. of blocks available. */
-    signed long long iavail;	/*!< No. of inodes available. */
+
+    signed long long bneeded;	/*!< No. of blocks needed. */
+    signed long long ineeded;	/*!< No. of inodes needed. */
+    dev_t dev;			/*!< File system device number. */
 };
 
 /** \ingroup rpmts
@@ -130,8 +137,6 @@ struct diskspaceInfo_s {
  */
 #define	adj_fs_blocks(_nb)	(((_nb) * 21) / 20)
 
-/* argon thought a shift optimization here was a waste of time...  he's
-   probably right :-( */
 #define BLOCK_ROUND(size, block) (((size) + (block) - 1) / (block))
 
 /** \ingroup rpmts
@@ -146,7 +151,8 @@ typedef enum tsStage_e {
  * The set of packages to be installed/removed atomically.
  */
 struct rpmts_s {
-    rpmtransFlags transFlags;	/*!< Bit(s) to control operation. */
+    rpmdepFlags depFlags;	/*!< Bit(s) to control rpmtsCheck(). */
+    rpmtransFlags transFlags;	/*!< Bit(s) to control rpmtsRun(). */
     tsmStage goal;		/*!< Transaction goal (i.e. mode) */
     rpmTSType type;		/*!< default, rollback, autorollback */
 
@@ -167,7 +173,8 @@ struct rpmts_s {
 /*@observer@*/ /*@null@*/
     rpmCallbackData notifyData;	/*!< Callback private data. */
 
-    struct rpmPRCO_s PRCO;	/*!< Current transaction dependencies. */
+/*@null@*/
+    rpmPRCO PRCO;		/*!< Current transaction dependencies. */
 
 /*@refcounted@*/ /*@null@*/
     rpmps probs;		/*!< Current problems in transaction. */
@@ -236,6 +243,7 @@ struct rpmts_s {
     int_32 tid;			/*!< Transaction id. */
 
     uint_32 color;		/*!< Transaction color bits. */
+    uint_32 prefcolor;		/*!< Preferred file color. */
 
     rpmVSFlags vsflags;		/*!< Signature/digest verification flags. */
 
@@ -284,6 +292,9 @@ extern "C" {
 int rpmtsCheck(rpmts ts)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState @*/;
+int rpmrbCheck(rpmts ts)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState @*/;
 
 /** \ingroup rpmts
  * Determine package order in a transaction set according to dependencies.
@@ -304,6 +315,9 @@ int rpmtsCheck(rpmts ts)
 int rpmtsOrder(rpmts ts)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState @*/;
+int rpmrbOrder(rpmts ts)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState @*/;
 
 /** \ingroup rpmts
  * Process all package elements in a transaction set.  Before calling
@@ -325,21 +339,22 @@ int rpmtsOrder(rpmts ts)
 int rpmtsRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState @*/;
-
-/**
- * Function to perform autorollback goal.  Today is called automatically
- * from rpmtsCheck(), rpmtsOrder() and rpmtsRun().  May change to be a 
- * programmer decision.
- *
- * @param failedTransaction	Failed transaction.
- * @param rbts			rollback transaction (can be NULL).
- * @param ignoreSet		Problems to ignore.
- * @return			RPMRC_OK, or RPMRC_FAIL
- */
-rpmRC rpmtsDoARBGoal(rpmts failedTransaction, rpmts rbts,
-    rpmprobFilterFlags ignoreSet)
+int rpmrbRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-	/*@modifies rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/;
+	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState @*/;
+
+/** \ingroup rpmts
+ * Rollback a failed transaction.
+ * @param rbts		failed transaction set
+ * @param ignoreSet     problems to ignore
+ * @param running       partial transaction?
+ * @param rbte		failed transaction element
+ * @return		RPMRC_OK, or RPMRC_FAIL
+ */
+rpmRC rpmtsRollback(rpmts rbts, rpmprobFilterFlags ignoreSet,
+		int running, rpmte rbte)
+    /*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+    /*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState @*/;
 
 /** \ingroup rpmts
  * Unreference a transaction instance.
@@ -811,6 +826,15 @@ pgpDigParams rpmtsPubkey(const rpmts ts)
 rpmdb rpmtsGetRdb(rpmts ts)
 	/*@*/;
 
+/** \ingroup rpmts
+ * Get transaction set dependencies.
+ * @param ts		transaction set
+ * @return		transaction set dependencies.
+ */
+/*@null@*/
+rpmPRCO rpmtsPRCO(rpmts ts)
+	/*@*/;
+
 /** \ingroup rpmts
  * Initialize disk space info for each and every mounted file systems.
  * @param ts		transaction set
@@ -901,6 +925,23 @@ rpmtransFlags rpmtsFlags(rpmts ts)
 rpmtransFlags rpmtsSetFlags(rpmts ts, rpmtransFlags transFlags)
 	/*@modifies ts @*/;
 
+/** \ingroup rpmts
+ * Get dependency flags, i.e. bits that control rpmtsCheck() and rpmtsOrder().
+ * @param ts		transaction set
+ * @return		dependency flags
+ */
+rpmdepFlags rpmtsDFlags(rpmts ts)
+	/*@*/;
+
+/** \ingroup rpmts
+ * Set dependency flags, i.e. bits that control rpmtsCheck() and rpmtsOrder().
+ * @param ts		transaction set
+ * @param depFlags	new dependency flags
+ * @return		previous dependency flags
+ */
+rpmdepFlags rpmtsSetDFlags(rpmts ts, rpmdepFlags depFlags)
+	/*@modifies ts @*/;
+
 /** \ingroup rpmts
  * Get spec control structure from transaction set.
  * @param ts		transaction set
@@ -947,6 +988,14 @@ rpmte rpmtsSetRelocateElement(rpmts ts, /*@null@*/ rpmte relocateElement)
 uint_32 rpmtsColor(rpmts ts)
 	/*@*/;
 
+/**
+ * Retrieve prefered file color
+ * @param ts		transaction set
+ * @return		color bits
+ */
+uint_32 rpmtsPrefColor(rpmts ts)
+	/*@*/;
+
 /**
  * Set color bits of transaction set.
  * @param ts		transaction set
@@ -988,8 +1037,8 @@ int rpmtsSetNotifyCallback(rpmts ts,
  */
 /*@newref@*/
 rpmts rpmtsCreate(void)
-	/*@globals rpmGlobalMacroContext, h_errno, internalState @*/
-	/*@modifies rpmGlobalMacroContext, internalState @*/;
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies rpmGlobalMacroContext, fileSystem, internalState @*/;
 
 /** \ingroup rpmts
  * Add package to be installed to transaction set.
@@ -1023,33 +1072,37 @@ int rpmtsAddEraseElement(rpmts ts, Header h, int dboffset)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, h, rpmGlobalMacroContext, fileSystem, internalState @*/;
 
+#if !defined(SWIG)
+#if defined(_RPMTS_INTERNAL)
 /**
- * Return (malloc'd) header name-version-release string.
- * @param h		header
- * @retval np		name tag value
- * @return		name-version-release string
+ * Print current transaction set contents.
+ * @param ts		transaction set
+ * @param fp		file handle (NULL uses stderr)
+ * @return		0 always
  */
-/*@only@*/
-char * hGetNEVR(Header h, /*@null@*/ /*@out@*/ const char ** np )
-	/*@modifies *np @*/;
+/*@unused@*/ static inline
+int rpmtsPrint(/*@null@*/ rpmts ts, /*@null@*/ FILE * fp)
+	/*@globals fileSystem @*/
+	/*@modifies ts, *fp, fileSystem @*/
+{
+    int tid = rpmtsGetTid(ts);
+    time_t ttid = tid;
+    rpmtsi tsi;
+    rpmte te;
 
-/**
- * Return (malloc'd) header name-version-release.arch string.
- * @param h		header
- * @retval np		name tag value
- * @return		name-version-release string
- */
-/*@only@*/
-char * hGetNEVRA(Header h, /*@null@*/ /*@out@*/ const char ** np )
-	/*@modifies *np @*/;
+    if (fp == NULL)
+	fp = stderr;
 
-/**
- * Return header color.
- * @param h		header
- * @return		header color
- */
-uint_32 hGetColor(Header h)
-	/*@modifies h @*/;
+    fprintf(fp, _("=== Transaction at %-24.24s (0x%08x):\n"), ctime(&ttid),tid);
+    tsi = rpmtsiInit(ts);
+    while ((te = rpmtsiNext(tsi, 0)) != NULL)
+        fprintf(fp, "t%s> %s\n", (rpmteType(te) == TR_ADDED ? "I" : "E"),
+		rpmteNEVRA(te));
+    tsi = rpmtsiFree(tsi);
+    return 0;
+}
+#endif	/* defined(_RPMTS_INTERNAL) */
+#endif	/* !defined(SWIG) */
 
 #ifdef __cplusplus
 }
