@@ -70,10 +70,10 @@ const char * rpmDetectPGPVersion(pgpVersion * pgpVer)
 {
     /* Actually this should support having more then one pgp version. */
     /* At the moment only one version is possible since we only       */
-    /* have one %_pgpbin and one %_pgp_path.                          */
+    /* have one %__pgp and one %_pgp_path.                          */
 
     static pgpVersion saved_pgp_version = PGP_UNKNOWN;
-    const char *pgpbin = rpmGetPath("%{?_pgpbin}", NULL);
+    const char *pgpbin = rpmGetPath("%{?__pgp}", NULL);
 
     if (saved_pgp_version == PGP_UNKNOWN) {
 	char *pgpvbin;
@@ -429,7 +429,6 @@ static int makePGPSignature(const char * file, /*@unused@*/ int_32 * sigTagp,
 	const char *path;
 	pgpVersion pgpVer;
 
-	(void) close(STDIN_FILENO);
 	(void) dup2(inpipe[0], 3);
 	(void) close(inpipe[1]);
 
@@ -577,7 +576,6 @@ static int makeGPGSignature(const char * file, int_32 * sigTagp,
     if (!(pid = fork())) {
 	const char *gpg_path = rpmExpand("%{?_gpg_path}", NULL);
 
-	(void) close(STDIN_FILENO);
 	(void) dup2(inpipe[0], 3);
 	(void) close(inpipe[1]);
 
