@@ -5,12 +5,12 @@
 
 #include "system.h"
 
+#define	_RPMPS_INTERNAL	/* XXX rpmps needs iterator. */
 #include <rpmcli.h>
 
 #include "psm.h"
 #include "rpmfi.h"
 
-#define	_RPMPS_INTERNAL	/* XXX rpmps needs iterator. */
 #include "rpmts.h"
 
 #include "legacy.h"	/* XXX dodigest(), uidToUname(), gnameToGid */
@@ -34,7 +34,6 @@ int rpmVerifyFile(const rpmts ts, const rpmfi fi,
     rpmVerifyAttrs flags = rpmfiVFlags(fi);
     const char * fn = rpmfiFN(fi);
     const char * rootDir = rpmtsRootDir(ts);
-    int selinuxEnabled = rpmtsSELinuxEnabled(ts);
     struct stat sb;
     int rc;
 
@@ -80,8 +79,6 @@ int rpmVerifyFile(const rpmts ts, const rpmfi fi,
 	return 1;
     }
 
-    flags |= RPMVERIFY_CONTEXTS;	/* no disable from package. */
-
     /*
      * Not all attributes of non-regular files can be verified.
      */
@@ -119,33 +116,6 @@ int rpmVerifyFile(const rpmts ts, const rpmfi fi,
      */
     flags &= ~(omitMask | RPMVERIFY_FAILURES);
 
-    /*
-     * Verify file security context.
-     */
-/*@-branchstate@*/
-    if (selinuxEnabled == 1 && (flags & RPMVERIFY_CONTEXTS)) {
-	security_context_t con;
-
-	rc = lgetfilecon(fn, &con);
-	if (rc == -1)
-	    *res |= (RPMVERIFY_LGETFILECONFAIL|RPMVERIFY_CONTEXTS);
-	else {
-	    rpmsx sx = rpmtsREContext(ts);
-	    const char * fcontext;
-
-	    if (sx != NULL) {
-		/* Get file security context from patterns. */
-		fcontext = rpmsxFContext(sx, fn, fmode);
-		sx = rpmsxFree(sx);
-	    } else {
-		/* Get file security context from package. */
-		fcontext = rpmfiFContext(fi);
-	    }
-	    if (fcontext == NULL || strcmp(fcontext, con))
-		*res |= RPMVERIFY_CONTEXTS;
-	    freecon(con);
-	}
-    }
 /*@=branchstate@*/
 
     if (flags & RPMVERIFY_FDIGEST) {
@@ -293,7 +263,6 @@ static int verifyHeader(QVA_t qva, const rpmts ts, rpmfi fi)
 	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies ts, fi, fileSystem, internalState  @*/
 {
-    int selinuxEnabled = rpmtsSELinuxEnabled(ts);
     rpmVerifyAttrs verifyResult = 0;
     /*@-type@*/ /* FIX: union? */
     rpmVerifyAttrs omitMask = ((qva->qva_flags & VERIFY_ATTRS) ^ VERIFY_ATTRS);
@@ -346,10 +315,9 @@ static int verifyHeader(QVA_t qva, const rpmts ts, rpmfi fi)
 	    }
 	} else if (verifyResult || rpmIsVerbose()) {
 	    const char * size, * digest, * link, * mtime, * mode;
-	    const char * group, * user, * rdev, *ctxt;
+	    const char * group, * user, * rdev;
 	    /*@observer@*/ static const char *const aok = ".";
 	    /*@observer@*/ static const char *const unknown = "?";
-	    /*@observer@*/ static const char *const ctxt_ignore = " ";
 
 	    ec = 1;
 
@@ -361,10 +329,6 @@ static int verifyHeader(QVA_t qva, const rpmts ts, rpmfi fi)
 #define	_verifyfile(_RPMVERIFY_F, _C)	\
 	((verifyResult & RPMVERIFY_READFAIL) ? unknown : \
 	 (verifyResult & _RPMVERIFY_F) ? _C : aok)
-#define	_verifyctxt(_RPMVERIFY_F, _C)	\
-	((selinuxEnabled != 1 ? ctxt_ignore : \
-	 (verifyResult & RPMVERIFY_LGETFILECONFAIL) ? unknown : \
-	 (verifyResult & _RPMVERIFY_F) ? _C : aok))
 	
 	    digest = _verifyfile(RPMVERIFY_FDIGEST, "5");
 	    size = _verify(RPMVERIFY_FILESIZE, "S");
@@ -374,15 +338,13 @@ static int verifyHeader(QVA_t qva, const rpmts ts, rpmfi fi)
 	    user = _verify(RPMVERIFY_USER, "U");
 	    group = _verify(RPMVERIFY_GROUP, "G");
 	    mode = _verify(RPMVERIFY_MODE, "M");
-	    ctxt = _verifyctxt(RPMVERIFY_CONTEXTS, "C");
 
-#undef _verifyctxt
 #undef _verifyfile
 #undef _verifylink
 #undef _verify
 
-	    sprintf(te, "%s%s%s%s%s%s%s%s%s %c %s",
-		size, mode, digest, rdev, link, user, group, mtime, ctxt,
+	    sprintf(te, "%s%s%s%s%s%s%s%s %c %s",
+		size, mode, digest, rdev, link, user, group, mtime,
 			((fflags & RPMFILE_CONFIG)	? 'c' :
 			 (fflags & RPMFILE_DOC)	? 'd' :
 			 (fflags & RPMFILE_GHOST)	? 'g' :
@@ -420,6 +382,7 @@ static int verifyDependencies(/*@unused@*/ QVA_t qva, rpmts ts,
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, h, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
+    int instance = headerGetInstance(h);
     rpmps ps;
     int numProblems;
     int rc = 0;		/* assume no problems */
@@ -427,7 +390,12 @@ static int verifyDependencies(/*@unused@*/ QVA_t qva, rpmts ts,
     int i;
 
     rpmtsEmpty(ts);
-    (void) rpmtsAddInstallElement(ts, h, NULL, 0, NULL);
+#ifdef	NOTYET
+    if (instance > 0)
+	(void) rpmtsAddEraseElement(ts, h, instance);
+    else
+#endif
+	(void) rpmtsAddInstallElement(ts, h, NULL, 0, NULL);
 
     xx = rpmtsCheck(ts);
     ps = rpmtsProblems(ts);
@@ -493,7 +461,7 @@ int showVerifyPackage(QVA_t qva, rpmts ts, Header h)
 #if defined(__x86_64__)
 static QVA_t Qva;
 #endif
-    int scareMem = 1;	/* XXX only rpmVerifyScript needs now */
+    int scareMem = 1;	/* XXX rpmpsmStage needs fi->h */
     rpmfi fi = NULL;
     int ec = 0;
     int rc;
@@ -540,7 +508,8 @@ qva = Qva;
 
 int rpmcliVerify(rpmts ts, QVA_t qva, const char ** argv)
 {
-    const char * arg;
+    rpmdepFlags depFlags = qva->depFlags, odepFlags;
+    rpmtransFlags transFlags = qva->transFlags, otransFlags;
     rpmVSFlags vsflags, ovsflags;
     int ec = 0;
 
@@ -557,23 +526,13 @@ int rpmcliVerify(rpmts ts, QVA_t qva, const char ** argv)
 	vsflags |= RPMVSF_NOHDRCHK;
     vsflags &= ~RPMVSF_NEEDPAYLOAD;
 
-    /* Initialize security context patterns (if not already done). */
-    if (qva->qva_flags & VERIFY_CONTEXTS) {
-	rpmsx sx = rpmtsREContext(ts);
-	if (sx == NULL) {
-	    arg = rpmGetPath("%{?_verify_file_context_path}", NULL);
-	    if (arg != NULL && *arg != '\0') {
-		sx = rpmsxNew(arg);
-		(void) rpmtsSetREContext(ts, sx);
-	    }
-	    arg = _free(arg);
-	}
-	sx = rpmsxFree(sx);
-    }
-
+    odepFlags = rpmtsSetDFlags(ts, depFlags);
+    otransFlags = rpmtsSetFlags(ts, transFlags);
     ovsflags = rpmtsSetVSFlags(ts, vsflags);
     ec = rpmcliArgIter(ts, qva, argv);
     vsflags = rpmtsSetVSFlags(ts, ovsflags);
+    transFlags = rpmtsSetFlags(ts, otransFlags);
+    depFlags = rpmtsSetDFlags(ts, odepFlags);
 
     if (qva->qva_showPackage == showVerifyPackage)
         qva->qva_showPackage = NULL;
