@@ -3,10 +3,12 @@
 
 #include "lposix.h"
 #include "lrexlib.h"
+#include "lsyck.h"
 
 #define LUA_EXTRALIBS \
 	{"posix", luaopen_posix}, \
 	{"rex", luaopen_rex}, \
+	{"syck", luaopen_syck}, \
 	{"luapath", luapath},
 
 #if 0	/* XXX don't depend on readline and history (#147013) */
@@ -50,8 +52,9 @@ static void mysaveline (lua_State *L, const char *s) {
 
 static int luapath(lua_State *L)
 {
+	const char * _lua_path = USRLIBRPM "/lua/?.lua;?.lua";
 	lua_pushstring(L, "LUA_PATH");
-	lua_pushstring(L, RPMCONFIGDIR "/lua/?.lua;?.lua");
+	lua_pushstring(L, _lua_path);
 	lua_rawset(L, LUA_GLOBALSINDEX);
 	return 0;
 }
