@@ -2,14 +2,15 @@
  * \file popt/findme.c
  */
 
-/* (C) 1998-2000 Red Hat, Inc. -- Licensing details are in the COPYING
+/* (C) 1998-2002 Red Hat, Inc. -- Licensing details are in the COPYING
    file accompanying popt source distributions, available from 
    ftp://ftp.rpm.org/pub/rpm/dist. */
 
 #include "system.h"
 #include "findme.h"
 
-const char * findProgramPath(const char * argv0) {
+const char * findProgramPath(const char * argv0)
+{
     char * path = getenv("PATH");
     char * pathbuf;
     char * start, * chptr;
@@ -28,6 +29,7 @@ const char * findProgramPath(const char * argv0) {
     strcpy(pathbuf, path);
 
     chptr = NULL;
+/*@-branchstate@*/
     do {
 	if ((chptr = strchr(start, ':')))
 	    *chptr = '\0';
@@ -41,6 +43,7 @@ const char * findProgramPath(const char * argv0) {
 	else
 	    start = NULL;
     } while (start && *start);
+/*@=branchstate@*/
 
     free(buf);
 
