@@ -12,6 +12,8 @@
 #include "header_internal.h"
 
 #include "rpmts.h"	/* XXX rpmtsCreate/rpmtsFree */
+#define	_RPMEVR_INTERNAL
+#include "rpmevr.h"
 
 #include "header-py.h"
 #include "rpmds-py.h"
@@ -93,10 +95,10 @@
  * 	fdno = os.open("/tmp/foo-1.0-1.i386.rpm", os.O_RDONLY)
  * 	hdr = ts.hdrFromFdno(fdno)
  *	os.close(fdno)
- *	if hdr[rpm.RPMTAG_SOURCEPACKAGE]:
- *	   print "header is from a source package"
- *	else:
+ *	if hdr[rpm.RPMTAG_SOURCERPM]:
  *	   print "header is from a binary package"
+ *	else:
+ *	   print "header is from a source package"
  * \endcode
  *
  * The Python interface to the header data is quite elegant.  It
@@ -497,65 +499,6 @@ static PyObject * hdrCompressFilelist(hdrObject * s)
     return Py_None;
 }
 
-/**
- */
-static PyObject * rhnUnload(hdrObject * s)
-	/*@*/
-{
-    int len;
-    char * uh;
-    PyObject * rc;
-    Header h;
-
-    h = headerLink(s->h);
-
-    /* Retrofit a RHNPlatform: tag. */
-    if (!headerIsEntry(h, RPMTAG_RHNPLATFORM)) {
-	const char * arch;
-	int_32 at;
-	if (headerGetEntry(h, RPMTAG_ARCH, &at, (void **)&arch, NULL))
-	    headerAddEntry(h, RPMTAG_RHNPLATFORM, at, arch, 1);
-    }
-
-    /* Legacy headers are forced into immutable region. */
-    if (!headerIsEntry(h, RPMTAG_HEADERIMMUTABLE)) {
-	Header nh = headerReload(h, RPMTAG_HEADERIMMUTABLE);
-	/* XXX Another unload/load cycle to "seal" the immutable region. */
-	uh = headerUnload(nh);
-	headerFree(nh);
-	h = headerLoad(uh);
-	headerAllocated(h);
-    }
-
-    /* All headers have SHA1 digest, compute and add if necessary. */
-    if (!headerIsEntry(h, RPMTAG_SHA1HEADER)) {
-	int_32 uht, uhc;
-	const char * digest;
-        size_t digestlen;
-        DIGEST_CTX ctx;
-
-	headerGetEntry(h, RPMTAG_HEADERIMMUTABLE, &uht, (void **)&uh, &uhc);
-
-	ctx = rpmDigestInit(PGPHASHALGO_SHA1, RPMDIGEST_NONE);
-        rpmDigestUpdate(ctx, uh, uhc);
-        rpmDigestFinal(ctx, (void **)&digest, &digestlen, 1);
-
-	headerAddEntry(h, RPMTAG_SHA1RHN, RPM_STRING_TYPE, digest, 1);
-
-	uh = headerFreeData(uh, uht);
-	digest = _free(digest);
-    }
-
-    len = headerSizeof(h, 0);
-    uh = headerUnload(h);
-    headerFree(h);
-
-    rc = PyString_FromStringAndSize(uh, len);
-    uh = _free(uh);
-
-    return rc;
-}
-
 /**
  */
 static PyObject * hdrGetOrigin(hdrObject * s)
@@ -642,8 +585,6 @@ static struct PyMethodDef hdr_methods[] = {
 	NULL },
     {"fullFilelist",	(PyCFunction) hdrFullFilelist,	METH_NOARGS,
 	NULL },
-    {"rhnUnload",	(PyCFunction) rhnUnload,	METH_NOARGS,
-	NULL },
     {"getorigin",	(PyCFunction) hdrGetOrigin,	METH_NOARGS,
 	NULL },
     {"setorigin",	(PyCFunction) hdrSetOrigin,	METH_NOARGS,
@@ -708,7 +649,7 @@ long tagNumFromPyObject (PyObject *item)
 
     if (PyInt_Check(item)) {
 	return PyInt_AsLong(item);
-    } else if (PyString_Check(item)) {
+    } else if (PyString_Check(item) || PyUnicode_Check(item)) {
 	str = PyString_AsString(item);
 	for (i = 0; i < rpmTagTableSize; i++)
 	    if (!xstrcasecmp(rpmTagTable[i].name + 7, str)) break;
@@ -791,14 +732,14 @@ static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
     int forceArray = 0;
     int freeData = 0;
     char * str;
-    struct headerSprintfExtension_s * ext = NULL;
+    const struct headerSprintfExtension_s * ext = NULL;
     const struct headerSprintfExtension_s * extensions = rpmHeaderFormats;
 
     if (PyCObject_Check (item))
         ext = PyCObject_AsVoidPtr(item);
     else
 	tag = tagNumFromPyObject (item);
-    if (tag == -1 && PyString_Check(item)) {
+    if (tag == -1 && (PyString_Check(item) || PyUnicode_Check(item))) {
 	/* if we still don't have the tag, go looking for the header
 	   extensions */
 	str = PyString_AsString(item);
@@ -865,6 +806,7 @@ static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
     case RPMTAG_CONFLICTNAME:
     case RPMTAG_CONFLICTFLAGS:
     case RPMTAG_CONFLICTVERSION:
+    case RPMTAG_CHANGELOGTIME:
 	forceArray = 1;
 	break;
     case RPMTAG_SUMMARY:
@@ -1093,59 +1035,6 @@ PyObject * hdrLoad(PyObject * self, PyObject * args, PyObject * kwds)
     return (PyObject *) hdr;
 }
 
-/**
- */
-PyObject * rhnLoad(PyObject * self, PyObject * args, PyObject * kwds)
-{
-    char * obj, * copy=NULL;
-    Header h;
-    int len;
-    char * kwlist[] = {"headers", NULL};
-
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "s#", kwlist, &obj, &len))
-	return NULL;
-
-    /* malloc is needed to avoid surprises from data swab in headerLoad(). */
-    copy = malloc(len);
-    if (copy == NULL) {
-	PyErr_SetString(pyrpmError, "out of memory");
-	return NULL;
-    }
-    memcpy (copy, obj, len);
-
-    h = headerLoad(copy);
-    if (!h) {
-	PyErr_SetString(pyrpmError, "bad header");
-	return NULL;
-    }
-    headerAllocated(h);
-
-    /* XXX avoid the false OK's from rpmverifyDigest() with missing tags. */
-    if (!headerIsEntry(h, RPMTAG_HEADERIMMUTABLE)) {
-	PyErr_SetString(pyrpmError, "bad header, not immutable");
-	headerFree(h);
-	return NULL;
-    }
-
-    /* XXX avoid the false OK's from rpmverifyDigest() with missing tags. */
-    if (!headerIsEntry(h, RPMTAG_SHA1HEADER)
-    &&  !headerIsEntry(h, RPMTAG_SHA1RHN)) {
-	PyErr_SetString(pyrpmError, "bad header, no digest");
-	headerFree(h);
-	return NULL;
-    }
-
-    /* Retrofit a RHNPlatform: tag. */
-    if (!headerIsEntry(h, RPMTAG_RHNPLATFORM)) {
-	const char * arch;
-	int_32 at;
-	if (headerGetEntry(h, RPMTAG_ARCH, &at, (void **)&arch, NULL))
-	    headerAddEntry(h, RPMTAG_RHNPLATFORM, at, arch, 1);
-    }
-
-    return (PyObject *) hdr_Wrap(h);
-}
-
 /**
  */
 PyObject * rpmReadHeaders (FD_t fd)
@@ -1384,33 +1273,24 @@ PyObject * versionCompare (PyObject * self, PyObject * args, PyObject * kwds)
     return Py_BuildValue("i", hdr_compare(h1, h2));
 }
 
-/**
- */
-static int compare_values(const char *str1, const char *str2)
-{
-    if (!str1 && !str2)
-	return 0;
-    else if (str1 && !str2)
-	return 1;
-    else if (!str1 && str2)
-	return -1;
-    return rpmvercmp(str1, str2);
-}
-
 PyObject * labelCompare (PyObject * self, PyObject * args)
 {
-    char *v1, *r1, *e1, *v2, *r2, *e2;
+    EVR_t A = memset(alloca(sizeof(*A)), 0, sizeof(*A));
+    EVR_t B = memset(alloca(sizeof(*B)), 0, sizeof(*B));
     int rc;
 
     if (!PyArg_ParseTuple(args, "(zzz)(zzz)",
-			&e1, &v1, &r1, &e2, &v2, &r2))
+			&A->E, &A->V, &A->R, &B->E, &B->V, &B->R))
 	return NULL;
 
-    rc = compare_values(e1, e2);
-    if (!rc) {
-	rc = compare_values(v1, v2);
-	if (!rc)
-	    rc = compare_values(r1, r2);
-    }
+    if (A->E == NULL)	A->E = "0";
+    if (B->E == NULL)	B->E = "0";
+    if (A->V == NULL)	A->E = "";
+    if (B->V == NULL)	B->E = "";
+    if (A->R == NULL)	A->E = "";
+    if (B->R == NULL)	B->E = "";
+
+    rc = rpmEVRcompare(A, B);
+
     return Py_BuildValue("i", rc);
 }
