@@ -426,13 +426,16 @@ static PyObject *
 rpmds_Sysinfo(rpmdsObject * s)
 	/*@*/
 {
-    rpmds ds = NULL;
+    rpmPRCO PRCO = rpmdsNewPRCO(NULL);
+    rpmds P = NULL;
     int xx;
 
     /* XXX check return code, permit arg (NULL uses system default). */
-    xx = rpmdsSysinfo(&ds, NULL);
+    xx = rpmdsSysinfo(PRCO, NULL);
+    P = rpmdsLink(rpmdsFromPRCO(PRCO, RPMTAG_PROVIDENAME), __FUNCTION__);
+    PRCO = rpmdsFreePRCO(PRCO);
 
-    return (PyObject *) rpmds_Wrap( ds );
+    return (PyObject *) rpmds_Wrap( P );
 }
 
 static PyObject *
@@ -452,14 +455,15 @@ static PyObject *
 rpmds_Ldconfig(rpmdsObject * s)
 	/*@*/
 {
-    rpmPRCO PRCO = memset(alloca(sizeof(*PRCO)), 0, sizeof(*PRCO));
+    rpmPRCO PRCO = rpmdsNewPRCO(NULL);
     rpmds P = NULL;
     int xx;
 
-    PRCO->Pdsp = &P;
     /* XXX check return code, permit arg (NULL uses system default). */
     xx = rpmdsLdconfig(PRCO, NULL);
 
+    P = rpmdsLink(rpmdsFromPRCO(PRCO, RPMTAG_PROVIDENAME), __FUNCTION__);
+    PRCO = rpmdsFreePRCO(PRCO);
     return (PyObject *) rpmds_Wrap( P );
 }
 
@@ -542,15 +546,15 @@ The current index in ds is positioned at matching member upon success.\n" },
 "ds.Search(element) -> matching ds index (-1 on failure)\n\
 - Check that element dependency range overlaps some member of ds.\n\
 The current index in ds is positioned at overlapping member upon success.\n" },
- {"Cpuinfo",	(PyCFunction)rpmds_Cpuinfo,	METH_NOARGS,
+ {"Cpuinfo",	(PyCFunction)rpmds_Cpuinfo,	METH_NOARGS|METH_STATIC,
 	"ds.Cpuinfo -> nds	- Return /proc/cpuinfo dependency set.\n"},
- {"Rpmlib",	(PyCFunction)rpmds_Rpmlib,	METH_NOARGS,
+ {"Rpmlib",	(PyCFunction)rpmds_Rpmlib,	METH_NOARGS|METH_STATIC,
 	"ds.Rpmlib -> nds	- Return internal rpmlib dependency set.\n"},
- {"Sysinfo",	(PyCFunction)rpmds_Sysinfo,	METH_NOARGS,
+ {"Sysinfo",	(PyCFunction)rpmds_Sysinfo,	METH_NOARGS|METH_STATIC,
 	"ds.Sysinfo -> nds	- Return /etc/rpm/sysinfo dependency set.\n"},
- {"Getconf",	(PyCFunction)rpmds_Getconf,	METH_NOARGS,
+ {"Getconf",	(PyCFunction)rpmds_Getconf,	METH_NOARGS|METH_STATIC,
 	"ds.Getconf -> nds	- Return getconf(1) dependency set.\n"},
- {"Ldconfig",	(PyCFunction)rpmds_Ldconfig,	METH_NOARGS,
+ {"Ldconfig",	(PyCFunction)rpmds_Ldconfig,	METH_NOARGS|METH_STATIC,
 	"ds.Ldconfig -> nds	- Return /etc/ld.so.cache dependency set.\n"},
 #ifdef	NOTYET
  {"Compare",	(PyCFunction)rpmds_Compare,	METH_VARARGS|METH_KEYWORDS,
