@@ -57,6 +57,26 @@ static PyObject * archScore(PyObject * self, PyObject * args, PyObject * kwds)
     return Py_BuildValue("i", score);
 }
 
+extern sigset_t rpmsqCaught;
+/**
+ */
+static PyObject * sqCaught(PyObject * self, PyObject * args)
+{
+    if (!PyArg_ParseTuple(args, ":sqCaught")) return NULL;
+
+    return Py_BuildValue("i", rpmsqCaught);
+}
+
+/**
+ */
+static PyObject * checkSignals(PyObject * self, PyObject * args)
+{
+    if (!PyArg_ParseTuple(args, ":checkSignals")) return NULL;
+    rpmdbCheckSignals();
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 /**
  */
 static PyObject * setLogFile (PyObject * self, PyObject * args, PyObject *kwds)
@@ -147,9 +167,12 @@ static PyMethodDef rpmModuleMethods[] = {
     { "archscore", (PyCFunction) archScore, METH_VARARGS|METH_KEYWORDS,
 	NULL },
 
-    { "headerLoad", (PyCFunction) hdrLoad, METH_VARARGS|METH_KEYWORDS,
+    { "sqCaught", (PyCFunction) sqCaught, METH_VARARGS,
 	NULL },
-    { "rhnLoad", (PyCFunction) rhnLoad, METH_VARARGS|METH_KEYWORDS,
+    { "checkSignals", (PyCFunction) checkSignals, METH_VARARGS,
+	NULL },
+
+    { "headerLoad", (PyCFunction) hdrLoad, METH_VARARGS|METH_KEYWORDS,
 	NULL },
     { "mergeHeaderListFromFD", (PyCFunction) rpmMergeHeadersFromFD, METH_VARARGS|METH_KEYWORDS,
 	NULL },
@@ -218,7 +241,7 @@ void init_rpm(void)
     if (m == NULL)
 	return;
 
-    rpmReadConfigFiles(NULL, NULL);
+    rpmReadConfigFiles(rpmRcfiles, NULL);
 
     d = PyModule_GetDict(m);
 
@@ -332,31 +355,21 @@ void init_rpm(void)
     REGISTER_ENUM(RPMDEP_SENSE_REQUIRES);
     REGISTER_ENUM(RPMDEP_SENSE_CONFLICTS);
 
-    REGISTER_ENUM(RPMSENSE_SERIAL);
     REGISTER_ENUM(RPMSENSE_LESS);
     REGISTER_ENUM(RPMSENSE_GREATER);
     REGISTER_ENUM(RPMSENSE_EQUAL);
-    REGISTER_ENUM(RPMSENSE_PREREQ);
-    REGISTER_ENUM(RPMSENSE_INTERP);
-    REGISTER_ENUM(RPMSENSE_SCRIPT_PRE);
-    REGISTER_ENUM(RPMSENSE_SCRIPT_POST);
-    REGISTER_ENUM(RPMSENSE_SCRIPT_PREUN);
-    REGISTER_ENUM(RPMSENSE_SCRIPT_POSTUN);
-    REGISTER_ENUM(RPMSENSE_SCRIPT_VERIFY);
-    REGISTER_ENUM(RPMSENSE_FIND_REQUIRES);
-    REGISTER_ENUM(RPMSENSE_FIND_PROVIDES);
-    REGISTER_ENUM(RPMSENSE_TRIGGERIN);
-    REGISTER_ENUM(RPMSENSE_TRIGGERUN);
-    REGISTER_ENUM(RPMSENSE_TRIGGERPOSTUN);
-    REGISTER_ENUM(RPMSENSE_SCRIPT_PREP);
-    REGISTER_ENUM(RPMSENSE_SCRIPT_BUILD);
-    REGISTER_ENUM(RPMSENSE_SCRIPT_INSTALL);
-    REGISTER_ENUM(RPMSENSE_SCRIPT_CLEAN);
-    REGISTER_ENUM(RPMSENSE_RPMLIB);
-    REGISTER_ENUM(RPMSENSE_TRIGGERPREIN);
-    REGISTER_ENUM(RPMSENSE_KEYRING);
-    REGISTER_ENUM(RPMSENSE_PATCHES);
-    REGISTER_ENUM(RPMSENSE_CONFIG);
+    REGISTER_ENUM(RPMSENSE_NOTEQUAL);
+
+    REGISTER_ENUM(RPMDEPS_FLAG_NOUPGRADE);
+    REGISTER_ENUM(RPMDEPS_FLAG_NOREQUIRES);
+    REGISTER_ENUM(RPMDEPS_FLAG_NOCONFLICTS);
+    REGISTER_ENUM(RPMDEPS_FLAG_NOOBSOLETES);
+    REGISTER_ENUM(RPMDEPS_FLAG_NOPARENTDIRS);
+    REGISTER_ENUM(RPMDEPS_FLAG_NOLINKTOS);
+    REGISTER_ENUM(RPMDEPS_FLAG_ANACONDA);
+    REGISTER_ENUM(RPMDEPS_FLAG_NOSUGGEST);
+    REGISTER_ENUM(RPMDEPS_FLAG_ADDINDEPS);
+    REGISTER_ENUM(RPMDEPS_FLAG_DEPLOOPS);
 
     REGISTER_ENUM(RPMTRANS_FLAG_TEST);
     REGISTER_ENUM(RPMTRANS_FLAG_BUILD_PROBS);
@@ -367,7 +380,6 @@ void init_rpm(void)
     REGISTER_ENUM(RPMTRANS_FLAG_ALLFILES);
     REGISTER_ENUM(RPMTRANS_FLAG_KEEPOBSOLETE);
     REGISTER_ENUM(RPMTRANS_FLAG_REPACKAGE);
-    REGISTER_ENUM(RPMTRANS_FLAG_REVERSE);
     REGISTER_ENUM(RPMTRANS_FLAG_NOPRE);
     REGISTER_ENUM(RPMTRANS_FLAG_NOPOST);
     REGISTER_ENUM(RPMTRANS_FLAG_NOTRIGGERPREIN);
@@ -376,10 +388,7 @@ void init_rpm(void)
     REGISTER_ENUM(RPMTRANS_FLAG_NOPREUN);
     REGISTER_ENUM(RPMTRANS_FLAG_NOPOSTUN);
     REGISTER_ENUM(RPMTRANS_FLAG_NOTRIGGERPOSTUN);
-    REGISTER_ENUM(RPMTRANS_FLAG_ANACONDA);
     REGISTER_ENUM(RPMTRANS_FLAG_NOFDIGESTS);
-    REGISTER_ENUM(RPMTRANS_FLAG_NOSUGGEST);
-    REGISTER_ENUM(RPMTRANS_FLAG_ADDINDEPS);
     REGISTER_ENUM(RPMTRANS_FLAG_NOCONFIGS);
 
     REGISTER_ENUM(RPMPROB_FILTER_IGNOREOS);
@@ -460,5 +469,5 @@ void init_rpm(void)
 
     REGISTER_ENUM(RPMDBI_PACKAGES);
 
-    REGISTER_ENUM(RPMAL_NOMATCH);
+    REGISTER_ENUM((int)RPMAL_NOMATCH);
 }
