@@ -471,7 +471,7 @@ fprintf(stderr, "*** rpmts_AddErase(%p) ts %p\n", s, s->ts);
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "O:AddErase", kwlist, &o))
         return NULL;
 
-    if (PyString_Check(o)) {
+    if (PyString_Check(o) || PyUnicode_Check(o)) {
 	char * name = PyString_AsString(o);
 
 	mi = rpmtsInitIterator(s->ts, RPMDBI_LABEL, name, 0);
@@ -785,7 +785,7 @@ rpmts_Rollback(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	/*@globals rpmGlobalMacroContext @*/
 	/*@modifies s, rpmGlobalMacroContext @*/
 {
-    struct rpmInstallArguments_s * ia = alloca(sizeof(*ia));
+    QVA_t ia = memset(alloca(sizeof(*ia)), 0, sizeof(*ia));
     rpmtransFlags transFlags;
     const char ** av = NULL;
     uint_32 rbtid;
@@ -799,7 +799,6 @@ fprintf(stderr, "*** rpmts_Rollback(%p) ts %p\n", s, s->ts);
     	return NULL;
 
     Py_BEGIN_ALLOW_THREADS
-    memset(ia, 0, sizeof(*ia));
     ia->qva_flags = (VERIFY_DIGEST|VERIFY_SIGNATURE|VERIFY_HDRCHK);
     ia->transFlags |= (INSTALL_UPGRADE|INSTALL_FRESHEN|INSTALL_INSTALL);
     ia->transFlags |= RPMTRANS_FLAG_NOFDIGESTS;
@@ -991,7 +990,7 @@ fprintf(stderr, "*** rpmts_HdrCheck(%p) ts %p\n", s, s->ts);
 	Py_INCREF(Py_None);
 	return Py_None;
     }
-    if (!PyString_Check(blob)) {
+    if (!(PyString_Check(blob) || PyUnicode_Check(blob))) {
 	PyErr_SetString(pyrpmError, "hdrCheck takes a string of octets");
 	return result;
     }
@@ -1093,11 +1092,11 @@ fprintf(stderr, "*** rpmts_PgpPrtPkts(%p) ts %p\n", s, s->ts);
 	Py_INCREF(Py_None);
 	return Py_None;
     }
-    if (!PyString_Check(blob)) {
+    if (!(PyString_Check(blob) || PyUnicode_Check(blob))) {
 	PyErr_SetString(pyrpmError, "pgpPrtPkts takes a string of octets");
 	return NULL;
     }
-    pkt = PyString_AsString(blob);
+    pkt = (unsigned char *) PyString_AsString(blob);
     pktlen = PyString_Size(blob);
 
     rc = pgpPrtPkts(pkt, pktlen, NULL, 1);
@@ -1130,11 +1129,11 @@ fprintf(stderr, "*** rpmts_PgpImportPubkey(%p) ts %p\n", s, s->ts);
 	Py_INCREF(Py_None);
 	return Py_None;
     }
-    if (!PyString_Check(blob)) {
+    if (!(PyString_Check(blob) || PyUnicode_Check(blob))) {
 	PyErr_SetString(pyrpmError, "PgpImportPubkey takes a string of octets");
 	return NULL;
     }
-    pkt = PyString_AsString(blob);
+    pkt = (unsigned char *) PyString_AsString(blob);
     pktlen = PyString_Size(blob);
 
     rc = rpmcliImportPubkey(s->ts, pkt, pktlen);
@@ -1156,7 +1155,7 @@ rpmts_SetFlags(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	return NULL;
 
 if (_rpmts_debug)
-fprintf(stderr, "*** rpmts_SetFlags(%p) ts %p transFlags %x\n", s, s->ts, transFlags);
+fprintf(stderr, "*** rpmts_SetFlags(%p) ts %p transFlags 0x%x\n", s, s->ts, transFlags);
 
     /* XXX FIXME: value check on flags, or build pure python object 
      * for it, and require an object of that type */
@@ -1164,6 +1163,28 @@ fprintf(stderr, "*** rpmts_SetFlags(%p) ts %p transFlags %x\n", s, s->ts, transF
     return Py_BuildValue("i", rpmtsSetFlags(s->ts, transFlags));
 }
 
+/**
+ */
+static PyObject *
+rpmts_SetDFlags(rpmtsObject * s, PyObject * args, PyObject * kwds)
+	/*@modifies s @*/
+{
+    rpmdepFlags depFlags = 0;
+    char * kwlist[] = {"flags", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "i:SetDFlags", kwlist,
+	    &depFlags))
+	return NULL;
+
+if (_rpmts_debug)
+fprintf(stderr, "*** rpmts_SetDFlags(%p) ts %p depFlags 0x%x\n", s, s->ts, depFlags);
+
+    /* XXX FIXME: value check on flags, or build pure python object 
+     * for it, and require an object of that type */
+
+    return Py_BuildValue("i", rpmtsSetDFlags(s->ts, depFlags));
+}
+
 /** \ingroup py_c
  */
 static PyObject *
@@ -1319,19 +1340,19 @@ spec_Parse(rpmtsObject * s, PyObject * args, PyObject * kwds)
 {
     const char * specfile;
     Spec spec;
-    char * buildRoot = NULL;
     int recursing = 0;
     char * passPhrase = "";
     char *cookie = NULL;
     int anyarch = 1;
+    int verify = 1;
     int force = 1;
     char * kwlist[] = {"specfile", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "s:Parse", kwlist, &specfile))
 	return NULL;
 
-    if (parseSpec(s->ts, specfile,"/", buildRoot,recursing, passPhrase,
-             cookie, anyarch, force)!=0) {
+    if (parseSpec(s->ts, specfile,"/", recursing, passPhrase,
+             cookie, anyarch, force, verify)!=0) {
              PyErr_SetString(pyrpmError, "can't parse specfile\n");
                      return NULL;
    }
@@ -1371,7 +1392,7 @@ fprintf(stderr, "*** rpmts_Match(%p) ts %p\n", s, s->ts);
 
     if (Key) {
 /*@-branchstate@*/
-	if (PyString_Check(Key)) {
+	if (PyString_Check(Key) || PyUnicode_Check(Key)) {
 	    key = PyString_AsString(Key);
 	    len = PyString_Size(Key);
 	} else if (PyInt_Check(Key)) {
@@ -1412,6 +1433,9 @@ static struct PyMethodDef rpmts_methods[] = {
 	NULL },
  {"addErase",	(PyCFunction) rpmts_AddErase,	METH_VARARGS|METH_KEYWORDS,
 	NULL },
+ {"setDFlags",	(PyCFunction) rpmts_SetDFlags,	METH_VARARGS|METH_KEYWORDS,
+"ts.setDFlags(depFlags) -> previous depFlags\n\
+- Set control bit(s) for executing ts.check() and ts.order().\n" },
  {"check",	(PyCFunction) rpmts_Check,	METH_VARARGS|METH_KEYWORDS,
 	NULL },
  {"order",	(PyCFunction) rpmts_Order,	METH_NOARGS,
