@@ -597,6 +597,7 @@ static int regionSwab(/*@null@*/ indexEntry entry, int il, int dl,
 	}
 
 	dl += ie.length;
+	if (dataEnd && dataStart + dl > dataEnd) return -1;
 	tl += tdel;
 	ieprev = ie;	/* structure assignment */
 
@@ -2610,10 +2611,12 @@ static char * hsaReserve(headerSprintfArgs hsa, size_t need)
  * @todo bsearch on sorted value table.
  * @param tbl		tag table
  * @param val		tag value to find
+ * @retval *typep	tag type (or NULL)
  * @return		tag name, NULL on not found
  */
 /*@observer@*/ /*@null@*/
-static const char * myTagName(headerTagTableEntry tbl, int val)
+static const char * myTagName(headerTagTableEntry tbl, int val,
+		/*@null@*/ int *typep)
 	/*@*/
 {
     static char name[128];
@@ -2632,6 +2635,8 @@ static const char * myTagName(headerTagTableEntry tbl, int val)
     while (*s != '\0')
 	*t++ = xtolower(*s++);
     *t = '\0';
+    if (typep)
+	*typep = tbl->type;
     return name;
 }
 
@@ -2653,6 +2658,7 @@ static int myTagValue(headerTagTableEntry tbl, const char * name)
 }
 
 /**
+ * Search extensions and tags for a name.
  * @param hsa		headerSprintf args
  * @param token		parsed fields
  * @param name		name to find
@@ -2723,6 +2729,7 @@ bingo:
 
 /* forward ref */
 /**
+ * Parse a headerSprintf expression.
  * @param hsa		headerSprintf args
  * @param token
  * @param str
@@ -2735,6 +2742,7 @@ static int parseExpression(headerSprintfArgs hsa, sprintfToken token,
 	/*@requires maxSet(endPtr) >= 0 @*/;
 
 /**
+ * Parse a headerSprintf term.
  * @param hsa		headerSprintf args
  * @param str
  * @retval *formatPtr
@@ -3097,10 +3105,10 @@ static int parseExpression(headerSprintfArgs hsa, sprintfToken token,
 /**
  * Call a header extension only once, saving results.
  * @param hsa		headerSprintf args
- * @param fn
- * @retval *typeptr
- * @retval *data
- * @retval *countptr
+ * @param fn		function
+ * @retval *typeptr	extension type
+ * @retval *data	extension data
+ * @retval *countptr	extension size
  * @retval ec		extension cache
  * @return		0 on success, 1 on failure
  */
@@ -3127,9 +3135,10 @@ static int getExtension(headerSprintfArgs hsa, headerTagTagFunction fn,
 }
 
 /**
+ * Format a single item value.
  * @param hsa		headerSprintf args
- * @param tag
- * @param element
+ * @param tag		tag
+ * @param element	element index
  * @return		end of formatted string (NULL on error)
  */
 /*@observer@*/ /*@null@*/
@@ -3350,9 +3359,10 @@ static char * formatValue(headerSprintfArgs hsa, sprintfTag tag, int element)
 }
 
 /**
+ * Format a single headerSprintf item.
  * @param hsa		headerSprintf args
- * @param token
- * @param element
+ * @param token		item to format
+ * @param element	element index
  * @return		end of formatted string (NULL on error)
  */
 /*@observer@*/
@@ -3360,6 +3370,7 @@ static char * singleSprintf(headerSprintfArgs hsa, sprintfToken token,
 		int element)
 	/*@modifies hsa @*/
 {
+    char numbuf[64];	/* XXX big enuf for "Tag_0x01234567" */
     char * t, * te;
     int i, j;
     int numElements;
@@ -3478,38 +3489,50 @@ static char * singleSprintf(headerSprintfArgs hsa, sprintfToken token,
 		!strcmp(spft->u.tag.type, "yaml"));
 
 	    if (isxml) {
-		const char * tagN = myTagName(hsa->tags, spft->u.tag.tag);
-
-assert(tagN != NULL);	/* XXX splats pick up old or unknown tags. */
-		need = sizeof("  <rpmTag name=\"\">\n") - 1;
-		if (tagN != NULL)
-		    need += strlen(tagN);
+		const char * tagN = myTagName(hsa->tags, spft->u.tag.tag, NULL);
+		/* XXX display "Tag_0x01234567" for unknown tags. */
+		if (tagN == NULL) {
+		    (void) snprintf(numbuf, sizeof(numbuf), "Tag_0x%08x",
+				spft->u.tag.tag);
+		    numbuf[sizeof(numbuf)-1] = '\0';
+		    tagN = numbuf;
+		}
+		need = sizeof("  <rpmTag name=\"\">\n") + strlen(tagN);
 		te = t = hsaReserve(hsa, need);
 /*@-boundswrite@*/
-		te = stpcpy(te, "  <rpmTag name=\"");
-		if (tagN != NULL)
-		    te = stpcpy(te, tagN);
-		te = stpcpy(te, "\">\n");
+		te = stpcpy( stpcpy( stpcpy(te, "  <rpmTag name=\""), tagN), "\">\n");
 /*@=boundswrite@*/
 		hsa->vallen += (te - t);
 	    }
 	    if (isyaml) {
-		const char * tagN = myTagName(hsa->tags, spft->u.tag.tag);
-
-assert(tagN != NULL);	/* XXX splats pick up old or unknown tags. */
-		need = sizeof("  : ") - 1;
-		if (tagN != NULL)
-		    need += strlen(tagN);
-		if (numElements > 1)
-		    need += sizeof("\n") - 1;
+		int tagT = -1;
+		const char * tagN = myTagName(hsa->tags, spft->u.tag.tag, &tagT);
+		/* XXX display "Tag_0x01234567" for unknown tags. */
+		if (tagN == NULL) {
+		    (void) snprintf(numbuf, sizeof(numbuf), "Tag_0x%08x",
+				spft->u.tag.tag);
+		    numbuf[sizeof(numbuf)-1] = '\0';
+		    tagN = numbuf;
+		    tagT = numElements > 1
+			?  RPM_ARRAY_RETURN_TYPE : RPM_SCALAR_RETURN_TYPE;
+		}
+		need = sizeof("  :     - ") + strlen(tagN);
 		te = t = hsaReserve(hsa, need);
 /*@-boundswrite@*/
-		te = stpcpy(te, "  ");
-		if (tagN != NULL)
-		    te = stpcpy(te, tagN);
-		te = stpcpy(te, ": ");
-		if (numElements > 1)
-		    te = stpcpy(te, "\n");
+		*te++ = ' ';
+		*te++ = ' ';
+		te = stpcpy(te, tagN);
+		*te++ = ':';
+		*te++ = (((tagT & RPM_MASK_RETURN_TYPE) == RPM_ARRAY_RETURN_TYPE)
+			? '\n' : ' ');
+		/* XXX Dirnames: in srpms need "    " indent */
+		if (((tagT & RPM_MASK_RETURN_TYPE) == RPM_ARRAY_RETURN_TYPE)
+		 && numElements == 1) {
+		    te = stpcpy(te, "    ");
+		    if (spft->u.tag.tag != 1118)
+			te = stpcpy(te, "- ");
+		}
+		*te = '\0';
 /*@=boundswrite@*/
 		hsa->vallen += (te - t);
 	    }
@@ -3713,6 +3736,7 @@ exit:
 }
 
 /**
+ * Return octal formatted data.
  * @param type		tag type
  * @param data		tag value
  * @param formatPrefix	sprintf format string
@@ -3749,6 +3773,7 @@ static char * octalFormat(int_32 type, hPTR_t data,
 }
 
 /**
+ * Return hex formatted data.
  * @param type		tag type
  * @param data		tag value
  * @param formatPrefix	sprintf format string
@@ -3785,6 +3810,14 @@ static char * hexFormat(int_32 type, hPTR_t data,
 }
 
 /**
+ * Return strftime formatted data.
+ * @param type		tag type
+ * @param data		tag value
+ * @param formatPrefix	sprintf format string
+ * @param padding	no. additional bytes needed by format string
+ * @param element	(unused)
+ * @param strftimeFormat strftime(3) format
+ * @return		formatted string
  */
 static char * realDateFormat(int_32 type, hPTR_t data, 
 		char * formatPrefix, int padding, /*@unused@*/int element,
@@ -3820,6 +3853,7 @@ static char * realDateFormat(int_32 type, hPTR_t data,
 }
 
 /**
+ * Return date formatted data.
  * @param type		tag type
  * @param data		tag value
  * @param formatPrefix	sprintf format string
@@ -3836,6 +3870,7 @@ static char * dateFormat(int_32 type, hPTR_t data,
 }
 
 /**
+ * Return day formatted data.
  * @param type		tag type
  * @param data		tag value
  * @param formatPrefix	sprintf format string
@@ -3852,6 +3887,7 @@ static char * dayFormat(int_32 type, hPTR_t data,
 }
 
 /**
+ * Return shell escape formatted data.
  * @param type		tag type
  * @param data		tag value
  * @param formatPrefix	sprintf format string
