@@ -5,7 +5,15 @@
 #include "system.h"
 
 #if HAVE_GELF_H
-
+#if LIBELF_H_LFS_CONFLICT
+/* some gelf.h/libelf.h implementations (Solaris) are
+ * incompatible with the Large File API
+ */
+# undef _LARGEFILE64_SOURCE
+# undef _LARGEFILE_SOURCE
+# undef _FILE_OFFSET_BITS
+# define _FILE_OFFSET_BITS 32
+#endif
 #include <gelf.h>
 
 #if !defined(DT_GNU_PRELINKED)
@@ -209,7 +217,7 @@ int dodigest(int digestalgo, const char * fn, unsigned char * digest, int asAsci
     case URL_IS_DASH:
     default:
 	/* Either use the pipe to prelink -y or open the URL. */
-	fd = (pid != 0) ? fdDup(fdno) : Fopen(fn, "r.ufdio");
+	fd = (pid != 0) ? fdDup(fdno) : Fopen(fn, "r");
 	(void) close(fdno);
 	if (fd == NULL || Ferror(fd)) {
 	    rc = 1;
@@ -290,15 +298,20 @@ void rpmfiBuildFNames(Header h, rpmTag tagN,
     xx = hge(h, dirIndexesTag, NULL, (void **) &dirIndexes, &count);
 
     size = sizeof(*fileNames) * count;
-    for (i = 0; i < count; i++)
-	size += strlen(baseNames[i]) + strlen(dirNames[dirIndexes[i]]) + 1;
+    for (i = 0; i < count; i++) {
+	const char * dn = NULL;
+	(void) urlPath(dirNames[dirIndexes[i]], &dn);
+	size += strlen(baseNames[i]) + strlen(dn) + 1;
+    }
 
     fileNames = xmalloc(size);
     t = ((char *) fileNames) + (sizeof(*fileNames) * count);
     /*@-branchstate@*/
     for (i = 0; i < count; i++) {
+	const char * dn = NULL;
 	fileNames[i] = t;
-	t = stpcpy( stpcpy(t, dirNames[dirIndexes[i]]), baseNames[i]);
+	(void) urlPath(dirNames[dirIndexes[i]], &dn);
+	t = stpcpy( stpcpy(t, dn), baseNames[i]);
 	*t++ = '\0';
     }
     /*@=branchstate@*/
