@@ -141,10 +141,10 @@ fprintf(stderr, "sql:chroot(%s)\n", dbi->dbi_root);
     }
 
     sqlCwd = currDir;
-    xx = chdir("/");
-/*@-superuser@*/
-    xx = chroot(dbi->dbi_root);
-/*@=superuser@*/
+/*@-globs@*/
+    xx = Chdir("/");
+/*@=globs@*/
+    xx = Chroot(dbi->dbi_root);
 assert(xx == 0);
     sqlInRoot=1;
 }
@@ -163,11 +163,13 @@ if (_debug)
 fprintf(stderr, "sql:chroot(.)\n");
 
 /*@-superuser@*/
-    xx = chroot(".");
+    xx = Chroot(".");
 /*@=superuser@*/
 assert(xx == 0);
     if (sqlCwd != NULL) {
-	xx = chdir(sqlCwd);
+/*@-globs@*/
+	xx = Chdir(sqlCwd);
+/*@=globs@*/
 	sqlCwd = _free(sqlCwd);
     }
 
