@@ -11,9 +11,10 @@ static const struct headerTagTableEntry_s rpmTagTbl[] = {
 	{ "RPMTAG_BLINKHDRID",	1165,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_BLINKNEVRA",	1166,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_BLINKPKGID",	1164,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
-	{ "RPMTAG_BUILDARCHS",	1089,	RPM_NULL_TYPE + RPM_ANY_RETURN_TYPE },
+	{ "RPMTAG_BUILDARCHS",	1089,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_BUILDCPUCLOCK",	1176,	RPM_INT32_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_BUILDHOST",	1007,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
+	{ "RPMTAG_BUILDPLATFORMS",	1183,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_BUILDTIME",	1006,	RPM_INT32_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_CACHECTIME",	1136,	RPM_INT32_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_CACHEPKGMTIME",	1139,	RPM_INT32_TYPE + RPM_SCALAR_RETURN_TYPE },
@@ -24,6 +25,7 @@ static const struct headerTagTableEntry_s rpmTagTbl[] = {
 	{ "RPMTAG_CHANGELOGTEXT",	1082,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_CHANGELOGTIME",	1080,	RPM_INT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_CLASSDICT",	1142,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_CONFLICTATTRSX",	1189,	RPM_INT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_CONFLICTFLAGS",	1053,	RPM_INT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_CONFLICTNAME",	1054,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_CONFLICTS",	RPMTAG_CONFLICTNAME,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE  },
@@ -31,6 +33,7 @@ static const struct headerTagTableEntry_s rpmTagTbl[] = {
 	{ "RPMTAG_COOKIE",	1094,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_C",	RPMTAG_CONFLICTNAME,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE  },
 	{ "RPMTAG_CVSID",	1163,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
+	{ "RPMTAG_DEPATTRSDICT",	1188,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_DEPENDSDICT",	1145,	RPM_INT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_DESCRIPTION",	1005,	RPM_I18NSTRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_DIRINDEXES",	1116,	RPM_INT32_TYPE + RPM_ARRAY_RETURN_TYPE },
@@ -45,10 +48,10 @@ static const struct headerTagTableEntry_s rpmTagTbl[] = {
 	{ "RPMTAG_ENHANCESVERSION",	1160,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_EPOCH",	1003,	RPM_INT32_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_E",	RPMTAG_EPOCH,	RPM_INT32_TYPE + RPM_SCALAR_RETURN_TYPE  },
-	{ "RPMTAG_EXCLUDEARCH",	1059,	RPM_NULL_TYPE + RPM_ANY_RETURN_TYPE },
-	{ "RPMTAG_EXCLUDEOS",	1060,	RPM_NULL_TYPE + RPM_ANY_RETURN_TYPE },
-	{ "RPMTAG_EXCLUSIVEARCH",	1061,	RPM_NULL_TYPE + RPM_ANY_RETURN_TYPE },
-	{ "RPMTAG_EXCLUSIVEOS",	1062,	RPM_NULL_TYPE + RPM_ANY_RETURN_TYPE },
+	{ "RPMTAG_EXCLUDEARCH",	1059,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_EXCLUDEOS",	1060,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_EXCLUSIVEARCH",	1061,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_EXCLUSIVEOS",	1062,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_FILECLASS",	1141,	RPM_INT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_FILECOLORS",	1140,	RPM_INT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_FILECONTEXTS",	1147,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
@@ -70,6 +73,7 @@ static const struct headerTagTableEntry_s rpmTagTbl[] = {
 	{ "RPMTAG_FILESTATES",	1029,	RPM_CHAR_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_FILEUSERNAME",	1039,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_FILEVERIFYFLAGS",	1045,	RPM_INT32_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_FILEXATTRSX",	1187,	RPM_INT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_FLINKHDRID",	1168,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_FLINKNEVRA",	1169,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_FLINKPKGID",	1167,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
@@ -82,13 +86,16 @@ static const struct headerTagTableEntry_s rpmTagTbl[] = {
 	{ "RPMTAG_HEADERIMMUTABLE",	HEADER_IMMUTABLE,	RPM_NULL_TYPE + RPM_ANY_RETURN_TYPE },
 	{ "RPMTAG_HEADERREGIONS",	HEADER_REGIONS,	RPM_NULL_TYPE + RPM_ANY_RETURN_TYPE },
 	{ "RPMTAG_HEADERSIGNATURES",	HEADER_SIGNATURES,	RPM_NULL_TYPE + RPM_ANY_RETURN_TYPE },
+	{ "RPMTAG_ICON",	1043,	RPM_BIN_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_INSTALLCOLOR",	1127,	RPM_INT32_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_INSTALLTID",	1128,	RPM_INT32_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_INSTALLTIME",	1008,	RPM_INT32_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_INSTPREFIXES",	1099,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_KEYWORDS",	1182,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_LICENSE",	1014,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_NAME",	1000,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_N",	RPMTAG_NAME,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE  },
+	{ "RPMTAG_OBSOLETEATTRSX",	1190,	RPM_INT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_OBSOLETEFLAGS",	1114,	RPM_INT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_OBSOLETENAME",	1090,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_OBSOLETES",	RPMTAG_OBSOLETENAME,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE  },
@@ -100,12 +107,14 @@ static const struct headerTagTableEntry_s rpmTagTbl[] = {
 	{ "RPMTAG_ORIGDIRNAMES",	1121,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_O",	RPMTAG_OBSOLETENAME,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE  },
 	{ "RPMTAG_OS",	1021,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
+	{ "RPMTAG_PACKAGECOLOR",	1184,	RPM_INT32_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_PACKAGEORIGIN",	1170,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
+	{ "RPMTAG_PACKAGEPREFCOLOR",	1185,	RPM_INT32_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_PACKAGER",	1015,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_PATCH",	1019,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
-	{ "RPMTAG_PATCHESFLAGS",	1134,	RPM_NULL_TYPE + RPM_ANY_RETURN_TYPE },
-	{ "RPMTAG_PATCHESNAME",	1133,	RPM_NULL_TYPE + RPM_ANY_RETURN_TYPE },
-	{ "RPMTAG_PATCHESVERSION",	1135,	RPM_NULL_TYPE + RPM_ANY_RETURN_TYPE },
+	{ "RPMTAG_PATCHESFLAGS",	1134,	RPM_INT32_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_PATCHESNAME",	1133,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_PATCHESVERSION",	1135,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_PAYLOADCOMPRESSOR",	1125,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_PAYLOADFLAGS",	1126,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_PAYLOADFORMAT",	1124,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
@@ -126,6 +135,7 @@ static const struct headerTagTableEntry_s rpmTagTbl[] = {
 	{ "RPMTAG_PREUN",	1025,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_PREUNPROG",	1087,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_PRIORITY",	1162,	RPM_INT32_TYPE + RPM_ARRAY_RETURN_TYPE },
+	{ "RPMTAG_PROVIDEATTRSX",	1191,	RPM_INT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_PROVIDEFLAGS",	1112,	RPM_INT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_PROVIDENAME",	1047,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_PROVIDES",	RPMTAG_PROVIDENAME,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE  },
@@ -135,6 +145,8 @@ static const struct headerTagTableEntry_s rpmTagTbl[] = {
 	{ "RPMTAG_RECONTEXTS",	1149,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_RELEASE",	1002,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_REMOVETID",	1129,	RPM_INT32_TYPE + RPM_SCALAR_RETURN_TYPE },
+	{ "RPMTAG_REPOTAG",	1181,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
+	{ "RPMTAG_REQUIREATTRSX",	1192,	RPM_INT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_REQUIREFLAGS",	1048,	RPM_INT32_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_REQUIRENAME",	1049,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_REQUIRES",	RPMTAG_REQUIRENAME,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE  },
@@ -174,6 +186,7 @@ static const struct headerTagTableEntry_s rpmTagTbl[] = {
 	{ "RPMTAG_VERIFYSCRIPTPROG",	1091,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_VERSION",	1001,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_V",	RPMTAG_VERSION,	RPM_STRING_TYPE + RPM_SCALAR_RETURN_TYPE  },
+	{ "RPMTAG_XATTRSDICT",	1186,	RPM_STRING_ARRAY_TYPE + RPM_ARRAY_RETURN_TYPE },
 	{ "RPMTAG_XMAJOR",	1179,	RPM_INT32_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_XMINOR",	1180,	RPM_INT32_TYPE + RPM_SCALAR_RETURN_TYPE },
 	{ "RPMTAG_XPM",	1013,	RPM_BIN_TYPE + RPM_SCALAR_RETURN_TYPE },
