@@ -89,7 +89,7 @@ urlinfo XurlNew(const char *msg, const char *file, unsigned line)
     u->data = NULL;
     u->bufAlloced = 0;
     u->buf = NULL;
-    u->httpHasRange = 1;
+    u->allow = RPMURL_SERVER_HASRANGE;
     u->httpVersion = 0;
     u->nrefs = 0;
     u->magic = URLMAGIC;
@@ -146,13 +146,7 @@ URLDBGREFS(0, (stderr, "--> url %p -- %d %s at %s:%u\n", u, u->nrefs, msg, file,
 			(u->scheme ? u->scheme : ""));
 	/*@=usereleased@*/
     }
-    if (u->sess != NULL) {
-#ifdef WITH_NEON
-	/* HACK: neon include has prototype. */
-	ne_session_destroy(u->sess);
-#endif
-	u->sess = NULL;
-    }
+    xx = davFree(u);
     u->buf = _free(u->buf);
     u->url = _free(u->url);
     u->scheme = _free((void *)u->scheme);
@@ -557,7 +551,7 @@ int urlGetFile(const char * url, const char * dest)
     if (*sfuPath == '\0')
 	return FTPERR_UNKNOWN;
 	
-    sfd = Fopen(url, "r.ufdio");
+    sfd = Fopen(url, "r");
     if (sfd == NULL || Ferror(sfd)) {
 	rpmMessage(RPMMESS_DEBUG, _("failed to open %s: %s\n"), url, Fstrerror(sfd));
 	rc = FTPERR_UNKNOWN;
@@ -574,11 +568,11 @@ int urlGetFile(const char * url, const char * dest)
     if (dest == NULL)
 	return FTPERR_UNKNOWN;
 
-    tfd = Fopen(dest, "w.ufdio");
+    /* XXX this can fail if directory in path does not exist. */
+    tfd = Fopen(dest, "w");
 if (_url_debug)
 fprintf(stderr, "*** urlGetFile sfd %p %s tfd %p %s\n", sfd, url, (tfd ? tfd : NULL), dest);
     if (tfd == NULL || Ferror(tfd)) {
-	/* XXX Fstrerror */
 	rpmMessage(RPMMESS_DEBUG, _("failed to create %s: %s\n"), dest, Fstrerror(tfd));
 	rc = FTPERR_UNKNOWN;
 	goto exit;
