@@ -16,19 +16,12 @@ case $1 in
     while read filename ; do
     case "${filename}" in
     *.pc)
-	# Assume that this file doesn't contain useful information.
-	needs_pkgconfig=false
 	# Query the dependencies of the package.
 	$pkgconfig --print-provides "$filename" 2> /dev/null | while read n r v ; do
 	    # We have a dependency.  Make a note that we need the pkgconfig
 	    # tool for this package.
 	    echo "pkgconfig($n)" "$r" "$v"
-	    needs_pkgconfig=true
 	done
-	# The dependency on the pkgconfig package itself.
-	if $needs_pkgconfig ; then
-	    echo pkgconfig
-	fi
 	;;
     esac
     done
@@ -38,6 +31,8 @@ case $1 in
     case "${filename}" in
     *.pc)
 	$pkgconfig --print-requires "$filename" 2> /dev/null | while read n r v ; do
+	    i="`expr $i + 1`"
+	    [ $i -eq 1 ] && echo "pkgconfig"
 	    echo "pkgconfig($n)" "$r" "$v"
 	done
     esac
