@@ -186,6 +186,10 @@ char *realpath(const char *path, char resolved_path []);
 #define R_OK 4
 #endif
 
+#ifdef HAVE_SIGNAL_H
+# include <signal.h>
+#endif
+
 #ifdef HAVE_DIRENT_H
 # include <dirent.h>
 # define NLENGTH(direct) (strlen((direct)->d_name))
@@ -291,6 +295,10 @@ extern int _tolower(int) __THROW	/*@*/;
 #include <malloc.h>
 #endif
 
+#if HAVE_LIBGEN_H
+#include <libgen.h>
+#endif
+
 #if WITH_SELINUX
 #include <selinux/selinux.h>
 #else
@@ -469,7 +477,7 @@ extern const char *__progname;
 
 #if ENABLE_NLS && !defined(__LCLINT__)
 # include <libintl.h>
-# define _(Text) gettext (Text)
+# define _(Text) dgettext (PACKAGE, Text)
 #else
 # undef bindtextdomain
 # define bindtextdomain(Domain, Directory) /* empty */
