@@ -1,9 +1,11 @@
 #include "system.h"
 const char *__progname;
 
-#include <rpmbuild.h>
+#include <rpmcli.h>
 #include <argv.h>
 #include <rpmds.h>
+#include <stringbuf.h>
+#define	_RPMFC_INTERNAL		/* XXX for debugging */
 #include <rpmfc.h>
 
 #include "debug.h"
@@ -220,7 +222,7 @@ main(int argc, char *const argv[])
     rpmfc fc = NULL;
     rpmds P = NULL;
     rpmds R = NULL;
-    rpmPRCO PRCO = memset(alloca(sizeof(*PRCO)), 0, sizeof(*PRCO));
+    rpmPRCO PRCO = rpmdsNewPRCO(NULL);
 const char * closure_name = "for";
     FILE * fp = NULL;
     int flags = 0;
@@ -237,9 +239,6 @@ char buf[BUFSIZ];
 	progname = argv[0];
 /*@=modobserver@*/
 
-    PRCO->Pdsp = &P;
-    PRCO->Rdsp = &R;
-
     optCon = rpmcliInit(argc, argv, optionsTable);
     if (optCon == NULL)
 	goto exit;
@@ -248,17 +247,8 @@ char buf[BUFSIZ];
     ac = argvCount(av);
 
     if (rpmdeps_mode == RPMDEP_RPMFC && ac == 0) {
-	char * b, * be;
 	av = NULL;
-if (_rpmfc_debug)
-fprintf(stderr, "*** Reading from stdin ...\n");
-	while ((b = fgets(buf, sizeof(buf), stdin)) != NULL) {
-	    buf[sizeof(buf)-1] = '\0';
-	    be = b + strlen(buf) - 1;
-	    while (strchr("\r\n", *be) != NULL)
-		*be-- = '\0';
-	    xx = argvAdd(&av, b);
-	}
+	xx = argvFgets(&av, NULL);
 	ac = argvCount(av);
     }
 
@@ -267,6 +257,9 @@ fprintf(stderr, "*** Reading from stdin ...\n");
 
     switch (rpmdeps_mode) {
     case RPMDEP_RPMFC:
+	if (ac <= 0)		/* XXX avoid rpmfcApply() segfault */
+	    break;
+
 	/* Build file class dictionary. */
 	fc = rpmfcNew();
 	xx = rpmfcClassify(fc, av, NULL);
@@ -293,7 +286,9 @@ rpmfcPrint(buf, fc, NULL);
 	xx = rpmdsRpmlib(&P, NULL);
 	break;
     case RPMDEP_RPMDSSYSINFO:
-	xx = rpmdsSysinfo(&P, NULL);
+	xx = rpmdsSysinfo(PRCO, NULL);
+	P = rpmdsLink(rpmdsFromPRCO(PRCO, RPMTAG_PROVIDENAME), __FUNCTION__);
+	R = rpmdsLink(rpmdsFromPRCO(PRCO, RPMTAG_REQUIRENAME), __FUNCTION__);
 	break;
     case RPMDEP_RPMDSGETCONF:
 	closure_name = "getconf(...)";
@@ -303,10 +298,14 @@ rpmfcPrint(buf, fc, NULL);
 	closure_name = "soname(...)";
 	for (i = 0; i < ac; i++)
 	    xx = rpmdsELF(av[i], flags, rpmdsMergePRCO, PRCO);
+	P = rpmdsLink(rpmdsFromPRCO(PRCO, RPMTAG_PROVIDENAME), __FUNCTION__);
+	R = rpmdsLink(rpmdsFromPRCO(PRCO, RPMTAG_REQUIRENAME), __FUNCTION__);
 	break;
     case RPMDEP_RPMDSLDCONFIG:
 	closure_name = "soname(...)";
 	xx = rpmdsLdconfig(PRCO, NULL);
+	P = rpmdsLink(rpmdsFromPRCO(PRCO, RPMTAG_PROVIDENAME), __FUNCTION__);
+	R = rpmdsLink(rpmdsFromPRCO(PRCO, RPMTAG_REQUIRENAME), __FUNCTION__);
 	break;
     case RPMDEP_RPMDSUNAME:
 	closure_name = "uname(...)";
@@ -362,8 +361,8 @@ fprintf(stderr, "\n*** Gathering rpmdb file Requires: using\n\t%s\n", _rpmdb_fil
 	break;
     case RPMDEP_RPMDSSONAME:
 	closure_name = "soname(...)";
-	PRCO->Rdsp = NULL;
 	xx = rpmdsLdconfig(PRCO, NULL);
+	P = rpmdsLink(rpmdsFromPRCO(PRCO, RPMTAG_PROVIDENAME), __FUNCTION__);
 if (print_closure || rpmIsVerbose()) {
 fprintf(stderr, "\n*** Gathering rpmdb soname Requires: using\n\t%s\n", _rpmdb_soname_requires);
 	xx = rpmdsPipe(&R, RPMTAG_REQUIRENAME, _rpmdb_soname_requires);
@@ -434,6 +433,7 @@ fprintf(stderr, "\n*** Checking %s Requires(%d): against Provides(%d): closure:\
     fc = rpmfcFree(fc);
     P = rpmdsFree(P);
     R = rpmdsFree(R);
+    PRCO = rpmdsFreePRCO(PRCO);
 
     ec = 0;
 
